@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -578,29 +578,6 @@ load_glyph(gs_fapi_server * a_server, gs_fapi_font * a_fapi_font,
         s->outline_glyph = NULL;
     }
 
-#if 0
-    if (a_fapi_font->is_type1) {
-        FT_Fixed coords[16] = {0};
-        int i;
-        gs_font_type1 *pfont1 = (gs_font_type1 *) a_fapi_font->client_font_data;
-        FT_Multi_Master amaster;
-        T1_Face *face = (T1_Face *)ft_face;
-
-        ft_error = FT_Get_Multi_Master(ft_face, &amaster);
-
-        if (pfont1->data.WeightVector.count > 0) {
-            for (i = 0; i < pfont1->data.WeightVector.count; i++) {
-                coords[i] = (FT_Fixed)(pfont1->data.WeightVector.values[i] * 65536);
-            }
-            coords[0] = coords[1];
-            coords[1] = coords[2];
-            ft_error = FT_Set_MM_Blend_Coordinates (ft_face, pfont1->data.WeightVector.count / 2, coords);
-        }
-        else
-            ft_error = FT_Set_MM_Blend_Coordinates (ft_face, 0, coords);
-    }
-#endif
-
     if (!a_char_ref->is_glyph_index) {
         if (ft_face->num_charmaps)
             index = FT_Get_Char_Index(ft_face, index);
@@ -997,13 +974,19 @@ make_rotation(FT_Matrix * a_transform, const FT_Vector * a_vector)
  */
 static void
 transform_decompose(FT_Matrix * a_transform, FT_UInt * xresp, FT_UInt * yresp,
-                    FT_Fixed * a_x_scale, FT_Fixed * a_y_scale)
+                    FT_Fixed * a_x_scale, FT_Fixed * a_y_scale, int units_per_EM)
 {
     double scalex, scaley, fact = 1.0;
     double factx = 1.0, facty = 1.0;
     FT_Matrix ftscale_mat;
     FT_UInt xres;
     FT_UInt yres;
+    /* We have to account for units_per_EM as we fiddle with the scaling
+     * in order to avoid underflow (mostly in the TTF hinting code), but
+     * we also want to clamp to a lower value (512, admittedly arrived at
+     * via experimentation) in order to preserve the fidelity of the outlines.
+     */
+    double upe = units_per_EM > 512 ? (float)units_per_EM : 512.0;
 
     scalex = hypot((double)a_transform->xx, (double)a_transform->xy);
     scaley = hypot((double)a_transform->yx, (double)a_transform->yy);
@@ -1092,7 +1075,7 @@ transform_decompose(FT_Matrix * a_transform, FT_UInt * xresp, FT_UInt * yresp,
 
     /* see above */
     fact = 1.0;
-    while (scaley * yres > 512.0 * 72.0 && (xres > 0 && yres > 0)
+    while (scaley * yres > (double)upe * 72.0 && (xres > 0 && yres > 0)
            && (scalex > 0.0 && scaley > 0.0)) {
         if (scaley < yres) {
             xres >>= 1;
@@ -1131,7 +1114,7 @@ gs_fapi_ft_get_scaled_font(gs_fapi_server * a_server, gs_fapi_font * a_font,
     ff_server *s = (ff_server *) a_server;
     ff_face *face = (ff_face *) a_font->server_font_data;
     FT_Error ft_error = 0;
-    int i, j;
+    int i, j, code;
     FT_CharMap cmap = NULL;
     bool data_owned = true;
 
@@ -1200,7 +1183,6 @@ gs_fapi_ft_get_scaled_font(gs_fapi_server * a_server, gs_fapi_font * a_font,
         /* Load a typeface from a file. */
         else if (a_font->font_file_path) {
             FT_Open_Args args;
-            int code;
 
             memset(&args, 0x00, sizeof(args));
 
@@ -1232,9 +1214,12 @@ gs_fapi_ft_get_scaled_font(gs_fapi_server * a_server, gs_fapi_font * a_font,
 
             if (a_font->is_type1) {
                 long length;
-                int type =
-                    a_font->get_word(a_font, gs_fapi_font_feature_FontType,
-                                     0);
+                unsigned short type = 0;
+
+                code = a_font->get_word(a_font, gs_fapi_font_feature_FontType, 0, &type);
+                if (code < 0) {
+                    return code;
+                }
 
                 /* Tell the FAPI interface that we need to decrypt the /Subrs data. */
                 a_font->need_decrypt = true;
@@ -1247,6 +1232,7 @@ gs_fapi_ft_get_scaled_font(gs_fapi_server * a_server, gs_fapi_font * a_font,
                     length = gs_fapi_serialize_type1_font(a_font, NULL, 0);
                 else
                     length = gs_fapi_serialize_type2_font(a_font, NULL, 0);
+
                 open_args.memory_base = own_font_data =
                     FF_alloc(s->ftmemory, length);
                 if (!open_args.memory_base)
@@ -1254,14 +1240,14 @@ gs_fapi_ft_get_scaled_font(gs_fapi_server * a_server, gs_fapi_font * a_font,
                 own_font_data_len = length;
                 if (type == 1)
                     open_args.memory_size =
-                        gs_fapi_serialize_type1_font(a_font, own_font_data,
-                                                     length);
+                        gs_fapi_serialize_type1_font(a_font, own_font_data, length);
                 else
                     open_args.memory_size =
-                        gs_fapi_serialize_type2_font(a_font, own_font_data,
-                                                     length);
+                        gs_fapi_serialize_type2_font(a_font, own_font_data, length);
+
                 if (open_args.memory_size != length)
                     return_error(gs_error_unregistered);        /* Must not happen. */
+
                 ft_inc_int = new_inc_int(a_server, a_font);
                 if (!ft_inc_int) {
                     FF_free(s->ftmemory, own_font_data);
@@ -1272,11 +1258,16 @@ gs_fapi_ft_get_scaled_font(gs_fapi_server * a_server, gs_fapi_font * a_font,
             /* It must be type 42 (see code in FAPI_FF_get_glyph in zfapi.c). */
             else {
                 /* Get the length of the TrueType data. */
-                open_args.memory_size =
-                    a_font->get_long(a_font, gs_fapi_font_feature_TT_size, 0);
-                if (open_args.memory_size == 0)
+                unsigned long ms;
+
+                code = a_font->get_long(a_font, gs_fapi_font_feature_TT_size, 0, &ms);
+                if (code < 0)
+                    return code;
+                if (ms == 0)
                     return_error(gs_error_invalidfont);
 
+                open_args.memory_size = (FT_Long)ms;
+
                 /* Load the TrueType data into a single buffer. */
                 open_args.memory_base = own_font_data =
                     FF_alloc(s->ftmemory, open_args.memory_size);
@@ -1285,10 +1276,10 @@ gs_fapi_ft_get_scaled_font(gs_fapi_server * a_server, gs_fapi_font * a_font,
 
                 own_font_data_len = open_args.memory_size;
 
-                if (a_font->
-                    serialize_tt_font(a_font, own_font_data,
-                                      open_args.memory_size))
-                    return_error(gs_error_invalidfont);
+                code = a_font->serialize_tt_font(a_font, own_font_data,
+                                      open_args.memory_size);
+                if (code < 0)
+                    return code;
 
                 /* We always load incrementally. */
                 ft_inc_int = new_inc_int(a_server, a_font);
@@ -1353,7 +1344,7 @@ gs_fapi_ft_get_scaled_font(gs_fapi_server * a_server, gs_fapi_font * a_font,
          * transform.
          */
         transform_decompose(&face->ft_transform, &face->horz_res,
-                            &face->vert_res, &face->width, &face->height);
+                            &face->vert_res, &face->width, &face->height, face->ft_face->units_per_EM);
 
         ft_error = FT_Set_Char_Size(face->ft_face, face->width, face->height,
                                     face->horz_res, face->vert_res);
@@ -1401,6 +1392,21 @@ gs_fapi_ft_get_scaled_font(gs_fapi_server * a_server, gs_fapi_font * a_font,
                  */
                 (void)FT_Select_Charmap(face->ft_face, ft_encoding_unicode);
             }
+            /* For PDF, we have to know which cmap table actually was selected */
+            if (face->ft_face->charmap != NULL) {
+                a_font->ttf_cmap_selected.platform_id = face->ft_face->charmap->platform_id;
+                a_font->ttf_cmap_selected.encoding_id = face->ft_face->charmap->encoding_id;
+            }
+            else {
+                /* Just in case */
+                a_font->ttf_cmap_selected.platform_id = -1;
+                a_font->ttf_cmap_selected.encoding_id = -1;
+            }
+        }
+        else {
+            /* Just in case */
+            a_font->ttf_cmap_selected.platform_id = -1;
+            a_font->ttf_cmap_selected.encoding_id = -1;
         }
     }
 
@@ -1788,12 +1794,37 @@ gs_fapi_ft_check_cmap_for_GID(gs_fapi_server * server, uint * index)
 static gs_fapi_retcode
 gs_fapi_ft_set_mm_weight_vector(gs_fapi_server *server, gs_fapi_font *ff, float *wvector, int length)
 {
-    (void)server;
-    (void)ff;
-    (void)wvector;
-    (void)length;
-    
+#if defined(SHARE_FT) && SHARE_FT == 1 && \
+    FREETYPE_MAJOR <= 2 && FREETYPE_MINOR <= 9 && FREETYPE_PATCH <= 1
+
     return gs_error_invalidaccess;
+#else
+    ff_face *face = (ff_face *) ff->server_font_data;
+    FT_Fixed nwv[16] = {0};
+    FT_Fixed cwv[16] = {0};
+    FT_UInt len = 16;
+    int i;
+    bool setit = false;
+    FT_Error ft_error;
+    (void)server;
+
+    ft_error = FT_Get_MM_WeightVector(face->ft_face, &len, cwv);
+    if (ft_error != 0) return_error(gs_error_invalidaccess);
+
+    for (i = 0; i < length; i++) {
+        nwv[i] = (FT_Fixed)(wvector[i] * 65536.0);
+        if (nwv[i] != cwv[i]) {
+            setit = true;
+        }
+    }
+    
+    if (setit == true) {
+        ft_error = FT_Set_MM_WeightVector(face->ft_face, length, nwv);
+        if (ft_error != 0) return_error(gs_error_invalidaccess);
+    }
+
+    return 0;
+#endif
 }
 
 static void gs_fapi_freetype_destroy(gs_fapi_server ** serv);
