@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -21,6 +21,8 @@
  * repeated inclusion.
  */
 
+#if defined(UNUSED) && defined(FLAG)
+
 UNUSED(0) /* Never use 0, as lots of things 'imply' 0. */
 FLAG(icc, 1, 'c', "ICC profile"),
 FLAG(validate_clumps, 2, 0, "Validate clumps during interpretation"),
@@ -28,8 +30,8 @@ FLAG(gc_disable, 3, 0, "Disable Garbage Collection (completely)"),
 FLAG(epo_disable, 4, 0, "Disable Erasepage Optimization (completely)"),
 FLAG(epo_details, 5, 0, "Erasepage Optimization tracing"),
 FLAG(epo_install_only, 6, 0, "Erasepage Optimization install only (for debugging subclass)"),
-UNUSED(7)
-UNUSED(8)
+FLAG(init_details, 7, 0, "Language initialisation (detail)"),
+FLAG(overprint, 8, 0, "Overprint"),
 UNUSED(9)
 UNUSED(10)
 UNUSED(11)
@@ -148,3 +150,7 @@ UNUSED('{')
 UNUSED('|') /* "Reserved for experimental code" */
 UNUSED('}')
 FLAG(math,              '~', 0,   "Math functions and Functions")
+
+#else
+int dummy;
+#endif
