@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -268,7 +268,7 @@ devn_get_color_comp_index(gx_device * dev, gs_devn_params * pdevn_params,
 {
     int num_order = pdevn_params->num_separation_order_names;
     int color_component_number = 0;
-    int max_spot_colors = GX_DEVICE_MAX_SEPARATIONS - MAX_DEVICE_PROCESS_COLORS;
+    int max_spot_colors = GX_DEVICE_MAX_SEPARATIONS - pdevn_params->num_std_colorant_names;
 
     /*
      * Check if the component is in either the process color model list
@@ -318,15 +318,19 @@ devn_get_color_comp_index(gx_device * dev, gs_devn_params * pdevn_params,
      * Check if we have room for another spot colorant.
      */
     if (auto_spot_colors == ENABLE_AUTO_SPOT_COLORS)
-        max_spot_colors = dev->color_info.max_components -
-            pdevn_params->num_std_colorant_names;
+        /* limit max_spot_colors to what the device can handle given max_components */
+        max_spot_colors = min(max_spot_colors,
+                              dev->color_info.max_components - pdevn_params->num_std_colorant_names);
     if (pdevn_params->separations.num_separations < max_spot_colors) {
         byte * sep_name;
         gs_separations * separations = &pdevn_params->separations;
         int sep_num = separations->num_separations++;
         /* We have a new spot colorant - put in stable memory to avoid "restore" */
         sep_name = gs_alloc_bytes(dev->memory->stable_memory, name_size, "devn_get_color_comp_index");
-
+        if (sep_name == NULL) {
+            separations->num_separations--;	/* we didn't add it */
+            return -1;
+        }
         memcpy(sep_name, pname, name_size);
         separations->names[sep_num].size = name_size;
         separations->names[sep_num].data = sep_name;
@@ -360,14 +364,17 @@ devn_get_params(gx_device * pdev, gs_param_list * plist,
     gs_param_string_array scna;
     gs_param_string_array sona;
     gs_param_int_array equiv_cmyk;
-    int equiv_elements[5 * (GX_DEVICE_MAX_SEPARATIONS - MAX_DEVICE_PROCESS_COLORS)] = { 0 }; /* 5 * max_spot_colors */
+    /* there are 5 ints  per colorant in equiv_elements: a valid flag and an int for C, M, Y and K */
+    int equiv_elements[5 * GX_DEVICE_MAX_SEPARATIONS] = { 0 }; /* 5 * max_colors */
+    /* limit in case num_separations in pdevn_params exceeds what is expected. */
+    int num_separations = min(pdevn_params->separations.num_separations, sizeof(equiv_elements)/(5*sizeof(int)));
 
 
     set_param_array(scna, NULL, 0);
     set_param_array(sona, NULL, 0);
 
     if (pequiv_colors != NULL) {
-        for (spot_num = 0; spot_num < pdevn_params->separations.num_separations; spot_num++) {
+        for (spot_num = 0; spot_num < num_separations; spot_num++) {
             equiv_elements[i++] = pequiv_colors->color[spot_num].color_info_valid ? 1 : 0;
             equiv_elements[i++] = pequiv_colors->color[spot_num].c;
             equiv_elements[i++] = pequiv_colors->color[spot_num].m;
@@ -444,7 +451,7 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
     {
         break;
     } END_ARRAY_PARAM(sona, sone);
-    if (sona.data != 0 && sona.size > GX_DEVICE_COLOR_MAX_COMPONENTS) {
+    if (sona.data != 0 && sona.size > pdev->color_info.max_components) {
         param_signal_error(plist, "SeparationOrder", gs_error_rangecheck);
         return_error(gs_error_rangecheck);
     }
@@ -455,7 +462,7 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
     {
         break;
     } END_ARRAY_PARAM(scna, scne);
-    if (scna.data != 0 && scna.size > GX_DEVICE_MAX_SEPARATIONS) {
+    if (scna.data != 0 && scna.size > pdev->color_info.max_components) {
         param_signal_error(plist, "SeparationColorNames", gs_error_rangecheck);
         return_error(gs_error_rangecheck);
     }
@@ -465,6 +472,10 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
     {
         break;
     } END_ARRAY_PARAM(equiv_cmyk, equiv_cmyk_e);
+    if (equiv_cmyk.data != 0 && scna.size > pdev->color_info.max_components) {
+        param_signal_error(plist, "SeparationColorNames", gs_error_rangecheck);
+        return_error(gs_error_rangecheck);
+    }
 
     /* Separations are only valid with a subrtractive color model */
     if (pdev->color_info.polarity == GX_CINFO_POLARITY_SUBTRACTIVE) {
@@ -486,6 +497,10 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
                     /* We have a new separation */
                     sep_name = (byte *)gs_alloc_bytes(pdev->memory,
                         name_size, "devicen_put_params_no_sep_order");
+                    if (sep_name == NULL) {
+                        param_signal_error(plist, "SeparationColorNames", gs_error_VMerror);
+                        return_error(gs_error_VMerror);
+                    }
                     memcpy(sep_name, scna.data[i].data, name_size);
                     pdevn_params->separations.names[num_spot].size = name_size;
                     pdevn_params->separations.names[num_spot].data = sep_name;
@@ -495,9 +510,10 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
                         pequiv_colors->all_color_info_valid = false;
                     }
                     num_spot++;
+                    num_spot_changed = true;
                 }
             }
-            num_spot_changed = true;
+
             for (i = pdevn_params->separations.num_separations; i < num_spot; i++)
                 pdevn_params->separation_order_map[i + pdevn_params->num_std_colorant_names] =
                 i + pdevn_params->num_std_colorant_names;
@@ -591,8 +607,8 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
                     param_signal_error(plist, "PageSpotColors", gs_error_rangecheck);
                     return_error(gs_error_rangecheck);
                 }
-                if (page_spot_colors > GX_DEVICE_COLOR_MAX_COMPONENTS - MAX_DEVICE_PROCESS_COLORS)
-                    page_spot_colors = GX_DEVICE_COLOR_MAX_COMPONENTS - MAX_DEVICE_PROCESS_COLORS;
+                if (page_spot_colors > pdev->color_info.max_components - pdevn_params->num_std_colorant_names)
+                    page_spot_colors = pdev->color_info.max_components - pdevn_params->num_std_colorant_names;
                     /* Need to leave room for the process colors in GX_DEVICE_COLOR_MAX_COMPONENTS  */
         }
         /*
@@ -700,6 +716,9 @@ devn_copy_params(gx_device * psrcdev, gx_device * pdesdev)
         int name_size = src_devn_params->separations.names[k].size;
         sep_name = (byte *)gs_alloc_bytes(pdesdev->memory->stable_memory,
                                           name_size, "devn_copy_params");
+        if (sep_name == NULL) {
+            return_error(gs_error_VMerror);
+        }
         memcpy(sep_name, src_devn_params->separations.names[k].data, name_size);
         des_devn_params->separations.names[k].size = name_size;
         des_devn_params->separations.names[k].data = sep_name;
@@ -716,6 +735,9 @@ devn_copy_params(gx_device * psrcdev, gx_device * pdesdev)
         int name_size = src_devn_params->pdf14_separations.names[k].size;
         sep_name = (byte *)gs_alloc_bytes(pdesdev->memory->stable_memory,
                                           name_size, "devn_copy_params");
+        if (sep_name == NULL) {
+            return_error(gs_error_VMerror);
+        }
         memcpy(sep_name, src_devn_params->pdf14_separations.names[k].data,
                name_size);
         des_devn_params->pdf14_separations.names[k].size = name_size;
@@ -1054,8 +1076,8 @@ const gx_devn_prn_device gs_devicen_device =
     gx_devn_prn_device_body(devicen_procs, "devicen", 4, GX_CINFO_POLARITY_SUBTRACTIVE, 32, 255, 255, "DeviceCMYK"),
     /* DeviceN device specific parameters */
     { 8,                        /* Bits per color - must match ncomp, depth, etc. above */
-      NULL,                     /* No names for standard DeviceN color model */
-      0,                        /* No standard colorants for DeviceN */
+      DeviceCMYKComponents,     /* Names of color model colorants */
+      4,                        /* Number colorants for CMYK */
       0,                        /* MaxSeparations has not been specified */
       -1,                       /* PageSpotColors has not been specified */
       {0},                      /* SeparationNames */
@@ -1382,7 +1404,11 @@ spotcmyk_print_page(gx_device_printer * pdev, gp_file * prn_stream)
     /* Open the output files for the spot colors */
     for(i = 0; i < nspot; i++) {
         gs_sprintf(spotname, "%ss%d", pdevn->fname, i);
+        code = gs_add_control_path(pdev->memory, gs_permit_file_writing, spotname);
+        if (code < 0)
+            goto prn_done;
         spot_file[i] = gp_fopen(pdev->memory, spotname, "wb");
+        (void)gs_remove_control_path(pdev->memory, gs_permit_file_writing, spotname);
         if (spot_file[i] == NULL) {
             code = gs_note_error(gs_error_VMerror);
             goto prn_done;
@@ -1391,7 +1417,9 @@ spotcmyk_print_page(gx_device_printer * pdev, gp_file * prn_stream)
 
     /* Now create the output bit image files */
     for (; lnum < bottom; ++lnum) {
-        gdev_prn_get_bits(pdev, lnum, in, &data);
+        code = gdev_prn_get_bits(pdev, lnum, in, &data);
+        if (code < 0)
+            goto prn_done;
         /* Now put the pcm data into the output file */
         if (npcmcolors) {
             first_bit = bpc * (ncomp - npcmcolors);
@@ -1755,27 +1783,39 @@ devn_write_pcx_file(gx_device_printer * pdev, char * filename, int ncomp,
 
     if (outname == NULL) {
         code = gs_note_error(gs_error_VMerror);
-	goto done;
+        goto done;
     }
 
+    code = gs_add_control_path(pdev->memory, gs_permit_file_reading, filename);
+    if (code < 0)
+        goto done;
+
     in = gp_fopen(pdev->memory, filename, "rb");
     if (!in) {
         code = gs_note_error(gs_error_invalidfileaccess);
-	goto done;
+        goto done;
     }
     gs_sprintf(outname, "%s.pcx", filename);
+    code = gs_add_control_path(pdev->memory, gs_permit_file_writing, outname);
+    if (code < 0)
+        goto done;
     out = gp_fopen(pdev->memory, outname, "wb");
     if (!out) {
         code = gs_note_error(gs_error_invalidfileaccess);
-	goto done;
+        goto done;
     }
 
+    if (ncomp == 4 && bpc == 8) {
+        ncomp = 3;		/* we will convert 32-bit to 24-bit RGB */
+    }
     planar = devn_setup_pcx_header(pdev, &header, ncomp, bpc);
     code = devn_pcx_write_page(pdev, in, linesize, out, &header, planar, depth);
     if (code >= 0)
         code = devn_finish_pcx_file(pdev, out, &header, ncomp, bpc);
 
 done:
+    (void)gs_remove_control_path(pdev->memory, gs_permit_file_reading, filename);
+    (void)gs_remove_control_path(pdev->memory, gs_permit_file_writing, outname);
     if (in)
       gp_fclose(in);
     if (out)
@@ -1799,12 +1839,22 @@ devn_pcx_write_page(gx_device_printer * pdev, gp_file * infile, int linesize, gp
     int height = pdev->height;
     uint lsize = raster + rsize;
     byte *line = gs_alloc_bytes(pdev->memory, lsize, "pcx file buffer");
+    byte *rgb_buff = NULL;
     byte *plane = line + raster;
+    bool convert_to_rgb = false;
     int y;
     int code = 0;               /* return code */
 
     if (line == 0)              /* can't allocate line buffer */
         return_error(gs_error_VMerror);
+    if (pdev->color_info.num_components == 4 && depth == 32) {
+        rgb_buff = gs_alloc_bytes(pdev->memory, lsize, "pcx_rgb_buff");
+        if (rgb_buff == 0)              /* can't allocate line buffer */
+            return_error(gs_error_VMerror);
+        raster = (raster * 3) / 4;	/* will be rounded up to even later */
+        depth = 24;			/* we will be writing 24-bit rgb */
+        convert_to_rgb = true;
+    }
 
     /* Fill in the other variable entries in the header struct. */
 
@@ -1829,6 +1879,21 @@ devn_pcx_write_page(gx_device_printer * pdev, gp_file * infile, int linesize, gp
         code = gp_fread(line, sizeof(byte), linesize, infile);
         if (code < 0)
             break;
+        /* If needed, convert to rgb */
+        if (convert_to_rgb) {
+            int i;
+			byte *row_in = line;
+
+            /* Transform the data. */
+            row = rgb_buff;	/* adjust to converted output buffer */
+            for (i=0; i < linesize; i += 4) {
+                *row++ = ((255 - row_in[0]) * (255 - row_in[3])) / 255;
+                *row++ = ((255 - row_in[1]) * (255 - row_in[3])) / 255;
+                *row++ = ((255 - row_in[2]) * (255 - row_in[3])) / 255;
+                row_in += 4;
+            }
+            row = rgb_buff;	/* adjust to converted output buffer */
+        }
         end = row + raster;
         if (!planar) {          /* Just write the bits. */
             if (raster & 1) {   /* Round to even, with predictable padding. */
@@ -1891,6 +1956,8 @@ devn_pcx_write_page(gx_device_printer * pdev, gp_file * infile, int linesize, gp
     }
 
   pcx_done:
+    if (rgb_buff != NULL)
+        gs_free_object(pdev->memory, rgb_buff, "pcx_rgb_buff");
     gs_free_object(pdev->memory, line, "pcx file buffer");
 
     return code;
