@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -18,23 +18,12 @@
 #ifndef gdevdevn_INCLUDED
 # define gdevdevn_INCLUDED
 
+#include "gxblend.h"
+#include "gsequivc.h"
+
 /* See Comments in gdevtsep.c or gdevpsd.c as to the purpose of this */
 #define LIMIT_TO_ICC 1
 
-/*
- * Define the maximum number of spot colors supported by this device.
- * This value is arbitrary.  It is set simply to define a limit on
- * on the separation_name_array and separation_order map.
- */
-#define GX_DEVICE_MAX_SEPARATIONS GX_DEVICE_COLOR_MAX_COMPONENTS
-/*
- * Define the maximum number of process model colorants.  Currently we only
- * have code for DeviceGray, DeviceRGB, and DeviceCMYK.  Thus this value
- * only needs to be 4.  However we are allowing for a future hexachrome
- * device.  (This value does not include spot colors.  See previous value.)
- */
-#define MAX_DEVICE_PROCESS_COLORS 6
-
 /*
  * Type definitions associated with the fixed color model names.
  */
@@ -57,15 +46,10 @@ struct gs_separations_s {
     devn_separation_name names[GX_DEVICE_MAX_SEPARATIONS];
 };
 
-#ifndef gs_separations_DEFINED
-#   define gs_separations_DEFINED
-    typedef struct gs_separations_s gs_separations;
-#endif
-
 /*
  * Type for holding a separation order map
  */
-typedef int gs_separation_map[GX_DEVICE_MAX_SEPARATIONS + MAX_DEVICE_PROCESS_COLORS];
+typedef int gs_separation_map[GX_DEVICE_MAX_SEPARATIONS];
 
 typedef struct gs_devn_params_s {
     /*
@@ -113,15 +97,10 @@ typedef struct gs_devn_params_s {
     gs_separations pdf14_separations;
 } gs_devn_params_t;
 
-#ifndef gs_devn_params_DEFINED
-#  define gs_devn_params_DEFINED
-typedef struct gs_devn_params_s gs_devn_params;
-#endif
-
+extern fixed_colorant_name DeviceGrayComponents[];
+extern fixed_colorant_name DeviceRGBComponents[];
 extern fixed_colorant_name DeviceCMYKComponents[];
 
-#include "gsequivc.h"
-
 /*
  * Utility routines for common DeviceN related parameters:
  *   SeparationColorNames, SeparationOrder, and MaxSeparations
