@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -767,6 +767,7 @@ gx_device_fill_in_procs(register gx_device * dev)
     fill_dev_proc(dev, copy_planes, gx_default_copy_planes);
     fill_dev_proc(dev, process_page, gx_default_process_page);
     fill_dev_proc(dev, transform_pixel_region, gx_default_transform_pixel_region);
+    fill_dev_proc(dev, fill_stroke_path, gx_default_fill_stroke_path);
 }
 
 
@@ -1025,6 +1026,8 @@ gx_default_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
         case gxdso_supports_saved_pages:
         case gxdso_needs_invariant_palette:
         case gxdso_supports_iccpostrender:
+        case gxdso_supports_alpha:
+        case gxdso_pdf14_sep_device:
             return 0;
         case gxdso_pattern_shfill_doesnt_need_path:
             return (dev_proc(pdev, fill_path) == gx_default_fill_path);
@@ -1062,6 +1065,8 @@ gx_default_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
         /* Just ignore information about events */
         case gxdso_event_info:
             return 0;
+        case gxdso_overprint_active:
+            return 0;
     }
     return_error(gs_error_undefined);
 }
@@ -1315,6 +1320,7 @@ int gx_copy_device_procs(gx_device *dest, gx_device *src, gx_device *prototype)
     set_dev_proc(dest, strip_tile_rect_devn, dev_proc(prototype, strip_tile_rect_devn));
     set_dev_proc(dest, process_page, dev_proc(prototype, process_page));
     set_dev_proc(dest, transform_pixel_region, dev_proc(prototype, transform_pixel_region));
+    set_dev_proc(dest, fill_stroke_path, dev_proc(prototype, fill_stroke_path));
 
     /*
      * We absolutely must set the 'set_graphics_type_tag' to the default subclass one
@@ -1869,13 +1875,19 @@ transform_pixel_region_render_portrait(gx_device *dev, gx_default_transform_pixe
         byte *out;
         int depth = spp;
         if (state->line == NULL) {
-            state->line = gs_alloc_bytes(state->mem, dev->width * depth,
+            state->line = gs_alloc_bytes(state->mem,
+                                         (size_t)dev->width * depth,
                                          "image line");
             if (state->line == NULL)
                 return gs_error_VMerror;
         }
         out = state->line;
 
+        if (minx < 0)
+            minx = 0;
+        if (maxx > dev->width)
+            maxx = dev->width;
+
         if (pending_left < minx)
             pending_left = minx;
         else if (pending_left > maxx)
@@ -1960,7 +1972,7 @@ transform_pixel_region_render_portrait(gx_device *dev, gx_default_transform_pixe
                 goto err;
         }
     }
-    return (code < 0 ? code : 1);
+    return 1;
     /* Save position if error, in case we resume. */
 err:
     buffer[0] = run;
@@ -2050,7 +2062,7 @@ transform_pixel_region_render_landscape(gx_device *dev, gx_default_transform_pix
             goto err;
         data = run;
     }
-    return (code < 0 ? code : 1);
+    return 1;
     /* Save position if error, in case we resume. */
 err:
     buffer[0] = run;
@@ -2115,10 +2127,11 @@ transform_pixel_region_render_skew(gx_device *dev, gx_default_transform_pixel_re
         prev = data;
         data += spp;
     }
-    return (code < 0 ? code : 1);
+    return 1;
     /* Save position if error, in case we resume. */
 err:
-    buffer[0] = prev;
+    /* Only set buffer[0] if we've managed to set prev to something valid. */
+    if (prev != &initial_run[0]) buffer[0] = prev;
     return code;
 }
 
