@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -117,7 +117,9 @@ gx_device_forward_fill_in_procs(register gx_device_forward * dev)
     fill_dev_proc(dev, set_graphics_type_tag, gx_forward_set_graphics_type_tag);
     fill_dev_proc(dev, strip_copy_rop2, gx_forward_strip_copy_rop2);
     fill_dev_proc(dev, strip_tile_rect_devn, gx_forward_strip_tile_rect_devn);
+    fill_dev_proc(dev, strip_tile_rect_devn, gx_forward_strip_tile_rect_devn);
     fill_dev_proc(dev, transform_pixel_region, gx_forward_transform_pixel_region);
+    fill_dev_proc(dev, fill_stroke_path, gx_forward_fill_stroke_path);
     gx_device_fill_in_procs((gx_device *) dev);
 }
 
@@ -449,6 +451,24 @@ gx_forward_stroke_path(gx_device * dev, const gs_gstate * pgs,
     return proc(tdev, pgs, ppath, params, pdcolor, pcpath);
 }
 
+int
+gx_forward_fill_stroke_path(gx_device * dev, const gs_gstate * pgs,
+                            gx_path * ppath,
+                            const gx_fill_params * params_fill,
+                            const gx_drawing_color * pdcolor_fill,
+                            const gx_stroke_params * params_stroke,
+                            const gx_drawing_color * pdcolor_stroke,
+                            const gx_clip_path * pcpath)
+{
+    gx_device_forward * const fdev = (gx_device_forward *)dev;
+    gx_device *tdev = fdev->target;
+    dev_proc_fill_stroke_path((*proc)) =
+        (tdev == 0 ? (tdev = dev, gx_default_fill_stroke_path) :
+         dev_proc(tdev, fill_stroke_path));
+
+    return proc(tdev, pgs, ppath, params_fill, pdcolor_fill, params_stroke, pdcolor_stroke, pcpath);
+}
+
 int
 gx_forward_fill_mask(gx_device * dev,
                      const byte * data, int dx, int raster, gx_bitmap_id id,
@@ -1064,6 +1084,7 @@ static dev_proc_strip_copy_rop(null_strip_copy_rop);
 static dev_proc_strip_copy_rop2(null_strip_copy_rop2);
 static dev_proc_strip_tile_rect_devn(null_strip_tile_rect_devn);
 static dev_proc_fill_rectangle_hl_color(null_fill_rectangle_hl_color);
+static dev_proc_dev_spec_op(null_spec_op);
 
 #define null_procs(get_initial_matrix, get_page_device) {\
         gx_default_open_device,\
@@ -1131,7 +1152,7 @@ static dev_proc_fill_rectangle_hl_color(null_fill_rectangle_hl_color);
         NULL, /* push_transparency_state */\
         NULL, /* pop_transparency_state */\
         NULL, /* put_image */\
-        gx_default_dev_spec_op, /* dev_spec_op */\
+        null_spec_op, /* dev_spec_op */\
         NULL, /* copy_planes */\
         NULL, /* get_profile */\
         NULL, /* set_graphics_type_tag */\
@@ -1202,11 +1223,19 @@ null_copy_color(gx_device * dev, const byte * data,
 static int
 null_put_params(gx_device * dev, gs_param_list * plist)
 {
+    int code;
+    cmm_dev_profile_t *iccs = dev->icc_struct;
+
     /*
      * If this is not a page device, we must defeat attempts to reset
      * the size; otherwise this is equivalent to gx_forward_put_params.
+     * Equally, we don't want it to unexpectectly error out on certain
+     * ICC parameters - so defeat those, too.
      */
-    int code = gx_forward_put_params(dev, plist);
+    dev->icc_struct = NULL;
+    code = gx_forward_put_params(dev, plist);
+    rc_decrement(dev->icc_struct, "null_put_params");
+    dev->icc_struct = iccs;
 
     if (code < 0 || dev_proc(dev, get_page_device)(dev) == dev)
         return code;
@@ -1326,6 +1355,18 @@ static int null_fill_rectangle_hl_color(gx_device *pdev,
     return 0;
 }
 
+static int
+null_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
+{
+    /* Defeat the ICC profile components check, which we want to do since
+       we also short-circuit ICC device parameters - see null_put_params.
+     */
+    if (dev_spec_op == gxdso_skip_icc_component_validation) {
+        return 1;
+    }
+    return gx_default_dev_spec_op(pdev, dev_spec_op, data, size);
+}
+
 bool
 fwd_uses_fwd_cmap_procs(gx_device * dev)
 {
@@ -1365,7 +1406,7 @@ static void do_device_dump(gx_device *dev, int n)
         dmlprintf(dev->memory, "NULL\n");
         return;
     }
-    dmlprintf3(dev->memory, "%p(%ld) = '%s'\n", dev, dev->rc.ref_count, dev->dname);
+    dmlprintf3(dev->memory, PRI_INTPTR"(%ld) = '%s'\n", (intptr_t)dev, dev->rc.ref_count, dev->dname);
 
     data.n = 0;
     do {
