@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -46,23 +46,11 @@
 #define PRN_MIN_MEMORY_LEFT MIN_MEMORY_LEFT	/* see gxdevice.h */
 #define PRN_MIN_BUFFER_SPACE MIN_BUFFER_SPACE	/* see gxdevice.h */
 
-/* Define the abstract type for a printer device. */
-#ifndef gx_device_printer_DEFINED
-#  define gx_device_printer_DEFINED
-typedef struct gx_device_printer_s gx_device_printer;
-#endif
-
 /* Define the abstract type for parameters describing buffer space. */
-#ifndef gdev_prn_space_params_DEFINED
-#  define gdev_prn_space_params_DEFINED
 typedef struct gdev_space_params_s gdev_prn_space_params;
-#endif
 
 /* Define the abstract type for parameters describing buffer space. */
-#ifndef gdev_prn_banding_type_DEFINED
-#  define gdev_prn_banding_type_DEFINED
 typedef struct gdev_banding_type gdev_prn_banding_type;
-#endif
 
 /* Define the abstract type for some band device procedures' arguments. */
 typedef struct gdev_prn_start_render_params_s gdev_prn_start_render_params;
@@ -79,7 +67,7 @@ typedef struct gx_printer_device_procs_s {
      */
 
 #define prn_dev_proc_print_page(proc)\
-  int proc(gx_device_printer *, FILE *)
+  int proc(gx_device_printer *, gp_file *)
     prn_dev_proc_print_page((*print_page));
 /* BACKWARD COMPATIBILITY */
 #define dev_proc_print_page(proc) prn_dev_proc_print_page(proc)
@@ -87,7 +75,7 @@ typedef struct gx_printer_device_procs_s {
     /* Print the page on the output file, with a given # of copies. */
 
 #define prn_dev_proc_print_page_copies(proc)\
-  int proc(gx_device_printer *, FILE *, int)
+  int proc(gx_device_printer *, gp_file *, int)
     prn_dev_proc_print_page_copies((*print_page_copies));
 /* BACKWARD COMPATIBILITY */
 #define dev_proc_print_page_copies(proc) prn_dev_proc_print_page_copies(proc)
@@ -111,7 +99,7 @@ typedef struct gx_printer_device_procs_s {
      */
 
 #define prn_dev_proc_get_space_params(proc)\
-  void proc(const gx_device_printer *, gdev_prn_space_params *)
+  void proc(const gx_device_printer *, gdev_space_params *)
     prn_dev_proc_get_space_params((*get_space_params));
 
 } gx_printer_device_procs;
@@ -139,8 +127,7 @@ typedef struct bg_print_s {
 } bg_print_t;
 
 #define gx_prn_device_common\
-        byte skip[max(sizeof(gx_device_memory), sizeof(gx_device_clist)) -\
-                  sizeof(gx_device) + sizeof(double) /* padding */];\
+        gx_device_clist_mutatable_common;\
         gx_printer_device_procs printer_procs;\
                 /* ------ Device parameters that must be set ------ */\
                 /* ------ before calling the device open routine. ------ */\
@@ -152,19 +139,12 @@ typedef struct bg_print_s {
         int Duplex_set;		        /* -1 = not supported */\
                 /* ------ End of parameters ------ */\
         bool file_is_new;		/* true iff file just opened */\
-        FILE *file;			/* output file */\
-        long buffer_space;	        /* amount of space for clist buffer, */\
-                                        /* 0 means not using clist */\
-        byte *buf;			/* buffer for rendering */\
-        gs_memory_t *buffer_memory;	/* allocator for command list */\
-        gs_memory_t *bandlist_memory;	/* allocator for bandlist files */\
-        uint clist_disable_mask;	/* mask of clist options to disable */\
+        gp_file *file;  		/* output file */\
         bool bg_print_requested;	/* request background printing of page from clist */\
         bg_print_t bg_print;            /* background printing data shared with thread */\
         int num_render_threads_requested;	/* for multiple band rendering threads */\
         gx_saved_pages_list *saved_pages_list;	/* list when we are saving pages instead of printing */\
-        gx_device_procs save_procs_while_delaying_erasepage;	/* save device procs while delaying erasepage. */\
-        gx_device_procs orig_procs	/* original (std_)procs */
+        gx_device_procs save_procs_while_delaying_erasepage	/* save device procs while delaying erasepage. */
 
 /* The device descriptor */
 struct gx_device_printer_s {
@@ -173,7 +153,7 @@ struct gx_device_printer_s {
 };
 
 /* A useful check to determine if the page is being rendered as a clist */
-#define PRINTER_IS_CLIST(pdev) ((gx_device_printer *)(pdev)->buffer_space != 0)
+#define PRINTER_IS_CLIST(pdev) (((gx_device_printer *)(pdev))->buffer_space != 0)
 
 extern_st(st_device_printer);
 #define public_st_device_printer()	/* in gdevprn.c */\
@@ -312,7 +292,7 @@ extern const gx_device_procs prn_bg_procs;
  */
 #define prn_device_body_rest2_(print_page, print_page_copies, duplex_set)\
          { 0 },		/* std_procs */\
-         { 0 },		/* skip */\
+         GX_CLIST_MUTATABLE_DEVICE_DEFAULTS,\
          { print_page,\
            print_page_copies,\
            { gx_default_create_buf_device,\
@@ -328,17 +308,11 @@ extern const gx_device_procs prn_bg_procs;
         0/*false*/, duplex_set,	/* Duplex[_set] */\
         0/*false*/,	/* file_is_new */\
         0,	        /* *file */\
-        0,		/* buffer_space */\
-        0,		/* *buf */\
-        0,		/* *buffer_memory */\
-        0,		/* *bandlist_memory */\
-        0,		/* clist_disable_mask */\
         0/*false*/,	/* bg_print_requested */\
         {  0/*sema*/, 0/*device*/, 0/*thread_id*/, 0/*num_copies*/, 0/*return_code*/ }, /* bg_print */\
         0, 		/* num_render_threads_requested */\
         0,              /* saved_pages_list */\
-        { 0 },	/* save_procs_while_delaying_erasepage */\
-        { 0 }	/* ... orig_procs */
+        { 0 }           /* save_procs_while_delaying_erasepage */
 #define prn_device_body_rest_(print_page)\
   prn_device_body_rest2_(print_page, gx_default_print_page_copies, -1)
 #define prn_device_body_copies_rest_(print_page_copies)\
@@ -592,10 +566,10 @@ int gdev_prn_print_scan_lines(gx_device *);
 
 /* Allocate / reallocate / free printer memory. */
 int gdev_prn_allocate_memory(gx_device *pdev,
-                             gdev_prn_space_params *space,
+                             gdev_space_params *space,
                              int new_width, int new_height);
 int gdev_prn_reallocate_memory(gx_device *pdev,
-                               gdev_prn_space_params *space,
+                               gdev_space_params *space,
                                int new_width, int new_height);
 int gdev_prn_free_memory(gx_device *pdev);
 
