@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -630,6 +630,11 @@ gdev_vector_dopath_init(gdev_vector_dopath_state_t *state,
         gs_make_scaling(vdev->scale.x, vdev->scale.y, &state->scale_mat);
     }
     state->first = true;
+
+    /* This is purely to prevent Coverity from thinking gdev_vector_dopath()
+    could use uninitialised state->start.x. */
+    state->start.x = 0;
+    state->start.y = 0;
 }
 
 /*
@@ -826,7 +831,7 @@ gdev_vector_update_clip_path(gx_device_vector * vdev,
 int
 gdev_vector_close_file(gx_device_vector * vdev)
 {
-    FILE *f = vdev->file;
+    gp_file *f = vdev->file;
     int err;
 
     if (vdev->dash_pattern) {
@@ -850,7 +855,7 @@ gdev_vector_close_file(gx_device_vector * vdev)
     }
     vdev->file = 0;
     if (f) {
-        err = ferror(f);
+        err = gp_ferror(f);
         /* We prevented sclose from closing the file. */
         if (gx_device_close_output_file((gx_device *)vdev, vdev->fname, f) != 0
                 || err != 0)
