@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -401,16 +401,16 @@ generic_pread(gp_file *f, size_t count, gs_offset_t offset, void *buf)
     int c;
     int64_t os, curroff = gp_ftell(f);
     if (curroff < 0) return curroff;
-    
+
     os = gp_fseek(f, offset, 0);
     if (os < 0) return os;
-    
+
     c = gp_fread(buf, 1, count, f);
     if (c < 0) return c;
-    
+
     os = gp_fseek(f, curroff, 0);
     if (os < 0) return os;
-    
+
     return c;
 }
 
@@ -420,22 +420,22 @@ generic_pwrite(gp_file *f, size_t count, gs_offset_t offset, const void *buf)
     int c;
     int64_t os, curroff = gp_ftell(f);
     if (curroff < 0) return curroff;
-    
+
     os = gp_fseek(f, offset, 0);
     if (os < 0) return os;
-    
+
     c = gp_fwrite(buf, 1, count, f);
     if (c < 0) return c;
-    
+
     os = gp_fseek(f, curroff, 0);
     if (os < 0) return os;
-    
+
     return c;
 }
 
-gp_file *gp_file_alloc(gs_memory_t *mem, const gp_file_ops_t *prototype, size_t size, const char *cname)
+gp_file *gp_file_alloc(const gs_memory_t *mem, const gp_file_ops_t *prototype, size_t size, const char *cname)
 {
-    gp_file *file = (gp_file *)gs_alloc_bytes(mem->non_gc_memory, size, cname ? cname : "gp_file");
+    gp_file *file = (gp_file *)gs_alloc_bytes(mem->thread_safe_memory, size, cname ? cname : "gp_file");
     if (file == NULL)
         return NULL;
 
@@ -449,7 +449,7 @@ gp_file *gp_file_alloc(gs_memory_t *mem, const gp_file_ops_t *prototype, size_t
         memset(((char *)file)+sizeof(*prototype),
                0,
                size - sizeof(*prototype));
-    file->memory = mem->non_gc_memory;
+    file->memory = mem->thread_safe_memory;
 
     return file;
 }
@@ -743,7 +743,7 @@ gp_open_printer(const gs_memory_t *mem,
                       char         fname[gp_file_name_sizeof],
                       int          binary_mode)
 {
-    gp_file *file;
+    gp_file *file = NULL;
     gs_lib_ctx_t *ctx = mem->gs_lib_ctx;
     gs_fs_list_t *fs = ctx->core->fs;
 
@@ -764,15 +764,17 @@ gp_open_printer(const gs_memory_t *mem,
     return file;
 }
 
-gp_file *
-gp_open_scratch_file(const gs_memory_t *mem,
+static gp_file *
+do_open_scratch_file(const gs_memory_t *mem,
                      const char        *prefix,
                      char              *fname,
-                     const char        *mode)
+                     const char        *mode,
+                     int                rm)
 {
-    gp_file *file;
+    gp_file *file = NULL;
     gs_lib_ctx_t *ctx = mem->gs_lib_ctx;
     gs_fs_list_t *fs = ctx->core->fs;
+    int code = 0;
 
     /* If the prefix is absolute, then we must check it's a permissible
      * path. If not, we're OK. */
@@ -782,46 +784,74 @@ gp_open_scratch_file(const gs_memory_t *mem,
 
     for (fs = ctx->core->fs; fs != NULL; fs = fs->next)
     {
-        int code = 0;
         if (fs->fs.open_scratch)
-            code = fs->fs.open_scratch(mem, fs->secret, prefix, fname, mode, 0, &file);
+            code = fs->fs.open_scratch(mem, fs->secret, prefix, fname, mode, rm, &file);
         if (code < 0)
             return NULL;
         if (file != NULL)
             break;
     }
 
+    if (file == NULL) {
+        /* The file failed to open. Don't add it to the list. */
+    } else if (rm) {
+        /* This file has already been deleted by the underlying system.
+         * We don't need to add it to the lists as it will never be
+         * deleted manually, nor do we need to tidy it up on closedown. */
+    } else {
+         /* This file was not requested to be deleted. We add it to the
+          * list so that it will either be deleted by any future call to
+          * zdeletefile, OR on closedown. */
+         /* Add the scratch file name to the lists. We can't do this any
+          * earlier as we didn't know the name until now! Unfortunately
+          * that makes cleanup harder. */
+        code = gs_add_control_path_flags(mem, gs_permit_file_control, fname,
+                                         gs_path_control_flag_is_scratch_file);
+        if (code >= 0)
+            code = gs_add_control_path_flags(mem, gs_permit_file_reading, fname,
+                                             gs_path_control_flag_is_scratch_file);
+        if (code >= 0)
+            code = gs_add_control_path_flags(mem, gs_permit_file_writing, fname,
+                                         gs_path_control_flag_is_scratch_file);
+
+        if (code < 0) {
+            gp_fclose(file);
+            file = NULL;
+            /* Call directly through to the unlink implementation. We know
+             * we're 'permitted' to do this, but we might not be on all the
+             * required permit lists because of the failure. The only bad
+             * thing here, is that we're deleting an fname that might not
+             * have come from the filing system itself. */
+            if (fname && fname[0])
+                gp_unlink_impl(ctx->memory, fname);
+            (void)gs_remove_control_path_flags(mem, gs_permit_file_control, fname,
+                                               gs_path_control_flag_is_scratch_file);
+            (void)gs_remove_control_path_flags(mem, gs_permit_file_reading, fname,
+                                               gs_path_control_flag_is_scratch_file);
+            (void)gs_remove_control_path_flags(mem, gs_permit_file_writing, fname,
+                                               gs_path_control_flag_is_scratch_file);
+        }
+    }
+
     return file;
 }
 
+gp_file *
+gp_open_scratch_file(const gs_memory_t *mem,
+                     const char        *prefix,
+                     char              *fname,
+                     const char        *mode)
+{
+    return do_open_scratch_file(mem, prefix, fname, mode, 0);
+}
+
 gp_file *
 gp_open_scratch_file_rm(const gs_memory_t *mem,
                         const char        *prefix,
                         char              *fname,
                         const char        *mode)
 {
-    gp_file *file;
-    gs_lib_ctx_t *ctx = mem->gs_lib_ctx;
-    gs_fs_list_t *fs = ctx->core->fs;
-
-    /* If the prefix is absolute, then we must check it's a permissible
-     * path. If not, we're OK. */
-    if (gp_file_name_is_absolute(prefix, strlen(prefix)) &&
-        gp_validate_path(mem, prefix, mode) != 0)
-            return NULL;
-
-    for (fs = ctx->core->fs; fs != NULL; fs = fs->next)
-    {
-        int code = 0;
-        if (fs->fs.open_scratch)
-            code = fs->fs.open_scratch(mem, fs->secret, prefix, fname, mode, 1, &file);
-        if (code < 0)
-            return NULL;
-        if (file != NULL)
-            break;
-    }
-
-    return file;
+    return do_open_scratch_file(mem, prefix, fname, mode, 1);
 }
 
 int
@@ -901,7 +931,7 @@ validate(const gs_memory_t *mem,
          gs_path_control_t  type)
 {
     gs_lib_ctx_core_t *core = mem->gs_lib_ctx->core;
-    const gs_path_control_set_t *control;
+    gs_path_control_set_t *control;
     unsigned int i, n;
 
     switch (type) {
@@ -921,7 +951,7 @@ validate(const gs_memory_t *mem,
     n = control->num;
     for (i = 0; i < n; i++) {
         const char *a = path;
-        const char *b = control->paths[i];
+        const char *b = control->entry[i].path;
         while (1) {
             if (*a == 0) {
                 if (*b == 0)
@@ -963,13 +993,17 @@ validate(const gs_memory_t *mem,
                 /* Continue matching */
                 a--; /* Subtract 1 as the loop will increment it again later */
             } else if (*b == 0) {
-                if (b != control->paths[i] &&
+                if (b != control->entry[i].path &&
                     gs_file_name_check_separator(b, -1, b)) {
                     const char *a2 = a;
                     const char *aend = path + strlen(path);
                     while (aend != a2 && !gs_file_name_check_separator(a2, 1, a2))
                       a2++;
-                    if (aend != a2)
+                    /* If the control path ends in a directory separator and we've scanned
+                       to the end of the candidate path with no further directory separators
+                       found, we have a match
+                     */
+                    if (aend == a2)
                       /* PATH=abc/? pattern=abc/ */
                       goto found; /* Bingo! */
                  }
@@ -999,62 +1033,119 @@ validate(const gs_memory_t *mem,
     return gs_error_invalidfileaccess;
 
 found:
-    return 0;
+    return control->entry[i].flags;
 }
 
 int
 gp_validate_path_len(const gs_memory_t *mem,
-                 const char        *path,
-                 const uint         len,
-                 const char        *mode)
+                     const char        *path,
+                     const uint         len,
+                     const char        *mode)
 {
-    char *buffer;
+    char *buffer, *bufferfull;
     uint rlen;
     int code = 0;
+    const char *cdirstr = gp_file_name_current();
+    int cdirstrl = strlen(cdirstr);
+    const char *dirsepstr = gp_file_name_separator();
+    int dirsepstrl = strlen(dirsepstr);
+    int prefix_len = cdirstrl + dirsepstrl;
 
     /* mem->gs_lib_ctx can be NULL when we're called from mkromfs */
     if (mem->gs_lib_ctx == NULL ||
         mem->gs_lib_ctx->core->path_control_active == 0)
         return 0;
 
+    /* For current directory accesses, we need handle both a "bare" name,
+     * and one with a cwd prefix (in Unix terms, both "myfile.ps" and
+     * "./myfile.ps".
+     *
+     * So we check up front if it's absolute, then just use that.
+     * If it includes cwd prefix, we try that, then remove the prefix
+     * and try again.
+     * If it doesn't include the cwd prefix, we try it, then add the
+     * prefix and try again.
+     * To facilitate that, we allocate a large enough buffer to take
+     * the path *and* the prefix up front.
+     */
+    if (gp_file_name_is_absolute(path, len)) {
+       /* Absolute path, we don't need anything extra */
+       prefix_len = cdirstrl = dirsepstrl = 0;
+    }
+    else if (len > prefix_len && !memcmp(path, cdirstr, cdirstrl)
+             && !memcmp(path + cdirstrl, dirsepstr, dirsepstrl)) {
+          prefix_len = 0;
+    }
     rlen = len+1;
-    buffer = (char *)gs_alloc_bytes(mem->non_gc_memory, rlen, "gp_validate_path");
-    if (buffer == NULL)
+    bufferfull = (char *)gs_alloc_bytes(mem->thread_safe_memory, rlen + prefix_len, "gp_validate_path");
+    if (bufferfull == NULL)
         return gs_error_VMerror;
 
+    buffer = bufferfull + prefix_len;
     if (gp_file_name_reduce(path, (uint)len, buffer, &rlen) != gp_combine_success)
         return gs_error_invalidfileaccess;
     buffer[rlen] = 0;
 
-    switch (mode[0])
-    {
-    case 'r': /* Read */
-        code = validate(mem, buffer, gs_permit_file_reading);
+    while (1) {
+        switch (mode[0])
+        {
+        case 'r': /* Read */
+            code = validate(mem, buffer, gs_permit_file_reading);
+            break;
+        case 'w': /* Write */
+            code = validate(mem, buffer, gs_permit_file_writing);
+            break;
+        case 'a': /* Append needs reading and writing */
+            code = (validate(mem, buffer, gs_permit_file_reading) |
+                    validate(mem, buffer, gs_permit_file_writing));
+            break;
+        case 'c': /* "Control" */
+            code =  validate(mem, buffer, gs_permit_file_control);
+            break;
+        case 'd': /* "Delete" (special case of control) */
+            code =  validate(mem, buffer, gs_permit_file_control);
+            break;
+        case 'f': /* "Rename from" */
+            code = (validate(mem, buffer, gs_permit_file_writing) |
+                    validate(mem, buffer, gs_permit_file_control));
+            break;
+        case 't': /* "Rename to" */
+            code = (validate(mem, buffer, gs_permit_file_writing) |
+                    validate(mem, buffer, gs_permit_file_control));
+            break;
+        default:
+            errprintf(mem, "gp_validate_path: Unknown mode='%s'\n", mode);
+            code = gs_note_error(gs_error_invalidfileaccess);
+        }
+        if (code < 0 && prefix_len > 0 && buffer > bufferfull) {
+            buffer = bufferfull;
+            memcpy(buffer, cdirstr, cdirstrl);
+            memcpy(buffer + cdirstrl, dirsepstr, dirsepstrl);
+            continue;
+        }
+        else if (code < 0 && cdirstrl > 0 && prefix_len == 0 && buffer == bufferfull) {
+            buffer = bufferfull + cdirstrl + dirsepstrl;
+            continue;
+        }
         break;
-    case 'w': /* Write */
-        code = validate(mem, buffer, gs_permit_file_writing);
-        break;
-    case 'a': /* Append needs reading and writing */
-        code = (validate(mem, buffer, gs_permit_file_reading) |
-                validate(mem, buffer, gs_permit_file_writing));
-        break;
-    case 'c': /* "Control" */
-        code =  validate(mem, buffer, gs_permit_file_control);
-        break;
-    case 't': /* "Rename to" */
-        code = (validate(mem, buffer, gs_permit_file_writing) |
-                validate(mem, buffer, gs_permit_file_control));
-        break;
-    default:
-        errprintf(mem, "gp_validate_path: Unknown mode='%s'\n", mode);
-        code = gs_note_error(gs_error_invalidfileaccess);
     }
-    gs_free_object(mem->non_gc_memory, buffer, "gp_validate_path");
+    if (code > 0 && (mode[0] == 'd' || mode[0] == 'f') &&
+        (code & gs_path_control_flag_is_scratch_file) != 0) {
+        (void)gs_remove_control_path_flags(mem, gs_permit_file_reading, buffer,
+                                           gs_path_control_flag_is_scratch_file);
+        (void)gs_remove_control_path_flags(mem, gs_permit_file_writing, buffer,
+                                           gs_path_control_flag_is_scratch_file);
+        (void)gs_remove_control_path_flags(mem, gs_permit_file_control, buffer,
+                                           gs_path_control_flag_is_scratch_file);
+    }
+
+    gs_free_object(mem->thread_safe_memory, bufferfull, "gp_validate_path");
 #ifdef EACCES
     if (code == gs_error_invalidfileaccess)
         errno = EACCES;
 #endif
-    return code;
+
+    return code < 0 ? code : 0;
 }
 
 int
@@ -1064,3 +1155,25 @@ gp_validate_path(const gs_memory_t *mem,
 {
     return gp_validate_path_len(mem, path, strlen(path), mode);
 }
+
+int
+gp_unlink(gs_memory_t *mem, const char *fname)
+{
+    if (gp_validate_path(mem, fname, "d") != 0)
+        return gs_error_invalidaccess;
+
+    return gp_unlink_impl(mem, fname);
+}
+
+int
+gp_rename(gs_memory_t *mem, const char *from, const char *to)
+{
+    /* Always check 'to' before 'from', in case 'from' is a tempfile,
+     * and testing it might remove it from the list! */
+    if (gp_validate_path(mem, to, "t") != 0)
+        return gs_error_invalidaccess;
+    if (gp_validate_path(mem, from, "f") != 0)
+        return gs_error_invalidaccess;
+
+    return gp_rename_impl(mem, from, to);
+}
