@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -20,6 +20,7 @@
 #  define gscdevn_INCLUDED
 
 #include "gscspace.h"
+#include "gsfunc.h"
 
 /*
  * Construct a new a DeviceN color space.
@@ -49,10 +50,6 @@ extern int gs_cspace_set_devn_proc(
                                       );
 
 /* Set the DeviceN tint transformation procedure to a Function. */
-#ifndef gs_function_DEFINED
-typedef struct gs_function_s gs_function_t;
-#  define gs_function_DEFINED
-#endif
 int gs_cspace_set_devn_function(gs_color_space *pcspace,
                                    gs_function_t *pfn);
 
@@ -79,6 +76,6 @@ int gx_serialize_device_n_map(const gs_color_space * pcs, gs_device_n_map * m, s
  * color space is in the current (temp) gstate.  The DeviceN color space is
  * in the next gstate down in the gstate list (pgs->saved).
  */
-int gs_attachattributecolorspace(gs_separation_name sep_name, gs_gstate * pgs);
+int gs_attachcolorant(char *sep_name, gs_gstate * pgs);
 
 #endif /* gscdevn_INCLUDED */
