@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -19,7 +19,6 @@
 #  define gscms_INCLUDED
 
 #include "std.h"
-#include "stdpre.h"
 #include "gstypes.h"
 #include "gscspace.h"      /* for gs_color_space */
 #include "gsdevice.h"     /* Need to carry pointer to clist reader */
@@ -29,6 +28,10 @@
 #define ICC_MAX_CHANNELS 15
 #define NUM_DEVICE_PROFILES 4
 #define NUM_SOURCE_PROFILES 3
+#define GS_DEFAULT_DEVICE_PROFILE 0
+#define GS_GRAPHIC_DEVICE_PROFILE 1
+#define GS_IMAGE_DEVICE_PROFILE 2
+#define GS_TEXT_DEVICE_PROFILE 3
 
 #define AB_NEUTRAL_8 5
 #define AB_NEUTRAL_16 5
@@ -58,11 +61,6 @@ typedef struct gs_range_icc_s {
 /* This object is used only for device post processing CM.  It and its objects
    must be allocated in non-gc memory. */
 
-#ifndef cmm_profile_DEFINED
-typedef struct cmm_profile_s cmm_profile_t;
-#define cmm_profile_DEFINED
-#endif
-
 typedef struct gsicc_device_cm_s {
     cmm_profile_t *gray_profile;
     cmm_profile_t *rgb_profile;
@@ -252,10 +250,7 @@ struct gsicc_colorname_s {
 
 typedef struct gsicc_namelist_s gsicc_namelist_t;
 
-#ifndef gs_devicen_color_map_DEFINED
-#  define gs_devicen_color_map_DEFINED
 typedef struct gs_devicen_color_map_s gs_devicen_color_map;
-#endif
 
 struct gsicc_namelist_s {
     int count;
@@ -373,11 +368,6 @@ struct cmm_profile_s {
     gscms_free_profile_proc_t release;  /* Release the profile handle at CMM */
 };
 
-#ifndef cmm_profile_DEFINED
-typedef struct cmm_profile_s cmm_profile_t;
-#define cmm_profile_DEFINED
-#endif
-
 /* The above definition is plagued with an offset issue.  Probably should
    do away with gsicc_serialized_profile_t type */
 #ifndef offsetof
@@ -400,8 +390,8 @@ typedef struct gsicc_profile_entry_s gsicc_profile_entry_t;
 
 struct gsicc_profile_entry_s {
     gs_color_space *color_space;     /* The color space with the profile */
-    gsicc_profile_entry_t *next;    /* next CS */
-    int64_t key;                    /* Key based off dictionary location */
+    gsicc_profile_entry_t *next;     /* next CS */
+    uint64_t key;                    /* Key based off dictionary location */
 };
 
 /* ProfileList. The size of the list is limited by max_memory_size.
@@ -428,11 +418,6 @@ typedef enum {
 
 /* The link object. */
 
-#ifndef gsicc_link_DEFINED
-typedef struct gsicc_link_s gsicc_link_t;
-#  define gsicc_link_DEFINED
-#endif
-
 typedef struct gsicc_hashlink_s {
     int64_t link_hashcode;
     int64_t src_hash;
@@ -471,6 +456,8 @@ typedef struct gsicc_link_cache_s {
     rc_header rc;
     gs_memory_t *memory;
     gx_monitor_t *lock;		/* handle for the monitor */
+    bool cache_full;		/* flag that some thread needs a cache slot */
+    gx_semaphore_t *full_wait;	/* semaphore for waiting when the cache is full */
 } gsicc_link_cache_t;
 
 /* A linked list structure to keep DeviceN ICC profiles
