@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -43,8 +43,7 @@ gs_setcolorspace_only(gs_gstate * pgs, gs_color_space * pcs)
     if (pcs->id != cs_old->id) {
         rc_increment_cs(pcs);
         pgs->color[0].color_space = pcs;
-        if ( (code = pcs->type->install_cspace(pcs, pgs)) < 0          ||
-              (pgs->overprint && (code = gs_do_set_overprint(pgs)) < 0)  ) {
+        if ( (code = pcs->type->install_cspace(pcs, pgs)) < 0) {
             pgs->color[0].color_space = cs_old;
             rc_decrement_only_cs(pcs, "gs_setcolorspace");
         } else {
@@ -86,6 +85,7 @@ gs_setcolor(gs_gstate * pgs, const gs_client_color * pcc)
     gs_client_color     cc_old = *pgs->color[0].ccolor;
     gx_device_color *dev_color = pgs->color[0].dev_color;
     bool do_unset  = true;
+    bool op;
 
     if (pgs->in_cachedevice)
         return_error(gs_error_undefined); /* PLRM3 page 215. */
@@ -105,6 +105,18 @@ gs_setcolor(gs_gstate * pgs, const gs_client_color * pcc)
     (*pcs->type->restrict_color)(pgs->color[0].ccolor, pcs);
     (*pcs->type->adjust_color_count)(&cc_old, pcs, -1);
 
+    /* If we are in a situation where we are doing overprint
+       with OPM, then we need to update the overprint
+       compositor as these values can affect what is drawn */
+    op = pgs->is_fill_color ? pgs->overprint : pgs->stroke_overprint;
+    if (pgs->color[0].effective_opm && op) {
+        if (pgs->is_fill_color) {
+            gs_setfilloverprint(pgs, op);
+        } else {
+            gs_setstrokeoverprint(pgs, op);
+        }
+    }
+
     return 0;
 }
 
