@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -170,11 +170,9 @@ gx_install_Separation(gs_color_space * pcs, gs_gstate * pgs)
          * Give the device an opportunity to capture equivalent colors for any
          * spot colors which might be present in the color space.
          */
-        if (code >= 0) {
-            if (dev_proc(pgs->device, update_spot_equivalent_colors))
-               code = dev_proc(pgs->device, update_spot_equivalent_colors)
-                                                            (pgs->device, pgs);
-        }
+        if (dev_proc(pgs->device, update_spot_equivalent_colors))
+           code = dev_proc(pgs->device, update_spot_equivalent_colors)
+                                                        (pgs->device, pgs);
     }
     return code;
 }
@@ -186,16 +184,17 @@ gx_set_overprint_Separation(const gs_color_space * pcs, gs_gstate * pgs)
     gs_devicen_color_map *  pcmap = &pgs->color_component_map;
 
     if (pcmap->use_alt_cspace)
-        return gx_spot_colors_set_overprint(pcs->base_space, pgs);
+        return gx_set_no_overprint(pgs);
     else {
         gs_overprint_params_t   params;
 
-        /* We should not have to blend if we don't need the alternate tint transform */
-        params.retain_any_comps = pgs->overprint &&
-                                  pcs->params.separation.sep_type != SEP_ALL;
+        params.retain_any_comps = (((pgs->overprint && pgs->is_fill_color) ||
+                                   (pgs->stroke_overprint && !pgs->is_fill_color)) &&
+                                   (pcs->params.separation.sep_type != SEP_ALL));
+        params.is_fill_color = pgs->is_fill_color;
+        params.drawn_comps = 0;
+        params.op_state = OP_STATE_NONE;
         if (params.retain_any_comps) {
-            params.retain_spot_comps = false;
-            params.drawn_comps = 0;
             if (pcs->params.separation.sep_type != SEP_NONE) {
                 int mcomp = pcmap->color_map[0];
 
@@ -204,7 +203,7 @@ gx_set_overprint_Separation(const gs_color_space * pcs, gs_gstate * pgs)
             }
         }
         /* Only DeviceCMYK can use overprint mode */
-        pgs->effective_overprint_mode = 0;
+        params.effective_opm = pgs->color[0].effective_opm = 0;
         return gs_gstate_update_overprint(pgs, &params);
     }
 }
@@ -546,7 +545,7 @@ gx_serialize_Separation(const gs_color_space * pcs, stream * s)
 
     if (code < 0)
         return code;
-    code = sputs(s, (const byte *)&p->sep_name, strlen(p->sep_name) + 1, &n);
+    code = sputs(s, (const byte *)p->sep_name, strlen(p->sep_name) + 1, &n);
     if (code < 0)
         return code;
     code = cs_serialize(pcs->base_space, s);
