@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -417,6 +417,11 @@ gs_copydevice2(gx_device ** pnew_dev, const gx_device * dev, bool keep_open,
      */
     new_dev->is_open = dev->is_open && keep_open;
     fill_dev_proc(new_dev, finish_copydevice, gx_default_finish_copydevice);
+    /* We really want to be able to interrogate the device for capabilities
+     * and/or preferences right from when it is created, so set dev_spec_op
+     * now (if not already set).
+     */
+    fill_dev_proc(new_dev, dev_spec_op, gx_default_dev_spec_op);
     code = dev_proc(new_dev, finish_copydevice)(new_dev, dev);
     if (code < 0) {
         gs_free_object(mem, new_dev, "gs_copydevice(device)");
@@ -605,7 +610,7 @@ gs_setdevice_no_init(gs_gstate * pgs, gx_device * dev)
     }
     rc_assign(pgs->device, dev, "gs_setdevice_no_init");
     gs_gstate_update_device(pgs, dev);
-    return pgs->overprint ? gs_do_set_overprint(pgs) : 0;
+    return 0;
 }
 
 /* Initialize a just-allocated device. */
@@ -730,9 +735,11 @@ gs_nulldevice(gs_gstate * pgs)
                We just need something so we can end this job cleanly.
              */
             spgs = pgs->saved;
-            while (spgs->saved) spgs = spgs->saved;
-            gs_currentdevice_inline(pgs) = gs_currentdevice_inline(spgs);
-            rc_increment(gs_currentdevice_inline(pgs));
+            if (spgs != NULL) {
+                while (spgs->saved) spgs = spgs->saved;
+                gs_currentdevice_inline(pgs) = gs_currentdevice_inline(spgs);
+                rc_increment(gs_currentdevice_inline(pgs));
+            }
             code = gs_note_error(gs_error_Fatal);
         }
         if (gs_currentdevice_inline(pgs) != NULL)
@@ -1178,7 +1185,7 @@ int gx_device_delete_output_file(const gx_device * dev, const char *fname)
         parsed.len = strlen(parsed.fname);
     }
     if (parsed.iodev)
-        code = parsed.iodev->procs.delete_file((gx_io_device *)(&parsed.iodev), (const char *)parsed.fname);
+        code = parsed.iodev->procs.delete_file((gx_io_device *)(parsed.iodev), (const char *)parsed.fname);
     else
         code = gs_note_error(gs_error_invalidfileaccess);
 
@@ -1189,10 +1196,16 @@ done:
     return(code);
 }
 
+static int
+noclose(FILE *f)
+{
+    return 0;
+}
+
 /* Open the output file for a device. */
 int
 gx_device_open_output_file(const gx_device * dev, char *fname,
-                           bool binary, bool positionable, FILE ** pfile)
+                           bool binary, bool positionable, gp_file ** pfile)
 {
     gs_parsed_file_name_t parsed;
     const char *fmt;
@@ -1202,7 +1215,7 @@ gx_device_open_output_file(const gx_device * dev, char *fname,
     if (pfname == NULL) {
         code = gs_note_error(gs_error_VMerror);
 	goto done;
-     }
+    }
 
     if (strlen(fname) == 0) {
         code = gs_note_error(gs_error_undefinedfilename);
@@ -1219,9 +1232,14 @@ gx_device_open_output_file(const gx_device * dev, char *fname,
             code = gs_note_error(gs_error_undefinedfilename);
 	    goto done;
 	}
-        *pfile = dev->memory->gs_lib_ctx->fstdout;
+        *pfile = gp_file_FILE_alloc(dev->memory);
+        if (*pfile == NULL) {
+            code = gs_note_error(gs_error_VMerror);
+            goto done;
+        }
+        gp_file_FILE_set(*pfile, dev->memory->gs_lib_ctx->core->fstdout, noclose);
         /* Force stdout to binary. */
-        code = gp_setmode_binary(*pfile, true);
+        code = gp_setmode_binary_impl(dev->memory->gs_lib_ctx->core->fstdout, true);
 	goto done;
     } else if (parsed.iodev && !strcmp(parsed.iodev->dname, "%pipe%")) {
         positionable = false;
@@ -1243,7 +1261,8 @@ gx_device_open_output_file(const gx_device * dev, char *fname,
         parsed.fname = pfname;
         parsed.len = strlen(parsed.fname);
     }
-    if (positionable || (parsed.iodev && parsed.iodev != iodev_default(dev->memory))) {
+    if (parsed.iodev &&
+        (positionable || parsed.iodev != iodev_default(dev->memory))) {
         char fmode[4];
 
         if (!parsed.fname) {
@@ -1254,13 +1273,12 @@ gx_device_open_output_file(const gx_device * dev, char *fname,
         if (positionable)
             strcat(fmode, "+");
         code = parsed.iodev->procs.gp_fopen(parsed.iodev, parsed.fname, fmode,
-                                         pfile, NULL, 0);
+                                            pfile, NULL, 0, dev->memory);
         if (code)
             emprintf1(dev->memory,
                       "**** Could not open the file %s .\n",
                       parsed.fname);
-    }
-    else {
+    } else {
         *pfile = gp_open_printer(dev->memory, (pfname[0] ? pfname : fname), binary);
         if (!(*pfile)) {
             emprintf1(dev->memory, "**** Could not open the file '%s'.\n", (pfname[0] ? pfname : fname));
@@ -1279,7 +1297,7 @@ done:
 /* Close the output file for a device. */
 int
 gx_device_close_output_file(const gx_device * dev, const char *fname,
-                            FILE *file)
+                            gp_file *file)
 {
     gs_parsed_file_name_t parsed;
     const char *fmt;
@@ -1295,7 +1313,7 @@ gx_device_close_output_file(const gx_device * dev, const char *fname,
         if (parsed.iodev != iodev_default(dev->memory))
             return parsed.iodev->procs.fclose(parsed.iodev, file);
     }
-    gp_close_printer(dev->memory, file, (parsed.fname ? parsed.fname : fname));
+    gp_close_printer(file, (parsed.fname ? parsed.fname : fname));
     return 0;
 }
 
@@ -1333,3 +1351,33 @@ bool gx_color_info_equal(const gx_device_color_info * p1, const gx_device_color_
         return false;
     return true;
 }
+
+int gx_callout(gx_device *dev, int id, int size, void *data)
+{
+    return gs_lib_ctx_callout(dev->memory, dev->dname,
+                              id, size, data);
+}
+
+/* compare two space_params, we can't do this with memcmp since there is padding in the structure */
+int
+gdev_space_params_cmp(const gdev_space_params sp1,
+                      const gdev_space_params sp2) {
+  if (sp1.MaxBitmap != sp2.MaxBitmap)
+    return(1);
+  if (sp1.BufferSpace != sp2.BufferSpace)
+    return(1);
+  if (sp1.band.BandWidth != sp2.band.BandWidth)
+    return(1);
+  if (sp1.band.BandHeight != sp2.band.BandHeight)
+    return(1);
+  if (sp1.band.BandBufferSpace != sp2.band.BandBufferSpace)
+    return(1);
+  if (sp1.band.tile_cache_size != sp2.band.tile_cache_size)
+    return(1);
+  if (sp1.params_are_read_only != sp2.params_are_read_only)
+    return(1);
+  if (sp1.banding_type != sp2.banding_type)
+    return(1);
+
+  return(0);
+}
