@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* Default device parameters for Ghostscript library */
@@ -78,6 +78,7 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     gsicc_rendering_intents_t profile_intents[NUM_DEVICE_PROFILES];
     gsicc_blackptcomp_t blackptcomps[NUM_DEVICE_PROFILES];
     gsicc_blackpreserve_t blackpreserve[NUM_DEVICE_PROFILES];
+    int color_accuracy = MAX_COLOR_ACCURACY;
     int depth = dev->color_info.depth;
     cmm_dev_profile_t *dev_profile;
     char null_str[1]={'\0'};
@@ -86,7 +87,7 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     bool devicegraytok = true;  /* Default if device profile stuct not set */
     bool graydetection = false;
     bool usefastcolor = false;  /* set for unmanaged color */
-    bool sim_overprint = false;  /* By default do not simulate overprinting */
+    bool sim_overprint = true;  /* By default simulate overprinting (only valid with cmyk devices) */
     bool prebandthreshold = true, temp_bool = false;
 
     if(strcmp(Param, "OutputDevice") == 0){
@@ -215,16 +216,16 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
         return param_write_bool(plist, ".LockSafetyParams", &dev->LockSafetyParams);
     }
     if (strcmp(Param, "MaxPatternBitmap") == 0) {
-        return param_write_int(plist, "MaxPatternBitmap", &dev->MaxPatternBitmap);
+        return param_write_size_t(plist, "MaxPatternBitmap", &dev->MaxPatternBitmap);
     }
     if (strcmp(Param, "PageUsesTransparency") == 0) {
-        param_write_bool(plist, "PageUsesTransparency", &dev->page_uses_transparency);
+        return param_write_bool(plist, "PageUsesTransparency", &dev->page_uses_transparency);
     }
     if (strcmp(Param, "MaxBitmap") == 0) {
-        return param_write_long(plist, "MaxBitmap", &(dev->space_params.MaxBitmap));
+        return param_write_size_t(plist, "MaxBitmap", &(dev->space_params.MaxBitmap));
     }
     if (strcmp(Param, "BandBufferSpace") == 0) {
-        return param_write_long(plist, "BandBufferSpace", &dev->space_params.band.BandBufferSpace);
+        return param_write_size_t(plist, "BandBufferSpace", &dev->space_params.band.BandBufferSpace);
     }
     if (strcmp(Param, "BandHeight") == 0) {
         return param_write_int(plist, "BandHeight", &dev->space_params.band.BandHeight);
@@ -233,7 +234,7 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
         return param_write_int(plist, "BandWidth", &dev->space_params.band.BandWidth);
     }
     if (strcmp(Param, "BufferSpace") == 0) {
-        return param_write_long(plist, "BufferSpace", &dev->space_params.BufferSpace);
+        return param_write_size_t(plist, "BufferSpace", &dev->space_params.BufferSpace);
     }
     if (strcmp(Param, "InterpolateControl") == 0) {
         int interpolate_control = dev->interpolate_control;
@@ -280,7 +281,11 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     /* Check if the device profile is null.  If it is, then we need to
        go ahead and get it set up at this time.  If the proc is not
        set up yet then we are not going to do anything yet */
-    if (dev->procs.get_profile != NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) != NULL) {
         int code;
         code = dev_proc(dev, get_profile)(dev,  &dev_profile);
         if (code < 0)
@@ -408,6 +413,9 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     if (strcmp(Param, "TextICCProfile") == 0) {
         return param_write_string(plist,"TextICCProfile", &(profile_array[3]));
     }
+    if (strcmp(Param, "ColorAccuracy") == 0) {
+        return param_write_int(plist, "ColorAccuracy", (const int *)(&(color_accuracy)));
+    }
     if (strcmp(Param, "RenderIntent") == 0) {
         return param_write_int(plist,"RenderIntent", (const int *) (&(profile_intents[0])));
     }
@@ -498,9 +506,10 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
     bool devicegraytok = true;  /* Default if device profile stuct not set */
     bool graydetection = false;
     bool usefastcolor = false;  /* set for unmanaged color */
-    bool sim_overprint = false;  /* By default do not simulate overprinting */
+    bool sim_overprint = true;  /* By default simulate overprinting */
     bool prebandthreshold = true, temp_bool;
     int k;
+    int color_accuracy = MAX_COLOR_ACCURACY;
     gs_param_float_array msa, ibba, hwra, ma;
     gs_param_string_array scna;
     char null_str[1]={'\0'};
@@ -546,11 +555,22 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
     /* Check if the device profile is null.  If it is, then we need to
        go ahead and get it set up at this time.  If the proc is not
        set up yet then we are not going to do anything yet */
-    if (dev->procs.get_profile != NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) != NULL) {
         code = dev_proc(dev, get_profile)(dev,  &dev_profile);
+        if (code < 0)
+            return code;
+
         if (dev_profile == NULL) {
             code = gsicc_init_device_profile_struct(dev, NULL, 0);
+            if (code < 0)
+                return code;
             code = dev_proc(dev, get_profile)(dev,  &dev_profile);
+            if (code < 0)
+                return code;
         }
         /* It is possible that the current device profile name is NULL if we
            have a pdf14 device in line with a transparency group that is in a
@@ -670,7 +690,8 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
         (code = param_write_string(plist, "BlendColorProfile", &(blend_profile))) < 0 ||
         (code = param_write_string(plist,"DeviceLinkProfile", &(link_profile))) < 0 ||
         (code = param_write_string(plist,"ICCOutputColors", &(icc_colorants))) < 0 ||
-        (code = param_write_int(plist,"RenderIntent", (const int *) (&(profile_intents[0])))) < 0 ||
+        (code = param_write_int(plist, "RenderIntent", (const int *)(&(profile_intents[0])))) < 0 ||
+        (code = param_write_int(plist, "ColorAccuracy", (const int *)(&(color_accuracy)))) < 0 ||
         (code = param_write_int(plist,"GraphicIntent", (const int *) &(profile_intents[1]))) < 0 ||
         (code = param_write_int(plist,"ImageIntent", (const int *) &(profile_intents[2]))) < 0 ||
         (code = param_write_int(plist,"TextIntent", (const int *) &(profile_intents[3]))) < 0 ||
@@ -698,13 +719,13 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
         (code = param_write_bool(plist, "AntidropoutDownscaler",
                                 &dev->color_info.use_antidropout_downscaler)) < 0 ||
         (code = param_write_bool(plist, ".LockSafetyParams", &dev->LockSafetyParams)) < 0 ||
-        (code = param_write_int(plist, "MaxPatternBitmap", &dev->MaxPatternBitmap)) < 0 ||
+        (code = param_write_size_t(plist, "MaxPatternBitmap", &dev->MaxPatternBitmap)) < 0 ||
         (code = param_write_bool(plist, "PageUsesTransparency", &dev->page_uses_transparency)) < 0 ||
-        (code = param_write_long(plist, "MaxBitmap", &(dev->space_params.MaxBitmap))) < 0 ||
-        (code = param_write_long(plist, "BandBufferSpace", &dev->space_params.band.BandBufferSpace)) < 0 ||
+        (code = param_write_size_t(plist, "MaxBitmap", &(dev->space_params.MaxBitmap))) < 0 ||
+        (code = param_write_size_t(plist, "BandBufferSpace", &dev->space_params.band.BandBufferSpace)) < 0 ||
         (code = param_write_int(plist, "BandHeight", &dev->space_params.band.BandHeight)) < 0 ||
         (code = param_write_int(plist, "BandWidth", &dev->space_params.band.BandWidth)) < 0 ||
-        (code = param_write_long(plist, "BufferSpace", &dev->space_params.BufferSpace)) < 0 ||
+        (code = param_write_size_t(plist, "BufferSpace", &dev->space_params.BufferSpace)) < 0 ||
         (code = param_write_int(plist, "InterpolateControl", &dev->interpolate_control)) < 0
         )
         return code;
@@ -733,7 +754,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
     }
     else
         param_string_from_string(pagelist, null_str);
-    if ((code = param_write_name(plist, "PageList", &pagelist)) < 0)
+    if ((code = param_write_string(plist, "PageList", &pagelist)) < 0)
         return code;
 
     temp_bool = dev->ObjectFilter & FILTERIMAGE;
@@ -994,7 +1015,11 @@ gx_default_put_graydetection(bool graydetection, gx_device * dev)
     int code = 0;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1026,7 +1051,11 @@ gx_default_put_graytok(bool graytok, gx_device * dev)
     int code = 0;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1060,7 +1089,11 @@ gx_default_put_prebandthreshold(bool prebandthreshold, gx_device * dev)
     int code = 0;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1094,7 +1127,11 @@ gx_default_put_usefastcolor(bool fastcolor, gx_device * dev)
     int code = 0;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1128,7 +1165,11 @@ gx_default_put_simulateoverprint(bool sim_overprint, gx_device * dev)
     int code = 0;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1163,7 +1204,11 @@ gx_default_put_intent(gsicc_rendering_intents_t icc_intent, gx_device * dev,
     int code;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1177,6 +1222,8 @@ gx_default_put_intent(gsicc_rendering_intents_t icc_intent, gx_device * dev,
         code = gsicc_set_device_profile_intent(dev, icc_intent, index);
     } else {
         code = dev_proc(dev, get_profile)(dev,  &profile_struct);
+        if (code < 0)
+            return code;
         if (profile_struct == NULL) {
             /* Create now  */
             dev->icc_struct = gsicc_new_device_profile_array(dev->memory);
@@ -1195,7 +1242,11 @@ gx_default_put_blackpreserve(gsicc_blackpreserve_t blackpreserve, gx_device * de
     int code;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1232,7 +1283,11 @@ gx_default_put_blackptcomp(gsicc_blackptcomp_t blackptcomp, gx_device * dev,
     int code;
     cmm_dev_profile_t *profile_struct;
 
-    if (dev->procs.get_profile == NULL) {
+    /* Although device methods should not be NULL, they are not completely filled in until
+     * gx_device_fill_in_procs is called, and its possible for us to get here before this
+     * happens, so we *must* make sure the method is not NULL before we use it.
+     */
+    if (dev_proc(dev, get_profile) == NULL) {
         /* This is an odd case where the device has not yet fully been
            set up with its procedures yet.  We want to make sure that
            we catch this so we assume here that we are dealing with
@@ -1351,7 +1406,7 @@ gx_default_put_params(gx_device * dev, gs_param_list * plist)
     long ColorValues = (depth >= 32 ? -1 : 1L << depth);
     int tab = dev->color_info.anti_alias.text_bits;
     int gab = dev->color_info.anti_alias.graphics_bits;
-    int mpbm = dev->MaxPatternBitmap;
+    size_t mpbm = dev->MaxPatternBitmap;
     int ic = dev->interpolate_control;
     bool page_uses_transparency = dev->page_uses_transparency;
     gdev_space_params sp = dev->space_params;
@@ -1362,10 +1417,11 @@ gx_default_put_params(gx_device * dev, gs_param_list * plist)
     gs_param_string cms, pagelist;
     int leadingedge = dev->LeadingEdge;
     int k;
+    int color_accuracy;
     bool devicegraytok = true;
     bool graydetection = false;
     bool usefastcolor = false;
-    bool sim_overprint = false;
+    bool sim_overprint = true;
     bool prebandthreshold = false;
     bool use_antidropout = dev->color_info.use_antidropout_downscaler;
     bool temp_bool;
@@ -1374,6 +1430,7 @@ gx_default_put_params(gx_device * dev, gs_param_list * plist)
                                                gsIMAGEPROFILE,
                                                gsTEXTPROFILE};
 
+    color_accuracy = gsicc_currentcoloraccuracy(dev->memory);
     if (dev->icc_struct != NULL) {
         for (k = 0; k < NUM_DEVICE_PROFILES; k++) {
             rend_intent[k] = dev->icc_struct->rendercond[k].rendering_intent;
@@ -1481,7 +1538,6 @@ e:	param_signal_error(plist, param_name, ecode);\
             if (param_read_null(plist, "LeadingEdge") == 0) {
                 /* if param is null, clear explicitly-set flag */
                 leadingedge &= ~LEADINGEDGE_SET_MASK;
-                code = 0;
             } else {
                 ecode = code;
             }
@@ -1564,6 +1620,18 @@ nce:
             param_signal_error(plist, "ICCOutputColors", ecode);
         }
     }
+    if ((code = param_read_string(plist, "DeviceLinkProfile", &icc_pro)) != 1) {
+        if ((code = gx_default_put_icc(&icc_pro, dev, gsLINKPROFILE)) < 0) {
+            ecode = code;
+            param_signal_error(plist, "DeviceLinkProfile", ecode);
+        }
+    }
+    if ((code = param_read_string(plist, "PostRenderProfile", &icc_pro)) != 1) {
+        if ((code = gx_default_put_icc(&icc_pro, dev, gsPRPROFILE)) < 0) {
+            ecode = code;
+            param_signal_error(plist, "PostRenderProfile", ecode);
+        }
+    }
     if ((code = param_read_string(plist, "OutputICCProfile", &icc_pro)) != 1) {
         if ((code = gx_default_put_icc(&icc_pro, dev, gsDEFAULTPROFILE)) < 0) {
             ecode = code;
@@ -1596,18 +1664,6 @@ nce:
             param_signal_error(plist, "ProofProfile", ecode);
         }
     }
-    if ((code = param_read_string(plist, "DeviceLinkProfile", &icc_pro)) != 1) {
-        if ((code = gx_default_put_icc(&icc_pro, dev, gsLINKPROFILE)) < 0) {
-            ecode = code;
-            param_signal_error(plist, "DeviceLinkProfile", ecode);
-        }
-    }
-    if ((code = param_read_string(plist, "PostRenderProfile", &icc_pro)) != 1) {
-        if ((code = gx_default_put_icc(&icc_pro, dev, gsPRPROFILE)) < 0) {
-            ecode = code;
-            param_signal_error(plist, "PostRenderProfile", ecode);
-        }
-    }
     if ((code = param_read_string(plist, "BlendColorProfile", &icc_pro)) != 1) {
         if ((code = gx_default_put_icc(&icc_pro, dev, gsBLENDPROFILE)) < 0) {
             ecode = code;
@@ -1674,6 +1730,11 @@ nce:
         ecode = code;
         param_signal_error(plist, param_name, ecode);
     }
+    if ((code = param_read_int(plist, (param_name = "ColorAccuracy"),
+                                                        &color_accuracy)) < 0) {
+        ecode = code;
+        param_signal_error(plist, param_name, ecode);
+    }
     if ((code = param_read_bool(plist, (param_name = "DeviceGrayToK"),
                                                         &devicegraytok)) < 0) {
         ecode = code;
@@ -1709,7 +1770,7 @@ nce:
         ecode = code;
     if ((code = param_read_bool(plist, "AntidropoutDownscaler", &use_antidropout)) < 0)
         ecode = code;
-    if ((code = param_read_int(plist, "MaxPatternBitmap", &mpbm)) < 0)
+    if ((code = param_read_size_t(plist, "MaxPatternBitmap", &mpbm)) < 0)
         ecode = code;
     if ((code = param_read_int(plist, "InterpolateControl", &ic)) < 0)
         ecode = code;
@@ -1718,7 +1779,7 @@ nce:
         ecode = code;
         param_signal_error(plist, param_name, ecode);
     }
-    if ((code = param_read_long(plist, "MaxBitmap", &sp.MaxBitmap)) < 0)
+    if ((code = param_read_size_t(plist, "MaxBitmap", &sp.MaxBitmap)) < 0)
         ecode = code;
 
 #define CHECK_PARAM_CASES(member, bad, label)\
@@ -1735,7 +1796,7 @@ label:\
     case 1:\
         break
 
-    switch (code = param_read_long(plist, (param_name = "BufferSpace"), &sp.BufferSpace)) {
+    switch (code = param_read_size_t(plist, (param_name = "BufferSpace"), &sp.BufferSpace)) {
         CHECK_PARAM_CASES(BufferSpace, sp.BufferSpace < 10000, bse);
     }
 
@@ -1747,8 +1808,8 @@ label:\
         CHECK_PARAM_CASES(band.BandHeight, sp.band.BandHeight < 0, bhe);
     }
 
-    switch (code = param_read_long(plist, (param_name = "BandBufferSpace"), &sp.band.BandBufferSpace)) {
-        CHECK_PARAM_CASES(band.BandBufferSpace, sp.band.BandBufferSpace < 0, bbse);
+    switch (code = param_read_size_t(plist, (param_name = "BandBufferSpace"), &sp.band.BandBufferSpace)) {
+        CHECK_PARAM_CASES(band.BandBufferSpace, 0, bbse);
     }
 
 
@@ -1864,9 +1925,16 @@ label:\
     if (code == 0)
         dev->DisablePageHandler = temp_bool;
 
-    if ((code = param_read_string(plist, "PageList", &pagelist)) != 1 && pagelist.size > 0) {
+    code = param_read_string(plist, "PageList", &pagelist);
+    if (code < 0)
+        ecode = code;
+    if (code == 0) {
         if (dev->PageList)
             rc_decrement(dev->PageList, "default put_params PageList");
+        dev->PageList = NULL;
+    }
+
+    if (code == 0 && pagelist.size > 0) {
         dev->PageList = (gdev_pagelist *)gs_alloc_bytes(dev->memory->non_gc_memory, sizeof(gdev_pagelist), "structure to hold page list");
         if (!dev->PageList)
             return gs_note_error(gs_error_VMerror);
@@ -1915,10 +1983,20 @@ label:\
     /* We must 'commit', in order to detect unknown parameters, */
     /* even if there were errors. */
     code = param_commit(plist);
-    if (ecode < 0)
+    if (ecode < 0) {
+        /* restore_page_device (zdevice2.c) will turn off LockSafetyParams, and relies on putparams
+         * to put it back if we are restoring a device. The locksafe value is picked up above from the
+         * device we are restoring to, and we *must* make sure it is preserved, even if setting the
+         * params failed. Otherwise an attacker can use a failed grestore to reset LockSafetyParams.
+         * See bug #699687.
+         */
+        dev->LockSafetyParams = locksafe;
         return ecode;
-    if (code < 0)
+    }
+    if (code < 0) {
+        dev->LockSafetyParams = locksafe;
         return code;
+    }
 
     /*
      * Now actually make the changes. Changing resolution, rotation
@@ -2045,6 +2123,7 @@ label:\
                 return code;
         }
     }
+    gsicc_setcoloraccuracy(dev->memory, color_accuracy);
     code = gx_default_put_graytok(devicegraytok, dev);
     if (code < 0)
         return code;
