@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -178,7 +178,7 @@ gs_screen_enum_alloc(gs_memory_t * mem, client_name_t cname)
 
 /* Set up for halftone sampling. */
 int
-gs_screen_init(gs_screen_enum * penum, gs_state * pgs,
+gs_screen_init(gs_screen_enum * penum, gs_gstate * pgs,
                gs_screen_halftone * phsp)
 {
     gs_lib_ctx_t *ctx = gs_lib_ctx_get_interp_instance(pgs->memory);
@@ -187,7 +187,7 @@ gs_screen_init(gs_screen_enum * penum, gs_state * pgs,
                                    ctx->screen_accurate_screens);
 }
 int
-gs_screen_init_memory(gs_screen_enum * penum, gs_state * pgs,
+gs_screen_init_memory(gs_screen_enum * penum, gs_gstate * pgs,
                 gs_screen_halftone * phsp, bool accurate, gs_memory_t * mem)
 {
     int code =
@@ -209,8 +209,8 @@ gs_screen_order_alloc(gx_ht_order *porder, gs_memory_t *mem)
 
     if (!FORCE_STRIP_HALFTONES &&
         ((ulong)porder->params.W1 * bitmap_raster(porder->params.W) +
-           num_levels * sizeof(*porder->levels) +
-           porder->params.W * porder->params.W1 * sizeof(gx_ht_bit)) <=
+           (ulong)num_levels * sizeof(*porder->levels) +
+           (ulong)porder->params.W * porder->params.W1 * sizeof(gx_ht_bit)) <=
         porder->screen_params.max_size) {
         /*
          * Allocate an order for the entire tile, but only sample one
@@ -232,7 +232,7 @@ gs_screen_order_alloc(gx_ht_order *porder, gs_memory_t *mem)
     return code;
 }
 int
-gs_screen_order_init_memory(gx_ht_order * porder, const gs_state * pgs,
+gs_screen_order_init_memory(gx_ht_order * porder, const gs_gstate * pgs,
                             gs_screen_halftone * phsp, bool accurate,
                             gs_memory_t * mem)
 {
@@ -346,6 +346,8 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
         return_error(gs_error_rangecheck);
     while ((fabs(u0) + fabs(v0)) * rt < 4)
         ++rt;
+    phcp->C = 0;
+
   try_size:
     better = false;
     {
@@ -366,6 +368,10 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
 
                 p.M1 = (int)floor(p.M / T + 0.5);
                 p.N1 = (int)floor(p.N * T + 0.5);
+
+                if (p.M1 == 0 && p.N1 == 0)
+                    return_error(gs_error_rangecheck);
+
                 gx_compute_cell_values(&p);
                 if_debug3('h', "[h]trying m=%d, n=%d, r=%d\n", p.M, p.N, rt);
                 wt = p.W;
@@ -383,11 +389,13 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
                 /* Compute the corresponding values of F and A. */
 
                 if (landscape)
-                    ar = atan2(p.M * pmat->xy, p.N * pmat->yx),
+                    ar = atan2(p.M * (double)pmat->xy,
+                               p.N * (double)pmat->yx),
                         fr = 72.0 * (p.M == 0 ? pmat->xy / p.N * cos(ar) :
                                      pmat->yx / p.M * sin(ar));
                 else
-                    ar = atan2(p.N * pmat->xx, p.M * pmat->yy),
+                    ar = atan2(p.N * (double)pmat->xx,
+                               p.M * (double)pmat->yy),
                         fr = 72.0 * (p.M == 0 ? pmat->yy / p.N * sin(ar) :
                                      pmat->xx / p.M * cos(ar));
                 ft = fabs(fr) * rt;
@@ -439,6 +447,15 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
                     goto done;
             }
     }
+    /* It is possible (for ridiculous halftone screens) to reach this point without ever
+     * having been able to find a suitable screen. In that case we will not have set
+     * phcp and so phcp->C will be undefined. We can detect this condition by checking rt
+     * is 1 (so its the first attempt to find a screen) and better is false (we didn't find
+     * a better match). If that happens, exit with an error.
+     */
+    if (rt == 1 && !better)
+        return_error(gs_error_rangecheck);
+
     if (phcp->C < min_levels) { /* We don't have enough levels yet.  Keep going. */
         ++rt;
         goto try_size;
@@ -471,9 +488,11 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
 /* This is the second half of gs_screen_init_accurate. */
 int
 gs_screen_enum_init_memory(gs_screen_enum * penum, const gx_ht_order * porder,
-                           gs_state * pgs, const gs_screen_halftone * phsp,
+                           gs_gstate * pgs, const gs_screen_halftone * phsp,
                            gs_memory_t * mem)
 {
+    int code;
+
     penum->pgs = pgs;           /* ensure clean for GC */
     if (&penum->order != porder) /* Pacify Valgrind */
         penum->order = *porder;
@@ -514,13 +533,13 @@ gs_screen_enum_init_memory(gs_screen_enum * penum, const gx_ht_order * porder,
         penum->mat.yy = Q * (R1 * M);
         penum->mat.tx = -1.0;
         penum->mat.ty = -1.0;
-        gs_matrix_invert(&penum->mat, &penum->mat_inv);
+        code = gs_matrix_invert(&penum->mat, &penum->mat_inv);
     }
     if_debug7('h', "[h]Screen: (%dx%d)/%d [%f %f %f %f]\n",
               porder->width, porder->height, porder->params.R,
               penum->mat.xx, penum->mat.xy,
               penum->mat.yx, penum->mat.yy);
-    return 0;
+    return code;
 }
 
 /* Report current point for sampling */
