@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -32,6 +32,7 @@
 #include "gsicc_manage.h"
 #include "gxdevice.h"
 #include "gsccolor.h"
+#include "gxdevsop.h"
 
 #define SAVEICCPROFILE 0
 
@@ -316,7 +317,8 @@ gx_remap_concrete_icc_devicen(const gs_color_space * pcs, const frac * pconc,
            up the equivalent CMYK colors for the spot colors that are present.
            This allows us to have some sort of composite viewing of the spot
            colors as they would colorimetrically appear. */
-        code = gsicc_set_devicen_equiv_colors(dev, pgs, dev_profile->device_profile[0]);
+        code = gsicc_set_devicen_equiv_colors(dev, pgs,
+                        dev_profile->device_profile[GS_DEFAULT_DEVICE_PROFILE]);
         dev_profile->spotnames->equiv_cmyk_set = true;
     }
     gx_remap_concrete_devicen(pconc, pdc, pgs, dev, select, pcs);
@@ -353,12 +355,11 @@ gx_remap_concrete_ICC(const gs_color_space * pcs, const frac * pconc,
  * To device space
  */
 int
-gx_remap_ICC(const gs_client_color * pcc, const gs_color_space * pcs,
+gx_remap_ICC_with_link(const gs_client_color * pcc, const gs_color_space * pcs,
         gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
-                gs_color_select_t select)
+                gs_color_select_t select, gsicc_link_t *icc_link)
 {
-    gsicc_link_t *icc_link;
-    gsicc_rendering_param_t rendering_params;
+    cmm_dev_profile_t *dev_profile;
     unsigned short psrc[GS_CLIENT_COLOR_MAX_COMPONENTS], psrc_cm[GS_CLIENT_COLOR_MAX_COMPONENTS];
     unsigned short *psrc_temp;
     frac conc[GS_CLIENT_COLOR_MAX_COMPONENTS];
@@ -368,20 +369,15 @@ gx_remap_ICC(const gs_client_color * pcc, const gs_color_space * pcs,
 #endif
     int num_des_comps;
     int code;
-    cmm_dev_profile_t *dev_profile;
 
     code = dev_proc(dev, get_profile)(dev, &dev_profile);
     if (code < 0)
         return code;
     if (dev_profile == NULL)
         return gs_throw(gs_error_Fatal, "Attempting to do ICC remap with no profile");
-    num_des_comps = gsicc_get_device_profile_comps(dev_profile);
-    rendering_params.black_point_comp = pgs->blackptcomp;
-    rendering_params.graphics_type_tag = dev->graphics_type_tag;
-    rendering_params.override_icc = false;
-    rendering_params.preserve_black = gsBKPRESNOTSPECIFIED;
-    rendering_params.rendering_intent = pgs->renderingintent;
-    rendering_params.cmm = gsCMM_DEFAULT;
+    if (icc_link == NULL)
+        return gs_throw(gs_error_Fatal, "Attempting to do ICC remap with no link");
+
     /* Need to clear out psrc_cm in case we have separation bands that are
        not color managed */
     memset(psrc_cm,0,sizeof(unsigned short)*GS_CLIENT_COLOR_MAX_COMPONENTS);
@@ -397,14 +393,7 @@ gx_remap_ICC(const gs_client_color * pcc, const gs_color_space * pcs,
             psrc[k] = (unsigned short) (pcc->paint.values[k]*65535.0);
         }
     }
-    /* Get a link from the cache, or create if it is not there. Need to get 16 bit profile */
-    icc_link = gsicc_get_link(pgs, dev, pcs, NULL, &rendering_params, pgs->memory);
-    if (icc_link == NULL) {
-        #ifdef DEBUG
-            gs_warn("Could not create ICC link:  Check profiles");
-        #endif
-        return -1;
-    }
+    num_des_comps = gsicc_get_device_profile_comps(dev_profile);
     if (icc_link->is_identity) {
         psrc_temp = &(psrc[0]);
     } else {
@@ -434,8 +423,6 @@ gx_remap_ICC(const gs_client_color * pcc, const gs_color_space * pcs,
         if_debug0m(gs_debug_flag_icc, dev->memory, "]\n");
     }
 #endif
-    /* Release the link */
-    gsicc_release_link(icc_link);
     /* Now do the remap for ICC which amounts to the alpha application
        the transfer function and potentially the halftoning */
     /* Right now we need to go from unsigned short to frac.  I really
@@ -453,6 +440,42 @@ gx_remap_ICC(const gs_client_color * pcc, const gs_color_space * pcs,
     return 0;
 }
 
+int
+gx_remap_ICC(const gs_client_color * pcc, const gs_color_space * pcs,
+        gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
+                gs_color_select_t select)
+{
+    gsicc_link_t *icc_link;
+    gsicc_rendering_param_t rendering_params;
+    cmm_dev_profile_t *dev_profile;
+    int code;
+
+    code = dev_proc(dev, get_profile)(dev, &dev_profile);
+    if (code < 0)
+        return code;
+    if (dev_profile == NULL)
+        return gs_throw(gs_error_Fatal, "Attempting to do ICC remap with no profile");
+
+    rendering_params.black_point_comp = pgs->blackptcomp;
+    rendering_params.graphics_type_tag = dev->graphics_type_tag;
+    rendering_params.override_icc = false;
+    rendering_params.preserve_black = gsBKPRESNOTSPECIFIED;
+    rendering_params.rendering_intent = pgs->renderingintent;
+    rendering_params.cmm = gsCMM_DEFAULT;
+    /* Get a link from the cache, or create if it is not there. Need to get 16 bit profile */
+    icc_link = gsicc_get_link(pgs, dev, pcs, NULL, &rendering_params, pgs->memory);
+    if (icc_link == NULL) {
+#ifdef DEBUG
+        gs_warn("Could not create ICC link:  Check profiles");
+#endif
+        return_error(gs_error_unknownerror);
+    }
+    code = gx_remap_ICC_with_link(pcc, pcs, pdc, pgs, dev, select, icc_link);
+    /* Release the link */
+    gsicc_release_link(icc_link);
+    return code;
+}
+
 /*
  * Same as above, but there is no rescale of CIELAB colors.  This is needed
    since the rescale is not needed when the source data is image based.
@@ -494,10 +517,10 @@ gx_remap_ICC_imagelab(const gs_client_color * pcc, const gs_color_space * pcs,
     /* Get a link from the cache, or create if it is not there. Need to get 16 bit profile */
     icc_link = gsicc_get_link(pgs, dev, pcs, NULL, &rendering_params, pgs->memory);
     if (icc_link == NULL) {
-        #ifdef DEBUG
-                gs_warn("Could not create ICC link:  Check profiles");
-        #endif
-        return -1;
+#ifdef DEBUG
+        gs_warn("Could not create ICC link:  Check profiles");
+#endif
+        return_error(gs_error_unknownerror);
     }
     if (icc_link->is_identity) {
         psrc_temp = &(psrc[0]);
@@ -562,10 +585,10 @@ gx_concretize_ICC(
     /* Get a link from the cache, or create if it is not there. Get 16 bit profile */
     icc_link = gsicc_get_link(pgs, dev, pcs, NULL, &rendering_params, pgs->memory);
     if (icc_link == NULL) {
-        #ifdef DEBUG
-                gs_warn("Could not create ICC link:  Check profiles");
-        #endif
-        return -1;
+#ifdef DEBUG
+        gs_warn("Could not create ICC link:  Check profiles");
+#endif
+        return_error(gs_error_unknownerror);
     }
     /* Transform the color */
     if (icc_link->is_identity) {
@@ -623,6 +646,9 @@ gs_cspace_build_ICC(
     gs_memory_t *       pmem )
 {
     gs_color_space *pcspace = gs_cspace_alloc(pmem, &gs_color_space_type_ICC);
+    if (!pcspace) {
+        return_error(gs_error_VMerror);
+    }
     *ppcspace = pcspace;
 
     return 0;
@@ -653,9 +679,10 @@ gx_set_overprint_ICC(const gs_color_space * pcs, gs_gstate * pgs)
     bool cs_ok;
     cmm_dev_profile_t *dev_profile;
     bool gray_to_k;
+    bool op = pgs->is_fill_color ? pgs->overprint : pgs->stroke_overprint;
 
     if (dev == 0 || pcinfo == NULL)
-        return gx_spot_colors_set_overprint(pcs, pgs);
+        return gx_set_no_overprint(pgs);
 
     dev_proc(dev, get_profile)(dev, &dev_profile);
     gray_to_k = dev_profile->devicegraytok;
@@ -666,9 +693,21 @@ gx_set_overprint_ICC(const gs_color_space * pcs, gs_gstate * pgs)
     cs_ok = ((pcs->cmm_icc_profile_data->data_cs == gsCMYK) ||
         (pcs->cmm_icc_profile_data->data_cs == gsGRAY && gray_to_k));
 
-    if (!pgs->overprint || pcinfo->opmode == GX_CINFO_OPMODE_NOT || !cs_ok)
-        return gx_spot_colors_set_overprint(pcs, pgs);
-    else
+    if_debug4m(gs_debug_flag_overprint, pgs->memory,
+        "[overprint] gx_set_overprint_ICC. cs_ok = %d is_fill_color = %d overprint = %d stroke_overprint = %d \n",
+        cs_ok, pgs->is_fill_color, pgs->overprint, pgs->stroke_overprint);
+
+    if (!op || pcinfo->opmode == GX_CINFO_OPMODE_NOT) {
+        return gx_set_no_overprint(pgs);
+    } else if (!cs_ok) {
+        /* In this case, we still need to maintain any spot
+           colorant channels.  Per Table 7.14. */
+        if (dev_proc(dev, dev_spec_op)(dev, gxdso_supports_devn, NULL, 0)) {
+            return gx_set_spot_only_overprint(pgs);
+        } else {
+            return gx_set_no_overprint(pgs);
+        }
+    } else
         return gx_set_overprint_cmyk(pcs, pgs);
 }
 
@@ -688,7 +727,7 @@ gx_change_color_model(gx_device *dev, int num_comps, int bit_depth)
     int k;
 
     if (!((num_comps == 1) || (num_comps == 3) || (num_comps == 4)))
-        return -1;
+        return_error(gs_error_unknownerror);
 
     dev->color_info.max_components = num_comps;
     dev->color_info.num_components = num_comps;
