@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -104,9 +104,10 @@ gsicc_cache_new(gs_memory_t *memory)
                              "gsicc_cache_new");
     if ( result == NULL )
         return(NULL);
-#ifdef MEMENTO_SQUEEZE_BUILD
-    result->lock = NULL;
-#else
+    result->head = NULL;
+    result->num_links = 0;
+    result->cache_full = false;
+    result->memory = memory->stable_memory;
     result->lock = gx_monitor_label(gx_monitor_alloc(memory->stable_memory),
                                     "gsicc_cache_new");
     if (result->lock == NULL) {
@@ -120,15 +121,10 @@ gsicc_cache_new(gs_memory_t *memory)
         gs_free_object(memory->stable_memory, result, "gsicc_cache_new");
         return(NULL);
     }
-#endif
     rc_init_free(result, memory->stable_memory, 1, rc_gsicc_link_cache_free);
-    result->head = NULL;
-    result->num_links = 0;
-    result->cache_full = false;
-    result->memory = memory->stable_memory;
     if_debug2m(gs_debug_flag_icc, memory,
-               "[icc] Allocating link cache = 0x%p memory = 0x%p\n",
-	       result, result->memory);
+               "[icc] Allocating link cache = "PRI_INTPTR" memory = "PRI_INTPTR"\n",
+	       (intptr_t)result, (intptr_t)result->memory);
     return(result);
 }
 
@@ -139,8 +135,8 @@ rc_gsicc_link_cache_free(gs_memory_t * mem, void *ptr_in, client_name_t cname)
     gsicc_link_cache_t *link_cache = (gsicc_link_cache_t * ) ptr_in;
 
     if_debug2m(gs_debug_flag_icc, mem,
-               "[icc] Removing link cache = 0x%p memory = 0x%p\n",
-               link_cache, link_cache->memory);
+               "[icc] Removing link cache = "PRI_INTPTR" memory = "PRI_INTPTR"\n",
+               (intptr_t)link_cache, (intptr_t)link_cache->memory);
     /* NB: freeing the link_cache will call icc_linkcache_finalize */
     gs_free_object(mem->stable_memory, link_cache, "rc_gsicc_link_cache_free");
 }
@@ -153,8 +149,8 @@ icc_linkcache_finalize(const gs_memory_t *mem, void *ptr)
 
     while (link_cache->head != NULL) {
         if (link_cache->head->ref_count != 0) {
-            emprintf2(mem, "link at 0x%p being removed, but has ref_count = %d\n",
-                      link_cache->head, link_cache->head->ref_count);
+            emprintf2(mem, "link at "PRI_INTPTR" being removed, but has ref_count = %d\n",
+                      (intptr_t)link_cache->head, link_cache->head->ref_count);
             link_cache->head->ref_count = 0;	/* force removal */
         }
         gsicc_remove_link(link_cache->head, mem);
@@ -165,12 +161,10 @@ icc_linkcache_finalize(const gs_memory_t *mem, void *ptr)
     }
 #endif
     if (link_cache->rc.ref_count == 0) {
-#ifndef MEMENTO_SQUEEZE_BUILD
         gx_monitor_free(link_cache->lock);
         link_cache->lock = NULL;
         gx_semaphore_free(link_cache->full_wait);
         link_cache->full_wait = 0;
-#endif
     }
 }
 
@@ -190,7 +184,6 @@ gsicc_alloc_link_dev(gs_memory_t *memory, cmm_profile_t *src_profile,
 
     if (result == NULL)
         return NULL;
-#ifndef MEMENTO_SQUEEZE_BUILD
     result->lock = gx_monitor_label(gx_monitor_alloc(memory->stable_memory),
         "gsicc_link_new");
     if (result->lock == NULL) {
@@ -198,7 +191,6 @@ gsicc_alloc_link_dev(gs_memory_t *memory, cmm_profile_t *src_profile,
         return NULL;
     }
     gx_monitor_enter(result->lock);
-#endif
 
     /* set up placeholder values */
     result->is_monitored = false;
@@ -222,8 +214,8 @@ gsicc_alloc_link_dev(gs_memory_t *memory, cmm_profile_t *src_profile,
     result->valid = true;
     result->memory = memory->stable_memory;
 
-    if_debug1m('^', result->memory, "[^]icclink 0x%p init = 1\n",
-               result);
+    if_debug1m('^', result->memory, "[^]icclink "PRI_INTPTR" init = 1\n",
+               (intptr_t)result);
 
     if (src_profile->profile_handle == NULL) {
         src_profile->profile_handle = gsicc_get_profile_handle_buffer(
@@ -283,15 +275,6 @@ gsicc_alloc_link(gs_memory_t *memory, gsicc_hashlink_t hashcode)
                              "gsicc_alloc_link");
     if (result == NULL)
         return NULL;
-#ifndef MEMENTO_SQUEEZE_BUILD
-    result->lock = gx_monitor_label(gx_monitor_alloc(memory->stable_memory),
-                                    "gsicc_link_new");
-    if (result->lock == NULL) {
-        gs_free_object(memory->stable_memory, result, "gsicc_alloc_link(lock)");
-        return NULL;
-    }
-    gx_monitor_enter(result->lock);     /* this link is owned by this thread until built and made "valid" */
-#endif
     /* set up placeholder values */
     result->is_monitored = false;
     result->orig_procs.map_buffer = NULL;
@@ -313,8 +296,16 @@ gsicc_alloc_link(gs_memory_t *memory, gsicc_hashlink_t hashcode)
     result->valid = false;		/* not yet complete */
     result->memory = memory->stable_memory;
 
-    if_debug1m('^', result->memory, "[^]icclink 0x%p init = 1\n",
-               result);
+    result->lock = gx_monitor_label(gx_monitor_alloc(memory->stable_memory),
+                                    "gsicc_link_new");
+    if (result->lock == NULL) {
+        gs_free_object(memory->stable_memory, result, "gsicc_alloc_link(lock)");
+        return NULL;
+    }
+    gx_monitor_enter(result->lock);     /* this link is owned by this thread until built and made "valid" */
+
+    if_debug1m('^', result->memory, "[^]icclink "PRI_INTPTR" init = 1\n",
+               (intptr_t)result);
     return result;
 }
 
@@ -324,9 +315,7 @@ gsicc_set_link_data(gsicc_link_t *icc_link, void *link_handle,
                     bool includes_softproof, bool includes_devlink,
                     bool pageneutralcolor, gsicc_colorbuffer_t data_cs)
 {
-#ifndef MEMENTO_SQUEEZE_BUILD
     gx_monitor_enter(lock);		/* lock the cache while changing data */
-#endif
     icc_link->link_handle = link_handle;
     gscms_get_link_dim(link_handle, &(icc_link->num_input), &(icc_link->num_output),
         icc_link->memory);
@@ -349,20 +338,16 @@ gsicc_set_link_data(gsicc_link_t *icc_link, void *link_handle,
 
     /* release the lock of the link so it can now be used */
     icc_link->valid = true;
-#ifndef MEMENTO_SQUEEZE_BUILD
     gx_monitor_leave(icc_link->lock);
     gx_monitor_leave(lock);	/* done with updating, let everyone run */
-#endif
 }
 
 static void
 gsicc_link_free_contents(gsicc_link_t *icc_link)
 {
     icc_link->procs.free_link(icc_link);
-#ifndef MEMENTO_SQUEEZE_BUILD
     gx_monitor_free(icc_link->lock);
     icc_link->lock = NULL;
-#endif
 }
 
 void
@@ -514,9 +499,7 @@ gsicc_findcachelink(gsicc_hashlink_t hash, gsicc_link_cache_t *icc_link_cache,
     int64_t hashcode = hash.link_hashcode;
 
     /* Look through the cache for the hashcode */
-#ifndef MEMENTO_SQUEEZE_BUILD
     gx_monitor_enter(icc_link_cache->lock);
-#endif
 
     /* List scanning is fast, so we scan the entire list, this includes   */
     /* links that are currently unused, but still in the cache (zero_ref) */
@@ -537,10 +520,9 @@ gsicc_findcachelink(gsicc_hashlink_t hash, gsicc_link_cache_t *icc_link_cache,
             }
             /* bump the ref_count since we will be using this one */
             curr->ref_count++;
-            if_debug3m('^', curr->memory, "[^]%s 0x%p ++ => %d\n",
-                       "icclink", curr, curr->ref_count);
+            if_debug3m('^', curr->memory, "[^]%s "PRI_INTPTR" ++ => %d\n",
+                       "icclink", (intptr_t)curr, curr->ref_count);
             while (curr->valid == false) {
-#ifndef MEMENTO_SQUEEZE_BUILD
                 gx_monitor_leave(icc_link_cache->lock); /* exit to let other threads run briefly */
                 gx_monitor_enter(curr->lock);			/* wait until we can acquire the lock */
                 gx_monitor_leave(curr->lock);			/* it _should be valid now */
@@ -548,22 +530,17 @@ gsicc_findcachelink(gsicc_hashlink_t hash, gsicc_link_cache_t *icc_link_cache,
                 /* that was building it failed to be able to complete building it		*/
                 /* this is probably a fatal error. MV ???					*/
                 if (curr->valid == false) {
-		  emprintf1(curr->memory, "link 0x%p lock released, but still not valid.\n", curr);	/* Breakpoint here */
+		  emprintf1(curr->memory, "link "PRI_INTPTR" lock released, but still not valid.\n", (intptr_t)curr);	/* Breakpoint here */
                 }
                 gx_monitor_enter(icc_link_cache->lock);	/* re-enter to loop and check */
-#endif
             }
-#ifndef MEMENTO_SQUEEZE_BUILD
             gx_monitor_leave(icc_link_cache->lock);
-#endif
             return(curr);	/* success */
         }
         prev = curr;
         curr = curr->next;
     }
-#ifndef MEMENTO_SQUEEZE_BUILD
     gx_monitor_leave(icc_link_cache->lock);
-#endif
     return NULL;
 }
 
@@ -575,14 +552,12 @@ gsicc_remove_link(gsicc_link_t *link, const gs_memory_t *memory)
     gsicc_link_cache_t *icc_link_cache = link->icc_link_cache;
 
     if_debug2m(gs_debug_flag_icc, memory,
-               "[icc] Removing link = 0x%p memory = 0x%p\n", link,
-               memory->stable_memory);
+               "[icc] Removing link = "PRI_INTPTR" memory = "PRI_INTPTR"\n",
+               (intptr_t)link, (intptr_t)memory->stable_memory);
     /* NOTE: link->ref_count must be 0: assert ? */
-#ifndef MEMENTO_SQUEEZE_BUILD
     gx_monitor_enter(icc_link_cache->lock);
-#endif
     if (link->ref_count != 0) {
-      emprintf2(memory, "link at 0x%p being removed, but has ref_count = %d\n", link, link->ref_count);
+      emprintf2(memory, "link at "PRI_INTPTR" being removed, but has ref_count = %d\n", (intptr_t)link, link->ref_count);
     }
     curr = icc_link_cache->head;
     prev = NULL;
@@ -604,19 +579,15 @@ gsicc_remove_link(gsicc_link_t *link, const gs_memory_t *memory)
     /* use it (ref_count > 0). Skip freeing it if so.                          */
     if (curr == link && link->ref_count == 0) {
         icc_link_cache->num_links--;	/* no longer in the cache */
-#ifndef MEMENTO_SQUEEZE_BUILD
         if (icc_link_cache->cache_full) {
             icc_link_cache->cache_full = false;
             gx_semaphore_signal(icc_link_cache->full_wait);	/* let a waiting thread run */
         }
         gx_monitor_leave(icc_link_cache->lock);
-#endif
         gsicc_link_free(link, memory);	/* outside link cache now. */
     } else {
         /* even if we didn't find the link to remove, unlock the cache */
-#ifndef MEMENTO_SQUEEZE_BUILD
         gx_monitor_leave(icc_link_cache->lock);
-#endif
     }
 }
 
@@ -763,9 +734,8 @@ gsicc_get_link(const gs_gstate *pgs1, gx_device *dev_in,
                             render_cond.black_point_comp & gsBP_MASK;
 
                             return gsicc_get_link_profile(pgs, dev, gs_input_profile,
-                                                          dev_profile->device_profile[0],
-                                                          rendering_params, memory,
-                                                          false);
+                                             dev_profile->device_profile[GS_DEFAULT_DEVICE_PROFILE],
+                                             rendering_params, memory, false);
                     }
                 } else {
                     /* In this case we may be wanting for a "unmanaged color"
@@ -784,7 +754,7 @@ gsicc_get_link(const gs_gstate *pgs1, gx_device *dev_in,
                         /* Set the identity case if we are in that situation */
                         if (link != NULL) {
                             if (gs_input_profile->num_comps ==
-                                dev_profile->device_profile[0]->num_comps) {
+                                dev_profile->device_profile[GS_DEFAULT_DEVICE_PROFILE]->num_comps) {
                                 link->is_identity = true;
                             }
                             return link;
@@ -805,18 +775,18 @@ gsicc_get_link(const gs_gstate *pgs1, gx_device *dev_in,
            an override setting.  Also, only use the blackpoint if overide_bp
            is set. Note that this can conflict with intents set from the source
            objects so the user needs to understand what options to set. */
-        gs_color_space_index index =
-                            gsicc_get_default_type(gs_input_profile);
         code = dev_proc(dev, get_profile)(dev,  &dev_profile);
+        if (code < 0)
+            return NULL;
+
         /* Check for unmanaged color case */
-        if (index < gs_color_space_index_DevicePixel &&
-            dev_profile->usefastcolor) {
+        if (gsicc_use_fast_color(gs_input_profile) > 0 && dev_profile->usefastcolor) {
             /* Return a "link" from the source space to the device color space */
             gsicc_link_t *link = gsicc_nocm_get_link(pgs, dev,
                                                      gs_input_profile->num_comps);
             if (link != NULL) {
                 if (gs_input_profile->num_comps ==
-                    dev_profile->device_profile[0]->num_comps) {
+                    dev_profile->device_profile[GS_DEFAULT_DEVICE_PROFILE]->num_comps) {
                     link->is_identity = true;
                 }
                 return link;
@@ -874,9 +844,7 @@ gsicc_alloc_link_entry(gsicc_link_cache_t *icc_link_cache,
     *ret_link = NULL;
     /* First see if we can add a link */
     /* TODO: this should be based on memory usage, not just num_links */
-#ifndef MEMENTO_SQUEEZE_BUILD
     gx_monitor_enter(icc_link_cache->lock);
-#endif
     while (icc_link_cache->num_links >= ICC_CACHE_MAXLINKS) {
         /* Look through the cache for first zero ref count to re-use that entry.
            When ref counts go to zero, the icc_link will have been moved to
@@ -890,19 +858,17 @@ gsicc_alloc_link_entry(gsicc_link_cache_t *icc_link_cache,
         while (link != NULL ) {
             if (link->ref_count == 0) {
                 /* we will use this one */
-                if_debug3m('^', cache_mem, "[^]%s 0x%lx ++ => %d\n",
-                           "icclink", (ulong)link, link->ref_count);
+                if_debug3m('^', cache_mem, "[^]%s "PRI_INTPTR" ++ => %d\n",
+                           "icclink", (intptr_t)link, link->ref_count);
                 break;
             }
             link = link->next;
         }
         if (link == NULL) {
-#ifndef MEMENTO_SQUEEZE_BUILD
             icc_link_cache->cache_full = true;
             /* unlock while waiting for a link to come available */
             gx_monitor_leave(icc_link_cache->lock);
             gx_semaphore_wait(icc_link_cache->full_wait);
-#endif
             /* repeat the findcachelink to see if some other thread has	*/
             /* already started building the link we need		*/
             *ret_link = gsicc_findcachelink(hash, icc_link_cache,
@@ -911,9 +877,7 @@ gsicc_alloc_link_entry(gsicc_link_cache_t *icc_link_cache,
             if (*ret_link != NULL)
                 return true;
 
-#ifndef MEMENTO_SQUEEZE_BUILD
             gx_monitor_enter(icc_link_cache->lock);	    /* restore the lock */
-#endif
         } else {
             /* Remove the zero ref_count link profile we found.		*/
             /* Even if we remove this link, we may still be maxed out so*/
@@ -934,10 +898,8 @@ gsicc_alloc_link_entry(gsicc_link_cache_t *icc_link_cache,
         icc_link_cache->head = *ret_link;
         icc_link_cache->num_links++;
     }
-#ifndef MEMENTO_SQUEEZE_BUILD
     /* unlock before returning */
     gx_monitor_leave(icc_link_cache->lock);
-#endif
     return false;	/* we didn't find it, but return a link to be filled */
 }
 
@@ -1011,8 +973,8 @@ gsicc_get_link_profile(const gs_gstate *pgs, gx_device *dev,
     /* Got a hit, return link (ref_count for the link was already bumped */
     if (found_link != NULL) {
         if_debug2m(gs_debug_flag_icc, memory,
-                   "[icc] Found Link = 0x%p, hash = %lld \n",
-                   found_link, (long long)hash.link_hashcode);
+                   "[icc] Found Link = "PRI_INTPTR", hash = %lld \n",
+                   (intptr_t)found_link, (long long)hash.link_hashcode);
         if_debug2m(gs_debug_flag_icc, memory,
                    "[icc] input_numcomps = %d, input_hash = %lld \n",
                    gs_input_profile->num_comps,
@@ -1054,7 +1016,7 @@ gsicc_get_link_profile(const gs_gstate *pgs, gx_device *dev,
             /* Set the identity case if we are in that situation */
             if (link != NULL) {
                 if (gs_input_profile->num_comps ==
-                    dev_profile->device_profile[0]->num_comps) {
+                    dev_profile->device_profile[GS_DEFAULT_DEVICE_PROFILE]->num_comps) {
                     link->is_identity = true;
                 }
                 return link;
@@ -1154,10 +1116,8 @@ gsicc_get_link_profile(const gs_gstate *pgs, gx_device *dev,
                                                     proof_profile->buffer_size,
                                                     memory);
                 proof_profile->profile_handle = cms_proof_profile;
-#if !defined(MEMENTO_SQUEEZE_BUILD)
                 if (!gscms_is_threadsafe())
                     gx_monitor_enter(proof_profile->lock);
-#endif
             } else {
                 /* Cant create the link */
                 gsicc_remove_link(link, cache_mem);
@@ -1174,10 +1134,8 @@ gsicc_get_link_profile(const gs_gstate *pgs, gx_device *dev,
                                                     devlink_profile->buffer_size,
                                                     memory);
                 devlink_profile->profile_handle = cms_devlink_profile;
-#if !defined(MEMENTO_SQUEEZE_BUILD)
                 if (!gscms_is_threadsafe())
                     gx_monitor_enter(devlink_profile->lock);
-#endif
             } else {
                 /* Cant create the link */
                 gsicc_remove_link(link, cache_mem);
@@ -1185,7 +1143,6 @@ gsicc_get_link_profile(const gs_gstate *pgs, gx_device *dev,
             }
         }
     }
-#if !defined(MEMENTO_SQUEEZE_BUILD)
     /* Profile reading of same structure not thread safe in CMM */
     if (!gscms_is_threadsafe()) {
         gx_monitor_enter(gs_input_profile->lock);
@@ -1193,7 +1150,6 @@ gsicc_get_link_profile(const gs_gstate *pgs, gx_device *dev,
             gx_monitor_enter(gs_output_profile->lock);
         }
     }
-#endif
     /* We may have to worry about special handling for DeviceGray to
        DeviceCMYK to ensure that Gray is mapped to K only.  This is only
        done once and then it is cached and the link used.  Note that Adobe
@@ -1235,7 +1191,6 @@ gsicc_get_link_profile(const gs_gstate *pgs, gx_device *dev,
                                                    rendering_params,
                                                    src_dev_link, cms_flags,
                                                    cache_mem->non_gc_memory);
-#if !defined(MEMENTO_SQUEEZE_BUILD)
     if (!gscms_is_threadsafe()) {
         if (include_softproof) {
             gx_monitor_leave(proof_profile->lock);
@@ -1244,20 +1199,17 @@ gsicc_get_link_profile(const gs_gstate *pgs, gx_device *dev,
             gx_monitor_leave(devlink_profile->lock);
         }
     }
-#endif
     } else {
         link_handle = gscms_get_link(cms_input_profile, cms_output_profile,
                                      rendering_params, cms_flags,
                                      cache_mem->non_gc_memory);
     }
-#if !defined(MEMENTO_SQUEEZE_BUILD)
     if (!gscms_is_threadsafe()) {
         if (!src_dev_link) {
             gx_monitor_leave(gs_output_profile->lock);
         }
         gx_monitor_leave(gs_input_profile->lock);
     }
-#endif
     if (link_handle != NULL) {
         if (gs_input_profile->data_cs == gsGRAY)
             pageneutralcolor = false;
@@ -1266,8 +1218,8 @@ gsicc_get_link_profile(const gs_gstate *pgs, gx_device *dev,
                             include_softproof, include_devicelink, pageneutralcolor,
                             gs_input_profile->data_cs);
         if_debug2m(gs_debug_flag_icc, cache_mem,
-                   "[icc] New Link = 0x%p, hash = %lld \n",
-                   link, (long long)hash.link_hashcode);
+                   "[icc] New Link = "PRI_INTPTR", hash = %lld \n",
+                   (intptr_t)link, (long long)hash.link_hashcode);
         if_debug2m(gs_debug_flag_icc, cache_mem,
                    "[icc] input_numcomps = %d, input_hash = %lld \n",
                    gs_input_profile->num_comps,
@@ -1281,16 +1233,14 @@ gsicc_get_link_profile(const gs_gstate *pgs, gx_device *dev,
         /* This could result in an infinite loop if other threads are waiting	*/
         /* for it to be made valid. (see gsicc_findcachelink).			*/
         link->ref_count--;	/* this thread no longer using this link entry	*/
-        if_debug2m('^', link->memory, "[^]icclink 0x%p -- => %d\n",
-                   link, link->ref_count);
+        if_debug2m('^', link->memory, "[^]icclink "PRI_INTPTR" -- => %d\n",
+                   (intptr_t)link, link->ref_count);
 
-#ifndef MEMENTO_SQUEEZE_BUILD
         if (icc_link_cache->cache_full) {
             icc_link_cache->cache_full = false;
             gx_semaphore_signal(icc_link_cache->full_wait);	/* let a waiting thread run */
         }
         gx_monitor_leave(link->lock);
-#endif
         gsicc_remove_link(link, cache_mem);
         return NULL;
     }
@@ -1530,8 +1480,8 @@ gsicc_support_named_color(const gs_color_space *pcs, const gs_gstate *pgs)
     int k, code, i, num_comp, num_spots=0, num_process=0, num_other=0;
     gs_color_space_index type = gs_color_space_get_index(pcs);
     char **names = NULL;
-    byte *pname;
-    uint name_size;
+    byte *pname = NULL; /* Silence compiler warning */
+    uint name_size = 0; /* Silence compiler warning */
     bool is_supported;
 
     /* Get the data for the named profile */
@@ -1553,6 +1503,7 @@ gsicc_support_named_color(const gs_color_space *pcs, const gs_gstate *pgs)
         num_comp = pcs->params.device_n.num_components;
     } else if (type == gs_color_space_index_Separation) {
         pname = (byte *)pcs->params.separation.sep_name;
+        name_size = strlen(pcs->params.separation.sep_name);
         num_comp = 1;
     } else
         return false;
@@ -1563,9 +1514,6 @@ gsicc_support_named_color(const gs_color_space *pcs, const gs_gstate *pgs)
             pname = (byte *)names[i];
             name_size = strlen(names[i]);
         }
-        else {
-            name_size = strlen(pcs->params.separation.sep_name);
-        }
 
         /* Classify */
         if (strncmp((char *)pname, "None", name_size) == 0 ||
@@ -1778,11 +1726,9 @@ gsicc_release_link(gsicc_link_t *icclink)
 
     icc_link_cache = icclink->icc_link_cache;
 
-#ifndef MEMENTO_SQUEEZE_BUILD
     gx_monitor_enter(icc_link_cache->lock);
-#endif
-    if_debug2m('^', icclink->memory, "[^]icclink 0x%p -- => %d\n",
-               icclink, icclink->ref_count - 1);
+    if_debug2m('^', icclink->memory, "[^]icclink "PRI_INTPTR" -- => %d\n",
+               (intptr_t)icclink, icclink->ref_count - 1);
     /* Decrement the reference count */
     if (--(icclink->ref_count) == 0) {
 
@@ -1818,17 +1764,13 @@ gsicc_release_link(gsicc_link_t *icclink)
             prev->next = icclink;
             icclink->next = curr;
         }
-#ifndef MEMENTO_SQUEEZE_BUILD
         /* Finally, if some thread was waiting because the cache was full, let it run */
         if (icc_link_cache->cache_full) {
             icc_link_cache->cache_full = false;
             gx_semaphore_signal(icc_link_cache->full_wait);	/* let a waiting thread run */
         }
-#endif
     }
-#ifndef MEMENTO_SQUEEZE_BUILD
     gx_monitor_leave(icc_link_cache->lock);
-#endif
 }
 
 /* Used to initialize the buffer description prior to color conversion */
@@ -1847,9 +1789,11 @@ gsicc_init_buffer(gsicc_bufferdesc_t *buffer_desc, unsigned char num_chan, unsig
     buffer_desc->num_rows = num_rows;
     buffer_desc->pixels_per_row = pixels_per_row;
 
-    /* sample endianess is consistent across platforms */
+#if ARCH_IS_BIG_ENDIAN
+    buffer_desc->little_endian = false;
+#else
     buffer_desc->little_endian = true;
-
+#endif
 }
 
 /* Return the proper component numbers based upon the profiles of the device.
@@ -1859,7 +1803,7 @@ int
 gsicc_get_device_profile_comps(const cmm_dev_profile_t *dev_profile)
 {
     if (dev_profile->link_profile == NULL) {
-       return dev_profile->device_profile[0]->num_comps;
+       return dev_profile->device_profile[GS_DEFAULT_DEVICE_PROFILE]->num_comps;
     } else {
        return dev_profile->link_profile->num_comps_out;
     }
