@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -18,15 +18,9 @@
 #ifndef gsicccache_INCLUDED
 #  define gsicccache_INCLUDED
 
-#ifndef gs_gstate_DEFINED
-#  define gs_gstate_DEFINED
-typedef struct gs_gstate_s gs_gstate;
-#endif
-
-#ifndef gx_device_DEFINED
-#  define gx_device_DEFINED
-typedef struct gx_device_s gx_device;
-#endif
+#include "gsgstate.h"
+#include "gscms.h"
+#include "gxcvalue.h"
 
 /* Used in named color handling */
 typedef struct gsicc_namedcolor_s {
@@ -67,6 +61,7 @@ int gsicc_transform_named_color(const float tint_values[],
                             const gs_gstate *pgs, gx_device *dev,
                             cmm_profile_t *gs_output_profile,
                             gsicc_rendering_param_t *rendering_params);
+bool gsicc_support_named_color(const gs_color_space *pcs, const gs_gstate *pgs);
 int  gsicc_get_device_profile_comps(const cmm_dev_profile_t *dev_profile);
 gsicc_link_t * gsicc_alloc_link_dev(gs_memory_t *memory, cmm_profile_t *src_profile,
     cmm_profile_t *des_profile, gsicc_rendering_param_t *rendering_params);
