@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -29,9 +29,7 @@
 #include "cal.h"
 #endif
 
-#ifndef MEMENTO_SQUEEZE_BUILD
 #define USE_LCMS2_LOCKING
-#endif
 
 #ifdef USE_LCMS2_LOCKING
 #include "gxsync.h"
@@ -877,6 +875,7 @@ gscms_create(gs_memory_t *memory)
 
 #ifdef WITH_CAL
     cmsPlugin(ctx, cal_cms_extensions());
+    cmsPlugin(ctx, cal_cms_extensions2());
 #endif
 
     cmsSetLogErrorHandler(ctx, gscms_error);
