@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -114,8 +114,8 @@ gsicc_add_cs(gs_gstate * pgs, gs_color_space * colorspace, uint64_t dictkey)
     rc_increment(colorspace);
     result->key = dictkey;
     if_debug2m(gs_debug_flag_icc, memory,
-               "[icc] Add cs to cache = 0x%p, hash = %"PRIu64"\n",
-               result->color_space, (uint64_t)result->key);
+               "[icc] Add cs to cache = "PRI_INTPTR", hash = %"PRIu64"\n",
+               (intptr_t)result->color_space, (uint64_t)result->key);
     profile_cache->num_entries++;
 }
 
@@ -129,8 +129,8 @@ gsicc_find_cs(uint64_t key_test, gs_gstate * pgs)
     while (curr != NULL ){
         if (curr->key == key_test){
             if_debug2m(gs_debug_flag_icc, pgs->memory,
-                       "[icc] Found cs = 0x%p, hash = %"PRIu64"\n",
-                       curr->color_space, (uint64_t)curr->key);
+                       "[icc] Found cs = "PRI_INTPTR", hash = %"PRIu64"\n",
+                       (intptr_t)curr->color_space, (uint64_t)curr->key);
             /* If not already at head of list, move this one there */
             if (curr != profile_cache->head) {
                 /* We need to move found one to the top of the list. */
@@ -182,8 +182,8 @@ gsicc_remove_cs_entry(gsicc_profile_cache_t *profile_cache)
     /* If found again in the source document, it will be regenerated
        and added back into the cache. */
     if_debug2m(gs_debug_flag_icc, memory,
-               "[icc] Remove cs from cache = 0x%p, hash = %"PRIu64"\n",
-               curr->color_space, (uint64_t)curr->key);
+               "[icc] Remove cs from cache = "PRI_INTPTR", hash = %"PRIu64"\n",
+               (intptr_t)curr->color_space, (uint64_t)curr->key);
     rc_decrement(curr->color_space, "gsicc_remove_cs_entry");
     gs_free_object(memory->stable_memory, curr, "gsicc_remove_cs_entry");
 }
