@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -66,7 +66,9 @@ const gx_io_device gs_iodev_ram = {
         ram_delete, ram_rename, ram_status,
         ram_enumerate_init, ram_enumerate_next, ram_enumerate_close,
         ram_get_params, iodev_no_put_params
-    }
+    },
+    NULL,
+    NULL
 };
 
 typedef struct ramfs_state_s {
@@ -111,8 +113,8 @@ static int
 s_ram_switch(stream *, bool);
 
 static int
-ramfs_errno_to_code(int errno) {
-    switch (errno) {
+ramfs_errno_to_code(int error_number) {
+    switch (error_number) {
     case RAMFS_NOTFOUND:
         return_error(gs_error_undefinedfilename);
     case RAMFS_NOACCESS:
@@ -218,7 +220,7 @@ sread_ram(register stream * s, ramhandle * file, byte * buf, uint len)
     };
 
     s_std_init(s, buf, len, &p,s_mode_read + s_mode_seek);
-    s->file = (FILE*)file;
+    s->file = (gp_file *)file;
     s->file_modes = s->modes;
     s->file_offset = 0;
     ramfile_seek(file, 0, RAMFS_SEEK_END);
@@ -310,7 +312,7 @@ swrite_ram(register stream * s, ramhandle * file, byte * buf, uint len)
     };
 
     s_std_init(s, buf, len, &p, s_mode_write + s_mode_seek);
-    s->file = (FILE*)file;
+    s->file = (gp_file *)file;
     s->file_modes = s->modes;
     s->file_offset = 0;        /* in case we switch to reading later */
     s->file_limit = S_FILE_LIMIT_MAX;
@@ -485,8 +487,8 @@ ram_status(gx_io_device * iodev, const char *fname, struct stat *pstat)
 }
 
 static file_enum *
-ram_enumerate_init(gx_io_device *iodev, const char *pat, uint patlen,
-    gs_memory_t *mem)
+ram_enumerate_init(gs_memory_t * mem, gx_io_device *iodev, const char *pat,
+                   uint patlen)
 {
     gsram_enum * penum = gs_alloc_struct(
     mem, gsram_enum, &st_gsram_enum,
@@ -495,6 +497,7 @@ ram_enumerate_init(gx_io_device *iodev, const char *pat, uint patlen,
     char *pattern = (char *)gs_alloc_bytes(
     mem, patlen+1, "ram_enumerate_file_init(pattern)"
     );
+
     ramfs_enum * e = ramfs_enum_new(GETRAMFS(iodev->state));
     if(penum && pattern && e) {
     memcpy(pattern, pat, patlen);
@@ -513,18 +516,19 @@ ram_enumerate_init(gx_io_device *iodev, const char *pat, uint patlen,
 }
 
 static void
-ram_enumerate_close(file_enum *pfen)
+ram_enumerate_close(gs_memory_t * mem, file_enum *pfen)
 {
     gsram_enum *penum = (gsram_enum *)pfen;
-    gs_memory_t *mem = penum->memory;
+    gs_memory_t *mem2 = penum->memory;
+    (void)mem;
 
     ramfs_enum_end(penum->e);
-    gs_free_object(mem, penum->pattern, "ramfs_enum_init(pattern)");
-    gs_free_object(mem, penum, "ramfs_enum_init(ramfs_enum)");
+    gs_free_object(mem2, penum->pattern, "ramfs_enum_init(pattern)");
+    gs_free_object(mem2, penum, "ramfs_enum_init(ramfs_enum)");
 }
 
 static uint
-ram_enumerate_next(file_enum *pfen, char *ptr, uint maxlen)
+ram_enumerate_next(gs_memory_t * mem, file_enum *pfen, char *ptr, uint maxlen)
 {
     gsram_enum *penum = (gsram_enum *)pfen;
 
@@ -539,7 +543,7 @@ ram_enumerate_next(file_enum *pfen, char *ptr, uint maxlen)
     }
     }
     /* ran off end of list, close the enum */
-    ram_enumerate_close(pfen);
+    ram_enumerate_close(mem, pfen);
     return ~(uint)0;
 }
 
