@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -28,7 +28,9 @@ int gs_erasepage(gs_gstate *),
     gs_fillpage(gs_gstate *),
     gs_fill(gs_gstate *),
     gs_eofill(gs_gstate *),
-    gs_stroke(gs_gstate *);
+    gs_stroke(gs_gstate *),
+    gs_fillstroke(gs_gstate * pgs, int *restart),
+    gs_eofillstroke(gs_gstate * pgs, int *restart);
 
 /* Image tracing */
 int gs_imagepath(gs_gstate *, int, int, const byte *);
