@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -118,8 +118,11 @@ gs_pattern1_init(gs_pattern1_template_t * ppat)
 }
 
 /* Make an instance of a PatternType 1 pattern. */
-static int compute_inst_matrix(gs_pattern1_instance_t * pinst,
-        gs_gstate * saved, gs_rect * pbbox, int width, int height);
+static int compute_inst_matrix(gs_pattern1_instance_t *pinst,
+                               gs_gstate *saved,
+                               gs_rect *pbbox,
+                               int width, int height,
+                               float *bbw, float *bbh);
 int
 gs_makepattern(gs_client_color * pcc, const gs_pattern1_template_t * pcp,
                const gs_matrix * pmat, gs_gstate * pgs, gs_memory_t * mem)
@@ -145,6 +148,7 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
     int code = gs_make_pattern_common(pcc, (const gs_pattern_template_t *)pcp,
                                       pmat, pgs, mem,
                                       &st_pattern1_instance);
+    float bbw, bbh;
 
     if (code < 0)
         return code;
@@ -181,7 +185,7 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
             goto fsaved;
     }
     inst.templat = *pcp;
-    code = compute_inst_matrix(&inst, saved, &bbox, dev_width, dev_height);
+    code = compute_inst_matrix(&inst, saved, &bbox, dev_width, dev_height, &bbw, &bbh);
     if (code < 0)
         goto fsaved;
 
@@ -214,9 +218,6 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
     if_debug5m('t', mem, "[t]bbox=(%g,%g),(%g,%g), uses_transparency=%d\n",
                bbox.p.x, bbox.p.y, bbox.q.x, bbox.q.y, inst.templat.uses_transparency);
     {
-        float bbw = bbox.q.x - bbox.p.x;
-        float bbh = bbox.q.y - bbox.p.y;
-
         /* If the step and the size agree to within 1/2 pixel, */
         /* make them the same. */
         if (ADJUST_SCALE_BY_GS_TRADITION) {
@@ -235,7 +236,7 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
             bbox.p.x = bbox.p.y = bbox.q.x = bbox.q.y = 0;
         } else {
             /* Check for singular stepping matrix. */
-            if (fabs(inst.step_matrix.xx * inst.step_matrix.yy - inst.step_matrix.xy * inst.step_matrix.yx) < 1.0e-6) {
+            if (fabs(inst.step_matrix.xx * inst.step_matrix.yy - inst.step_matrix.xy * inst.step_matrix.yx) < 1.0e-9) {
                 code = gs_note_error(gs_error_rangecheck);
                 goto fsaved;
             }
@@ -247,7 +248,7 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
                 gs_scale(saved, fabs(inst.size.x / inst.step_matrix.xx),
                          fabs(inst.size.y / inst.step_matrix.yy));
                 code = compute_inst_matrix(&inst, saved, &bbox,
-                                                dev_width, dev_height);
+                                           dev_width, dev_height, &bbw, &bbh);
                 if (code < 0)
                     goto fsaved;
                 if (ADJUST_SCALE_FOR_THIN_LINES) {
@@ -529,14 +530,30 @@ clamp_pattern_bbox(gs_pattern1_instance_t * pinst, gs_rect * pbbox,
 /* from the step values and the saved matrix. */
 static int
 compute_inst_matrix(gs_pattern1_instance_t * pinst, gs_gstate * saved,
-                            gs_rect * pbbox, int width, int height)
+                    gs_rect * pbbox, int width, int height,
+                    float *pbbw, float *pbbh)
 {
     float xx, xy, yx, yy, dx, dy, temp;
     int code;
+    gs_matrix m = ctm_only(saved);
 
-    code = gs_bbox_transform(&pinst->templat.BBox, &ctm_only(saved), pbbox);
+    /* Bug 702124: Due to the limited precision of floats, we find that
+     * transforming (say) small height boxes in the presence of large tx/ty
+     * values can cause the box heights to map to 0. So calculate the
+     * width/height of the bbox before we roll the offset into it. */
+    m.tx = 0; m.ty = 0;
+
+    code = gs_bbox_transform(&pinst->templat.BBox, &m, pbbox);
     if (code < 0)
         return code;
+
+    *pbbw = pbbox->q.x - pbbox->p.x;
+    *pbbh = pbbox->q.y - pbbox->p.y;
+
+    pbbox->p.x += ctm_only(saved).tx;
+    pbbox->p.y += ctm_only(saved).ty;
+    pbbox->q.x += ctm_only(saved).tx;
+    pbbox->q.y += ctm_only(saved).ty;
     /*
      * Adjust saved.ctm to map the bbox origin to pixels.
      */
@@ -701,10 +718,14 @@ gs_pattern1_set_color(const gs_client_color * pcc, gs_gstate * pgs)
         pcs = pcs->base_space;
         return pcs->type->set_overprint(pcs, pgs);
     } else {
-        gs_overprint_params_t   params;
+        gs_overprint_params_t   params = {0};
 
         params.retain_any_comps = false;
-        pgs->effective_overprint_mode = 0;
+        params.effective_opm = pgs->color[0].effective_opm = 0;
+        params.op_state = OP_STATE_NONE;
+        params.is_fill_color = false;
+        params.idle = false;
+
         return gs_gstate_update_overprint(pgs, &params);
     }
 }
@@ -749,6 +770,15 @@ typedef struct pixmap_info_s {
     void (*free_proc)(gs_memory_t *, void *, client_name_t);
 } pixmap_info;
 
+void *
+gs_get_pattern_client_data(const gs_client_color * pcc)
+{
+    const gs_pattern_instance_t *pinst = pcc->pattern;
+
+    return (pinst == 0 || pinst->type != &gs_pattern1_type ? 0 :
+            (void *)pinst->client_data);
+}
+
 gs_private_st_suffix_add1(st_pixmap_info,
                           pixmap_info,
                           "pixmap info. struct",
@@ -761,26 +791,16 @@ gs_private_st_suffix_add1(st_pixmap_info,
 #define st_pixmap_info_max_ptrs (1 + st_tile_bitmap_max_ptrs)
 
 /*
- *  Free routine for pattern instances created from pixmaps. This overwrites
- *  the free procedure originally stored in the pattern instance, and stores
- *  the pointer to that procedure in the pixmap_info structure. This procedure
- *  will call the original procedure, then free the pixmap_info structure.
+ *  Free routine for pattern instances created from pixmaps.
  *
  *  Note that this routine does NOT release the data in the original pixmap;
  *  that remains the responsibility of the client.
  */
-static void
-free_pixmap_pattern(
-    gs_memory_t *           pmem,
-    void *                  pvpinst,
-    client_name_t           cname
-)
+static void pixmap_free_notify (gs_memory_t * mem, void *vpinst)
 {
-    gs_pattern1_instance_t *pinst = (gs_pattern1_instance_t *)pvpinst;
-    pixmap_info *ppmap = pinst->templat.client_data;
+    gs_pattern1_instance_t *pinst = (gs_pattern1_instance_t *)vpinst;
 
-    ppmap->free_proc(pmem, pvpinst, cname);
-    gs_free_object(pmem, ppmap, cname);
+    gs_free_object(mem, pinst->client_data, "pixmap_free_notify");
 }
 
 /*
@@ -791,10 +811,10 @@ static int bitmap_paint(gs_image_enum * pen, gs_data_image_t * pim,
 static int
 mask_PaintProc(const gs_client_color * pcolor, gs_gstate * pgs)
 {
-    const pixmap_info *ppmap = gs_getpattern(pcolor)->client_data;
+    int code;
+    const pixmap_info *ppmap = (pixmap_info *)gs_get_pattern_client_data(pcolor);
     const gs_depth_bitmap *pbitmap = &(ppmap->bitmap);
-    gs_image_enum *pen =
-    gs_image_enum_alloc(gs_gstate_memory(pgs), "mask_PaintProc");
+    gs_image_enum *pen = gs_image_enum_alloc(gs_gstate_memory(pgs), "mask_PaintProc");
     gs_image1_t mask;
 
     if (pen == 0)
@@ -803,12 +823,14 @@ mask_PaintProc(const gs_client_color * pcolor, gs_gstate * pgs)
     mask.Width = pbitmap->size.x;
     mask.Height = pbitmap->size.y;
     gs_image_init(pen, &mask, false, false, pgs);
-    return bitmap_paint(pen, (gs_data_image_t *) & mask, pbitmap, pgs);
+    code = bitmap_paint(pen, (gs_data_image_t *) & mask, pbitmap, pgs);
+    gs_free_object(gs_gstate_memory(pgs), pen, "mask_PaintProc");
+    return code;
 }
 static int
 image_PaintProc(const gs_client_color * pcolor, gs_gstate * pgs)
 {
-    const pixmap_info *ppmap = gs_getpattern(pcolor)->client_data;
+    const pixmap_info *ppmap = gs_get_pattern_client_data(pcolor);
     const gs_depth_bitmap *pbitmap = &(ppmap->bitmap);
     gs_image_enum *pen =
         gs_image_enum_alloc(gs_gstate_memory(pgs), "image_PaintProc");
@@ -846,8 +868,10 @@ image_PaintProc(const gs_client_color * pcolor, gs_gstate * pgs)
     if (code < 0)
         goto fail;
     code = gs_setcolorspace(pgs, pcspace);
-    if (code < 0)
-        return code;
+    if (code < 0) {
+        gs_grestore(pgs);
+        goto fail;
+    }
     if (transparent)
         gs_image4_t_init( (gs_image4_t *) &image, pcspace);
     else
@@ -877,8 +901,11 @@ image_PaintProc(const gs_client_color * pcolor, gs_gstate * pgs)
                                      (gs_data_image_t *)&image,
                                      pgs )) >= 0 &&
         (code = bitmap_paint(pen, (gs_data_image_t *) & image, pbitmap, pgs)) >= 0) {
+        gs_free_object(gs_gstate_memory(pgs), pen, "image_PaintProc");
         return gs_grestore(pgs);
     }
+    /* Failed above, need to undo the gsave */
+    gs_grestore(pgs);
 
 fail:
     gs_free_object(gs_gstate_memory(pgs), pen, "image_PaintProc");
@@ -901,7 +928,7 @@ bitmap_paint(gs_image_enum * pen, gs_data_image_t * pim,
     else
         for (n = pim->Height; n > 0 && code >= 0; dp += raster, --n)
             code = gs_image_next(pen, dp, nbytes, &used);
-    code1 = gs_image_cleanup_and_free_enum(pen, pgs);
+    code1 = gs_image_cleanup(pen, pgs);
     if (code >= 0 && code1 < 0)
         code = code1;
     return code;
@@ -918,7 +945,7 @@ int pixmap_high_level_pattern(gs_gstate * pgs)
     gs_color_space *pcs;
     gs_pattern1_instance_t *pinst =
         (gs_pattern1_instance_t *)gs_currentcolor(pgs)->pattern;
-    const pixmap_info *ppmap = ppat->client_data;
+    const pixmap_info *ppmap = (const pixmap_info *)gs_get_pattern_client_data((const gs_client_color *)&pdc->ccolor);
 
     code = gx_pattern_cache_add_dummy_entry(pgs, pinst, pgs->device->color_info.depth);
     if (code < 0)
@@ -964,13 +991,17 @@ int pixmap_high_level_pattern(gs_gstate * pgs)
         code = image_PaintProc(&pdc->ccolor, pgs);
     else {
         pcs = gs_cspace_new_DeviceGray(pgs->memory);
-        if (pcs == NULL)
+        if (pcs == NULL) {
+            gs_grestore(pgs);
             return_error(gs_error_VMerror);
+        }
         gs_setcolorspace(pgs, pcs);
         code = mask_PaintProc(&pdc->ccolor, pgs);
     }
-    if (code < 0)
+    if (code < 0) {
+        gs_grestore(pgs);
         return code;
+    }
 
     code = gs_grestore(pgs);
     if (code < 0)
@@ -1107,7 +1138,6 @@ gs_makepixmappattern(
     pat.XStep = (float)pbitmap->size.x;
     pat.YStep = (float)pbitmap->size.y;
     pat.PaintProc = (mask ? pixmap_remap_mask_pattern : pixmap_remap_image_pattern);
-    pat.client_data = ppmap;
 
     /* set the ctm to be the identity */
     gs_currentmatrix(pgs, &smat);
@@ -1133,9 +1163,8 @@ gs_makepixmappattern(
         if (!mask && (white_index >= (1 << pbitmap->pix_depth)))
             pinst->uses_mask = false;
 
-        /* overwrite the free procedure for the pattern instance */
-        ppmap->free_proc = pinst->rc.free;
-        pinst->rc.free = free_pixmap_pattern;
+        pinst->client_data = ppmap;
+        pinst->notify_free = pixmap_free_notify;
 
         /*
          * Since the PaintProcs don't reference the saved color space or
@@ -1620,6 +1649,7 @@ typedef struct gx_dc_serialized_tile_s {
 } gx_dc_serialized_tile_t;
 
 enum {
+    TILE_IS_LOCKED   = (int)0x80000000,
     TILE_HAS_OVERLAP = 0x40000000,
     TILE_IS_SIMPLE   = 0x20000000,
     TILE_USES_TRANSP = 0x10000000,
@@ -1638,9 +1668,9 @@ gx_dc_pattern_write_raster(gx_color_tile *ptile, int64_t offset, byte *data,
     int left = *psize;
     int64_t offset1 = offset;
 
-    size_b = sizeof(gx_strip_bitmap) +
+    size_b = (int)sizeof(gx_strip_bitmap) +
          ptile->tbits.size.y * ptile->tbits.raster * ptile->tbits.num_planes;
-    size_c = ptile->tmask.data ? sizeof(gx_strip_bitmap) + ptile->tmask.size.y * ptile->tmask.raster : 0;
+    size_c = ptile->tmask.data ? (int)sizeof(gx_strip_bitmap) + ptile->tmask.size.y * ptile->tmask.raster : 0;
     if (data == NULL) {
         *psize = sizeof(gx_dc_serialized_tile_t) + size_b + size_c;
         return 0;
@@ -1664,7 +1694,8 @@ gx_dc_pattern_write_raster(gx_color_tile *ptile, int64_t offset, byte *data,
         buf.flags = ptile->depth
                   | (ptile->tiling_type<<TILE_TYPE_SHIFT)
                   | (ptile->is_simple ? TILE_IS_SIMPLE : 0)
-                  | (ptile->has_overlap ? TILE_HAS_OVERLAP : 0);
+                  | (ptile->has_overlap ? TILE_HAS_OVERLAP : 0)
+                  | (ptile->is_locked ? TILE_IS_LOCKED : 0);
         if (sizeof(buf) > left) {
             /* For a while we require the client to provide enough buffer size. */
             return_error(gs_error_unregistered); /* Must not happen. */
@@ -1756,7 +1787,8 @@ gx_dc_pattern_trans_write_raster(gx_color_tile *ptile, int64_t offset, byte *dat
                   | TILE_USES_TRANSP
                   | (ptile->tiling_type<<TILE_TYPE_SHIFT)
                   | (ptile->is_simple ? TILE_IS_SIMPLE : 0)
-                  | (ptile->has_overlap ? TILE_HAS_OVERLAP : 0);
+                  | (ptile->has_overlap ? TILE_HAS_OVERLAP : 0)
+                  | (ptile->is_locked ? TILE_IS_LOCKED : 0);
         buf.step_matrix = ptile->step_matrix;
         buf.bbox = ptile->bbox;
         buf.blending_mode = ptile->blending_mode;
@@ -1881,6 +1913,7 @@ gx_dc_pattern_write(
                   | (ptile->tiling_type<<TILE_TYPE_SHIFT)
                   | (ptile->is_simple ? TILE_IS_SIMPLE : 0)
                   | (ptile->has_overlap ? TILE_HAS_OVERLAP : 0)
+                  | (ptile->is_locked ? TILE_IS_LOCKED : 0)
                   | (ptile->cdev->common.page_uses_transparency ? TILE_USES_TRANSP : 0);
         buf.blending_mode = ptile->blending_mode;    /* in case tile has transparency */
         if (sizeof(buf) > left) {
@@ -2039,6 +2072,7 @@ gx_dc_pattern_read(
     int code, l;
     tile_trans_clist_info_t trans_info = { { { 0 } } };
     int cache_space_needed;
+    bool deep = device_is_deep(dev);
 
     if (offset == 0) {
         pdevc->mask.id = gx_no_bitmap_id;
@@ -2087,8 +2121,22 @@ gx_dc_pattern_read(
             /* the following works for raster or clist patterns */
             cache_space_needed = buf.size_b + buf.size_c;
         }
+
+        /* Free up any unlocked patterns if needed */
         gx_pattern_cache_ensure_space((gs_gstate *)pgs, cache_space_needed);
 
+        /* If the pattern tile is already in the cache, make sure it isn't locked */
+        /* The lock will be reset below, but the read logic needs to finish loading the pattern. */
+        ptile = &(pgs->pattern_cache->tiles[buf.id % pgs->pattern_cache->num_tiles]);
+        if (ptile->id != gs_no_id && ptile->is_locked) {
+            /* we shouldn't have miltiple tiles locked, but check if OK before unlocking */
+            if (ptile->id != buf.id)
+                return_error(gs_error_unregistered);	/* can't unlock some other tile in this slot */
+            code = gx_pattern_cache_entry_set_lock((gs_gstate *)pgs, buf.id, false);        /* make sure not locked */
+            if (code < 0)
+                return code;	/* can't happen since we call ensure_space above, but Coverity doesn't know that */
+        }
+        /* get_entry will free the tile in the cache slot if it isn't empty */
         code = gx_pattern_cache_get_entry((gs_gstate *)pgs, /* Break 'const'. */
                         buf.id, &ptile);
         if (code < 0)
@@ -2105,8 +2153,9 @@ gx_dc_pattern_read(
         ptile->tiling_type = (buf.flags & TILE_TYPE_MASK)>>TILE_TYPE_SHIFT;
         ptile->is_simple = !!(buf.flags & TILE_IS_SIMPLE);
         ptile->has_overlap = !!(buf.flags & TILE_HAS_OVERLAP);
+        ptile->is_locked = !!(buf.flags & TILE_IS_LOCKED);
         ptile->blending_mode = buf.blending_mode;
-        ptile->is_dummy = 0;
+        ptile->is_dummy = false;
 
         if (!(buf.flags & TILE_IS_CLIST)) {
 
@@ -2128,6 +2177,7 @@ gx_dc_pattern_read(
                 ptile->ttrans->rect.q.y = trans_info.rect.q.y;
                 ptile->ttrans->rowstride = trans_info.rowstride;
                 ptile->ttrans->width = trans_info.width;
+                ptile->ttrans->deep = deep;
                 pdevc->type = &gx_dc_pattern_trans;
                 if_debug2m('v', pgs->memory,
                            "[v*] Reading trans tile from clist into cache, uid = %ld id = %ld \n",
