@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -22,7 +22,6 @@
 #include "gsutil.h"             /* for gs_next_ids */
 #include "gzstate.h"
 #include "gxcspace.h"           /* here for gscolor2.h */
-#include "gsalpha.h"
 #include "gscolor2.h"
 #include "gscoord.h"            /* for gs_initmatrix */
 #include "gscie.h"
@@ -37,8 +36,10 @@
 #include "gzcpath.h"
 #include "gsovrc.h"
 #include "gxcolor2.h"
+#include "gscolor3.h" /* for gs_smoothness() */
 #include "gxpcolor.h"
 #include "gsicc_manage.h"
+#include "gxdevsop.h"
 
 /* Forward references */
 static gs_gstate *gstate_alloc(gs_memory_t *, client_name_t,
@@ -232,7 +233,9 @@ gs_gstate_alloc(gs_memory_t * mem)
     code = gs_nulldevice(pgs);
     if (code < 0)
         goto fail;
-    gs_setalpha(pgs, 1.0);
+    gs_setfillconstantalpha(pgs, 1.0);
+    gs_setstrokeconstantalpha(pgs, 1.0);
+    gs_setalphaisshape(pgs, false);
     gs_settransfer(pgs, gs_identity_transfer);
     gs_setflat(pgs, 1.0);
     gs_setfilladjust(pgs, 0.3, 0.3);
@@ -316,8 +319,8 @@ gs_gsave(gs_gstate * pgs)
     if (pgs->show_gstate == pgs)
         pgs->show_gstate = pnew->show_gstate = pnew;
     pgs->level++;
-    if_debug2m('g', pgs->memory, "[g]gsave -> 0x%lx, level = %d\n",
-              (ulong) pnew, pgs->level);
+    if_debug2m('g', pgs->memory, "[g]gsave -> "PRI_INTPTR", level = %d\n",
+              (intptr_t)pnew, pgs->level);
     return 0;
 }
 
@@ -351,6 +354,14 @@ gs_gsave_for_save(gs_gstate * pgs, gs_gstate ** psaved)
     /* Cut the stack so we can't grestore past here. */
     *psaved = pgs->saved;
     pgs->saved = 0;
+
+    code = gs_gsave(pgs);
+    if (code < 0) {
+        pgs->saved = *psaved;
+        *psaved = NULL;
+        gs_grestore(pgs);
+        return code;
+    }
     return code;
 fail:
     if (new_cpath)
@@ -366,10 +377,9 @@ gs_grestore_only(gs_gstate * pgs)
     gs_gstate tmp_gstate;
     void *pdata = pgs->client_data;
     void *sdata;
-    bool prior_overprint = pgs->overprint;
 
-    if_debug2m('g', pgs->memory, "[g]grestore 0x%lx, level was %d\n",
-               (ulong) saved, pgs->level);
+    if_debug2m('g', pgs->memory, "[g]grestore "PRI_INTPTR", level was %d\n",
+               (intptr_t)saved, pgs->level);
     if (!saved)
         return 1;
     sdata = saved->client_data;
@@ -388,11 +398,6 @@ gs_grestore_only(gs_gstate * pgs)
     *saved = tmp_gstate;            /* restore "freed" state (pointers zeroed after contents freed) */
     gs_free_object(pgs->memory, saved, "gs_grestore");
 
-    /* update the overprint compositor, if necessary */
-    if (prior_overprint || pgs->overprint)
-    {
-        return gs_do_set_overprint(pgs);
-    }
     return 0;
 }
 
@@ -521,12 +526,7 @@ gs_setgstate(gs_gstate * pgs, const gs_gstate * pfrom)
     pgs->view_clip = view_clip;
     pgs->show_gstate =
         (pgs->show_gstate == pfrom ? pgs : saved_show);
-
-    /* update the overprint compositor, unconditionally. Unlike grestore, this */
-    /* may skip over states where overprint was set, so the prior state can    */
-    /* not be relied on to avoid this call. setgstate is not as commonly used  */
-    /* as grestore, so the overhead of the compositor call is acceptable.      */
-    return(gs_do_set_overprint(pgs));
+    return 0;
 }
 
 /* Get the allocator pointer of a graphics state. */
@@ -593,8 +593,11 @@ gs_gstate_update_overprint(gs_gstate * pgs, const gs_overprint_params_t * pparam
                                                    pgs->memory,
                                                    NULL);
         if (code >= 0 || code == gs_error_handled){
-            if (ovptdev != dev)
+            if (ovptdev != dev) {
                 gx_set_device_only(pgs, ovptdev);
+                /* Get rid of extra reference */
+                rc_decrement(ovptdev, "gs_gstate_update_overprint(ovptdev)");
+            }
             code = 0;
         }
     }
@@ -632,28 +635,64 @@ gs_do_set_overprint(gs_gstate * pgs)
 
     if (cs_num_components(pcs) < 0 && pcc->pattern != 0)
         code = pcc->pattern->type->procs.set_color(pcc, pgs);
-    else
-    {
+    else {
+        gx_device* dev = pgs->device;
+        cmm_dev_profile_t* dev_profile;
+        gs_color_space_index pcs_index = gs_color_space_get_index(pcs);
+
+        dev_proc(dev, get_profile)(dev, &dev_profile);
+        if (!dev_profile->sim_overprint)
+            return code;
+
+        /* Transparency device that supports spots and where we have
+           sep or devicen colors needs special consideration if the device
+           is in a additive blend mode.  This could
+           be written more compactly, but it would be unreadable. */
+        if (dev_proc(dev, dev_spec_op)(dev, gxdso_pdf14_sep_device, NULL, 0) &&
+            (dev->color_info.polarity != GX_CINFO_POLARITY_SUBTRACTIVE) &&
+            (pcs_index == gs_color_space_index_DeviceN ||
+             pcs_index == gs_color_space_index_Separation)) {
+            if (pcs_index == gs_color_space_index_Separation) {
+                if (!(pcs->params.separation.color_type == SEP_MIX ||
+                      pcs->params.separation.color_type == SEP_ENUM)) {
+                    /* Sep color is not a spot color.  We can't do OP and trans */
+                    return code;
+                }
+            }
+            if (pcs_index == gs_color_space_index_DeviceN) {
+                if (pcs->params.device_n.color_type != SEP_PURE_SPOT) {
+                    /* DeviceN has process colors  We can't do OP and trans. */
+                    return code;
+                }
+            }
+
+        }
+
         /* The spaces that do not allow opm (e.g. ones that are not ICC or DeviceCMYK)
            will blow away any true setting later. But we have to be prepared
-           in case this is an CMYK ICC space for example. Hence we set effective mode
+           in case this is a CMYK ICC space for example. Hence we set effective mode
            to mode here (Bug 698721)*/
-        pgs->effective_overprint_mode = pgs->overprint_mode;
+        pgs->color[0].effective_opm = pgs->overprint_mode;
+
+        if_debug2m(gs_debug_flag_overprint, pgs->memory,
+            "[overprint] gs_do_set_overprint. Preset effective mode. pgs->color[0].effective_opm = %d pgs->color[1].effective_opm = %d\n",
+            pgs->color[0].effective_opm, pgs->color[1].effective_opm);
+
         pcs->type->set_overprint(pcs, pgs);
     }
     return code;
 }
 
-/* setoverprint */
+/* setoverprint (non-stroke case) interpreter code
+   ensures that this is called when appropriate. This
+   should only be coming when we are doing PS files.
+   As they don't have separate stroke and fill overprint
+   controls */
 void
 gs_setoverprint(gs_gstate * pgs, bool ovp)
 {
-    bool    prior_ovp = pgs->overprint;
-
     pgs->overprint = ovp;
     pgs->stroke_overprint = ovp;
-    if (prior_ovp != ovp)
-        (void)gs_do_set_overprint(pgs);
 }
 
 /* currentoverprint */
@@ -681,11 +720,7 @@ gs_currentstrokeoverprint(const gs_gstate * pgs)
 void
 gs_setfilloverprint(gs_gstate * pgs, bool ovp)
 {
-    bool    prior_ovp = pgs->overprint;
-
     pgs->overprint = ovp;
-    if (prior_ovp != ovp)
-        (void)gs_do_set_overprint(pgs);
 }
 
 /* currentstrokeoverprint */
@@ -699,15 +734,11 @@ gs_currentfilloverprint(const gs_gstate * pgs)
 int
 gs_setoverprintmode(gs_gstate * pgs, int mode)
 {
-    int     prior_mode = pgs->effective_overprint_mode;
-    int     code = 0;
-
     if (mode < 0 || mode > 1)
         return_error(gs_error_rangecheck);
     pgs->overprint_mode = mode;
-    if (pgs->overprint && prior_mode != mode)
-        code = gs_do_set_overprint(pgs);
-    return code;
+
+    return 0;
 }
 
 /* currentoverprintmode */
@@ -912,7 +943,46 @@ gs_initgraphics(gs_gstate * pgs)
     }
     pgs->in_cachedevice = 0;
 
+    code = gs_settextspacing(pgs, (double)0.0);
+    if (code < 0)
+        goto exit;
+    code = gs_settextleading(pgs, (double)0.0);
+    if (code < 0)
+        goto exit;
+
+    gs_settextrenderingmode(pgs, 0);
+
+    code = gs_setwordspacing(pgs, (double)0.0);
+    if (code < 0)
+        goto exit;
+    code = gs_settexthscaling(pgs, (double)100.0);
+    if (code < 0)
+        goto exit;
+
+    gs_setaccuratecurves(pgs, true);
+
+    code = gs_setstrokeconstantalpha(pgs, 1.0);
+    if (code < 0)
+        goto exit;
+    code = gs_setfillconstantalpha(pgs, 1.0);
+    if (code < 0)
+        goto exit;
+    code = gs_setalphaisshape(pgs, 0);
+    if (code < 0)
+        goto exit;
+    code = gs_setblendmode(pgs, BLEND_MODE_Compatible);
+    if (code < 0)
+        goto exit;
+    code = gs_settextknockout(pgs, true);
+    if (code < 0)
+        goto exit;
+    code = gs_setsmoothness(pgs, 0.02); /* Match gs code */
+    if (code < 0)
+        goto exit;
+
     return 0;
+exit:
+    return code;
 }
 
 /* setfilladjust */
@@ -977,6 +1047,13 @@ gs_currenttextspacing(const gs_gstate *pgs)
 int
 gs_settextspacing(gs_gstate *pgs, double Tc)
 {
+    int code = 0;
+    gs_fixed_point dxy;
+
+    code = gs_distance_transform2fixed(&pgs->ctm, Tc, 1, &dxy);
+    if (code < 0)
+        return code;
+
     pgs->textspacing = (float)Tc;
     return 0;
 }
@@ -1205,11 +1282,11 @@ gstate_alloc(gs_memory_t * mem, client_name_t cname, const gs_gstate * pfrom)
 
 /* Copy the dash pattern from one gstate to another. */
 static int
-gstate_copy_dash(gs_gstate * pto, const gs_gstate * pfrom)
+gstate_copy_dash(gs_memory_t *mem, gx_dash_params *dash , const gs_gstate * pfrom)
 {
-    return gs_setdash(pto, pfrom->line_params.dash.pattern,
+    return gx_set_dash(dash, pfrom->line_params.dash.pattern,
                       pfrom->line_params.dash.pattern_size,
-                      pfrom->line_params.dash.offset);
+                      pfrom->line_params.dash.offset, mem);
 }
 
 /* Clone an existing graphics state. */
@@ -1222,28 +1299,35 @@ gstate_clone(gs_gstate * pfrom, gs_memory_t * mem, client_name_t cname,
 {
     gs_gstate *pgs = gstate_alloc(mem, cname, pfrom);
     gs_gstate_parts parts;
+    void *pdata = NULL;
+    gx_dash_params dash;
 
-    if (pgs == 0)
+    if (pgs == NULL)
         return 0;
     GSTATE_ASSIGN_PARTS(&parts, pgs);
-    *pgs = *pfrom;
-    /* Copy the dash pattern if necessary. */
-    if (pgs->line_params.dash.pattern) {
-        int code;
+    if (pfrom->client_data != NULL) {
+        pdata = (*pfrom->client_procs.alloc) (mem);
 
-        pgs->line_params.dash.pattern = 0;      /* force allocation */
-        code = gstate_copy_dash(pgs, pfrom);
-        if (code < 0)
-            goto fail;
-    }
-    if (pgs->client_data != 0) {
-        void *pdata = pgs->client_data = (*pgs->client_procs.alloc) (mem);
-
-        if (pdata == 0 ||
-         gstate_copy_client_data(pgs, pdata, pfrom->client_data, reason) < 0
+        if (pdata == NULL ||
+         gstate_copy_client_data(pfrom, pdata, pfrom->client_data, reason) < 0
             )
             goto fail;
     }
+    /* Copy the dash and dash pattern if necessary. */
+    dash = gs_currentlineparams_inline(pfrom)->dash;
+    if (pfrom->line_params.dash.pattern) {
+        int code;
+
+        dash.pattern = NULL; /* Ensures a fresh allocation */
+        code = gstate_copy_dash(mem, &dash, pfrom);
+        if (code < 0)
+            goto fail;
+    }
+    *pgs = *pfrom;
+    pgs->client_data = pdata;
+    gs_currentlineparams_inline(pgs)->dash = dash;
+    pgs->memory = mem;
+
     gs_gstate_copied(pgs);
     /* Don't do anything to clip_stack. */
 
@@ -1268,6 +1352,8 @@ gstate_clone(gs_gstate * pfrom, gs_memory_t * mem, client_name_t cname,
     cs_adjust_counts_icc(pgs, 1);
     return pgs;
   fail:
+    if (pdata != NULL)
+        (*pfrom->client_procs.free) (pdata, mem, pgs);
     memset(pgs->color, 0, 2*sizeof(gs_gstate_color));
     gs_free_object(mem, pgs->line_params.dash.pattern, cname);
     GSTATE_ASSIGN_PARTS(pgs, &parts);
@@ -1321,15 +1407,15 @@ gstate_free_contents(gs_gstate * pgs)
         gx_cpath_free(pgs->view_clip, cname);
         pgs->view_clip = NULL;
     }
+    if (pgs->client_data != 0)
+        (*pgs->client_procs.free) (pgs->client_data, mem, pgs);
+    pgs->client_data = 0;
     gs_swapcolors_quick(pgs);
     cs_adjust_counts_icc(pgs, -1);
     gs_swapcolors_quick(pgs);
     cs_adjust_counts_icc(pgs, -1);
     pgs->color[0].color_space = 0;
     pgs->color[1].color_space = 0;
-    if (pgs->client_data != 0)
-        (*pgs->client_procs.free) (pgs->client_data, mem);
-    pgs->client_data = 0;
     gs_free_object(mem, pgs->line_params.dash.pattern, cname);
     pgs->line_params.dash.pattern = 0;
     gstate_free_parts(pgs, mem, cname);     /* this also clears pointers to freed elements */
@@ -1346,7 +1432,8 @@ gstate_copy(gs_gstate * pto, const gs_gstate * pfrom,
     GSTATE_ASSIGN_PARTS(&parts, pto);
     /* Copy the dash pattern if necessary. */
     if (pfrom->line_params.dash.pattern || pto->line_params.dash.pattern) {
-        int code = gstate_copy_dash(pto, pfrom);
+        int code = gstate_copy_dash(pto->memory,
+                             &(gs_currentlineparams_inline(pto)->dash), pfrom);
 
         if (code < 0)
             return code;
@@ -1425,8 +1512,13 @@ gs_id gx_get_clip_path_id(gs_gstate *pgs)
     return pgs->clip_path->id;
 }
 
-void gs_swapcolors_quick(gs_gstate *pgs)
+void gs_swapcolors_quick(const gs_gstate *cpgs)
 {
+    union {
+        const gs_gstate *cpgs;
+        gs_gstate *pgs;
+    } const_breaker;
+    gs_gstate *pgs;
     struct gx_cie_joint_caches_s *tmp_cie;
     gs_devicen_color_map          tmp_ccm;
     gs_client_color              *tmp_cc;
@@ -1434,6 +1526,11 @@ void gs_swapcolors_quick(gs_gstate *pgs)
     gx_device_color              *tmp_dc;
     gs_color_space               *tmp_cs;
 
+    /* Break const just once, neatly, here rather than
+     * hackily in every caller. */
+    const_breaker.cpgs = cpgs;
+    pgs = const_breaker.pgs;
+
     tmp_cc               = pgs->color[0].ccolor;
     pgs->color[0].ccolor = pgs->color[1].ccolor;
     pgs->color[1].ccolor = tmp_cc;
@@ -1446,6 +1543,11 @@ void gs_swapcolors_quick(gs_gstate *pgs)
     pgs->color[0].color_space = pgs->color[1].color_space;
     pgs->color[1].color_space = tmp_cs;
 
+    /* Overprint and effective_op vary with stroke/fill and cs */
+    tmp                         = pgs->color[0].effective_opm;
+    pgs->color[0].effective_opm = pgs->color[1].effective_opm;
+    pgs->color[1].effective_opm = tmp;
+
     /* Swap the bits of the gs_gstate that depend on the current color */
     tmp_cie                   = pgs->cie_joint_caches;
     pgs->cie_joint_caches     = pgs->cie_joint_caches_alt;
@@ -1455,32 +1557,5 @@ void gs_swapcolors_quick(gs_gstate *pgs)
     pgs->color_component_map     = pgs->color_component_map_alt;
     pgs->color_component_map_alt = tmp_ccm;
 
-    tmp                = pgs->overprint;
-    pgs->overprint     = pgs->stroke_overprint;
-    pgs->stroke_overprint = tmp;
-}
-
-int gs_swapcolors(gs_gstate *pgs)
-{
-    int prior_overprint = pgs->overprint;
-
-    gs_swapcolors_quick(pgs);
-
-    /* The following code will only call gs_do_set_overprint when we
-     * have a change:
-     * if ((prior_overprint != pgs->overprint) ||
-     *    ((prior_mode != pgs->effective_overprint_mode) &&
-     *     (pgs->overprint)))
-     *    return gs_do_set_overprint(pgs);
-     * Sadly, that's no good, as we need to call when we have swapped
-     * image space types too (separation <-> non separation for example).
-     *
-     * So instead, we call whenever at least one of them had overprint
-     * turned on.
-     */
-    if (prior_overprint || pgs->overprint)
-    {
-        return gs_do_set_overprint(pgs);
-    }
-    return 0;
+    pgs->is_fill_color = !(pgs->is_fill_color);	/* used by overprint for fill_stroke */
 }
