@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -220,7 +220,7 @@ clist_fill_mask(gx_device * dev,
         if (code < 0)
             return code;
         code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re,
-                                     devn_not_tile);
+                                     devn_not_tile_fill);
         if (code == gs_error_unregistered)
             return code;
         if (depth > 1 && code >= 0)
@@ -537,7 +537,8 @@ clist_begin_typed_image(gx_device * dev, const gs_gstate * pgs,
             indexed = false;
             num_components = gs_color_space_num_components(pcs);
         }
-        uses_color = pim->CombineWithColor && rop3_uses_T(pgs->log_op);
+        uses_color = pim->CombineWithColor &&
+                    (rop3_uses_T(pgs->log_op) || rop3_uses_S(pgs->log_op));
     }
     code = gx_image_enum_common_init((gx_image_enum_common_t *) pie,
                                      (const gs_data_image_t *) pim,
@@ -755,8 +756,6 @@ clist_begin_typed_image(gx_device * dev, const gs_gstate * pgs,
             goto use_default;
     }
     if (pim->Interpolate) {
-        if (gx_device_is_pattern_clist(dev))
-            goto use_default;
         pie->support.x = pie->support.y = MAX_ISCALE_SUPPORT + 1;
     } else {
         pie->support.x = pie->support.y = 0;
@@ -999,7 +998,9 @@ clist_image_plane_data(gx_image_enum_common_t * info,
     sbox.p.y = (y0 = y_orig) - pie->support.y;
     sbox.q.x = pie->rect.q.x + pie->support.x;
     sbox.q.y = (y1 = pie->y += yh_used) + pie->support.y;
-    gs_bbox_transform(&sbox, &pie->matrix, &dbox);
+    code = gs_bbox_transform(&sbox, &pie->matrix, &dbox);
+    if (code < 0)
+        return code;
     /*
      * In order to keep the band list consistent, we must write out
      * the image data in precisely those bands whose begin_image
@@ -1095,8 +1096,8 @@ clist_image_plane_data(gx_image_enum_common_t * info,
             pie->color_map_is_known = true;
             if (code >= 0) {
                 uint want_known = ctm_known | clip_path_known |
-                            op_bm_tk_known | opacity_alpha_known |
-                            shape_alpha_known | alpha_known |
+                            op_bm_tk_known | ais_known |
+                            fill_alpha_known | stroke_alpha_known | fill_adjust_known |
                             (pie->color_space.id == gs_no_id ? 0 :
                                                      color_space_known);
 
@@ -1109,6 +1110,8 @@ clist_image_plane_data(gx_image_enum_common_t * info,
             if (code < 0)
                 return code;
             if (pie->uses_color) {
+                gs_int_point color_phase;
+
                 /* We want to write the color taking into account the entire image so */
                 /* we set re.rect_nbands from pie->ymin and pie->ymax so that we will */
                 /* make the decision to write 'all_bands' the same for the whole image */
@@ -1117,9 +1120,17 @@ clist_image_plane_data(gx_image_enum_common_t * info,
                 re.rect_nbands = ((pie->ymax + re.band_height - 1) / re.band_height) -
                                  ((pie->ymin) / re.band_height);
                 code = cmd_put_drawing_color(cdev, re.pcls, &pie->dcolor,
-                                             &re, devn_not_tile);
+                                             &re, devn_not_tile_fill);
                 if (code < 0)
                     return code;
+                /* see if phase informaiton must be inserted in the command list */
+                /* if so, go ahead and do it for all_bands */
+                if ( pie->dcolor.type->get_phase(&pie->dcolor, &color_phase) &&
+                     (color_phase.x != re.pcls->tile_phase.x ||
+                      color_phase.y != re.pcls->tile_phase.y ) &&
+                     (code = cmd_set_tile_phase_generic(cdev, re.pcls,
+                                                        color_phase.x, color_phase.y, true)) < 0  )
+                    return code;
             }
             if (entire_box.p.x != 0 || entire_box.p.y != 0 ||
                 entire_box.q.x != pie->image.Width ||
@@ -1827,10 +1838,10 @@ clist_image_unknowns(gx_device *dev, const clist_image_enum *pie)
     uint unknown = 0;
 
     /*
-     * Determine if the CTM, color space, and clipping region (and, for
-     * masked images or images with CombineWithColor, the current color)
-     * are unknown. Set the device state in anticipation of the values
-     * becoming known.
+     * Determine if the CTM, color space, fill_adjust and clipping region,
+     * (and, for masked images or images with CombineWithColor, the current
+     * color) are unknown. Set the device state in anticipation of the
+     * values becoming known.
      */
     if (cdev->gs_gstate.ctm.xx != pgs->ctm.xx ||
         cdev->gs_gstate.ctm.xy != pgs->ctm.xy ||
@@ -1845,14 +1856,18 @@ clist_image_unknowns(gx_device *dev, const clist_image_enum *pie)
     if (pie->color_space.id == gs_no_id) { /* masked image */
         cdev->color_space.space = 0; /* for GC */
     } else {                    /* not masked */
-        if (cdev->color_space.id == pie->color_space.id) {
-            /* The color space pointer might not be valid: update it. */
-            cdev->color_space.space = pie->color_space.space;
-        } else {
+        if (cdev->color_space.id != pie->color_space.id ||
+            cdev->color_space.space != pie->color_space.space) {
             unknown |= color_space_known;
+            cdev->color_space.space = pie->color_space.space;
             cdev->color_space = pie->color_space;
         }
     }
+    if (cdev->gs_gstate.fill_adjust.x != pgs->fill_adjust.x ||
+        cdev->gs_gstate.fill_adjust.y != pgs->fill_adjust.y) {
+        unknown |= fill_adjust_known;
+        cdev->gs_gstate.fill_adjust = pgs->fill_adjust;
+    }
     if (cmd_check_clip_path(cdev, pie->pcpath))
         unknown |= clip_path_known;
     /*
@@ -1873,17 +1888,17 @@ clist_image_unknowns(gx_device *dev, const clist_image_enum *pie)
         cdev->gs_gstate.text_knockout = pgs->text_knockout;
         cdev->gs_gstate.renderingintent = pgs->renderingintent;
     }
-    if (cdev->gs_gstate.opacity.alpha != pgs->opacity.alpha) {
-        unknown |= opacity_alpha_known;
-        cdev->gs_gstate.opacity.alpha = pgs->opacity.alpha;
+    if (cdev->gs_gstate.alphaisshape != pgs->alphaisshape) {
+        unknown |= ais_known;
+        cdev->gs_gstate.alphaisshape = pgs->alphaisshape;
     }
-    if (cdev->gs_gstate.shape.alpha != pgs->shape.alpha) {
-        unknown |= shape_alpha_known;
-        cdev->gs_gstate.shape.alpha = pgs->shape.alpha;
+    if (cdev->gs_gstate.strokeconstantalpha != pgs->strokeconstantalpha) {
+        unknown |= stroke_alpha_known;
+        cdev->gs_gstate.strokeconstantalpha = pgs->strokeconstantalpha;
     }
-    if (cdev->gs_gstate.alpha != pgs->alpha) {
-        unknown |= alpha_known;
-        cdev->gs_gstate.alpha = pgs->alpha;
+    if (cdev->gs_gstate.fillconstantalpha != pgs->fillconstantalpha) {
+        unknown |= fill_alpha_known;
+        cdev->gs_gstate.fillconstantalpha = pgs->fillconstantalpha;
     }
     return unknown;
 }
