@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -675,13 +675,12 @@ typedef struct gx_device_cached_colors_s {
  * Define the parameters controlling banding.
  */
 /* if you make any additions/changes to this structure you need to make
-   the appropriate additions/changes to the compare_gdev_prn_space_params()
-   function in gdevprn.c */
+   the appropriate additions/changes to gdev_space_params_cmp() */
 typedef struct gx_band_params_s {
     int BandWidth;		/* (optional) band width in pixels */
     int BandHeight;		/* (optional) */
-    long BandBufferSpace;	/* (optional) */
-    long tile_cache_size;	/* (optional) */
+    size_t BandBufferSpace;	/* (optional) */
+    size_t tile_cache_size;	/* (optional) */
 } gx_band_params_t;
 
 #define BAND_PARAMS_INITIAL_VALUES 0, 0, 0, 0
@@ -693,16 +692,20 @@ typedef enum {
 } gdev_banding_type;
 
 /* if you make any additions/changes to this structure you need to make
-   the appropriate additions/changes to the compare_gdev_prn_space_params()
-   function in gdevprn.c */
+   the appropriate additions/changes to the gdev_space_params_cmp() */
 typedef struct gdev_space_params_s {
-    long MaxBitmap;		/* max size of non-buffered bitmap */
-    long BufferSpace;		/* space to use for buffer */
+    size_t MaxBitmap;		/* max size of non-buffered bitmap */
+    size_t BufferSpace;		/* space to use for buffer */
     gx_band_params_t band;	/* see gxband.h */
     bool params_are_read_only;	/* true if put_params may not modify this struct */
     gdev_banding_type banding_type;	/* used to force banding or bitmap */
 } gdev_space_params;
 
+/* Returns 0 for a match, non-zero otherwise. Like memcmp, but allowing
+ * for uninitialised padding. */
+int gdev_space_params_cmp(const gdev_space_params sp1,
+                          const gdev_space_params sp2);
+
 typedef struct gdev_pagelist_s {
         rc_header rc;
         char *Pages;
@@ -767,7 +770,7 @@ typedef struct gdev_pagelist_s {
         long band_offset_y;		/* for rendering that is phase sensitive (old wtsimdi) */\
         bool BLS_force_memory;\
         gx_stroked_gradient_recognizer_t sgr;\
-        int MaxPatternBitmap;		/* Threshold for switching to pattern_clist mode */\
+        size_t MaxPatternBitmap;	/* Threshold for switching to pattern_clist mode */\
         bool page_uses_transparency;    /* PDF 1.4 transparency is used. */\
         gdev_space_params space_params;\
         cmm_dev_profile_t *icc_struct;  /* object dependent profiles */\
@@ -1035,6 +1038,19 @@ typedef enum FILTER_FLAGS {
 #define dev_proc_stroke_path(proc)\
   dev_t_proc_stroke_path(proc, gx_device)
 
+                /* Added in release 9.22 */
+
+#define dev_t_proc_fill_stroke_path(proc, dev_t)\
+  int proc(dev_t *dev,\
+    const gs_gstate *pgs, gx_path *ppath,\
+    const gx_fill_params *fill_params,\
+    const gx_drawing_color *pdcolor_fill,\
+    const gx_stroke_params *stroke_params,\
+    const gx_drawing_color *pdcolor_stroke,\
+    const gx_clip_path *pcpath)
+#define dev_proc_fill_stroke_path(proc)\
+  dev_t_proc_fill_stroke_path(proc, gx_device)
+
                 /* Added in release 3.60 */
 
 #define dev_t_proc_fill_mask(proc, dev_t)\
@@ -1642,6 +1658,7 @@ typedef struct {
         dev_t_proc_copy_alpha_hl_color((*copy_alpha_hl_color), dev_t);\
         dev_t_proc_process_page((*process_page), dev_t);\
         dev_t_proc_transform_pixel_region((*transform_pixel_region), dev_t);\
+        dev_t_proc_fill_stroke_path((*fill_stroke_path), dev_t);\
 }
 
 /*
@@ -1968,4 +1985,7 @@ void gx_device_dump(gx_device *dev, const char *text);
 /* Compare color information structures */
 bool gx_color_info_equal(const gx_device_color_info *p1, const gx_device_color_info *p2);
 
+/* Perform a callout to registered handlers from the device. */
+int gx_callout(gx_device *dev, int id, int size, void *data);
+
 #endif /* gxdevcli_INCLUDED */
