@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -52,8 +52,8 @@
 #define DEFAULT_WIDTH_10THS_US_LETTER 85
 #define DEFAULT_HEIGHT_10THS_US_LETTER 110
 /* A4 paper (210mm x 297mm), we use 595pt x 842pt. */
-#define DEFAULT_WIDTH_10THS_A4 82.6389
-#define DEFAULT_HEIGHT_10THS_A4 116.9444
+#define DEFAULT_WIDTH_10THS_A4 82.6389f
+#define DEFAULT_HEIGHT_10THS_A4 116.9444f
 /* Choose a default.  A4 may be set in the makefile. */
 #ifdef A4
 #  define DEFAULT_WIDTH_10THS DEFAULT_WIDTH_10THS_A4
@@ -314,6 +314,7 @@ dev_proc_strip_tile_rect_devn(gx_default_strip_tile_rect_devn);
 dev_proc_copy_alpha_hl_color(gx_default_copy_alpha_hl_color);
 dev_proc_process_page(gx_default_process_page);
 dev_proc_transform_pixel_region(gx_default_transform_pixel_region);
+dev_proc_fill_stroke_path(gx_default_fill_stroke_path);
 dev_proc_begin_transparency_group(gx_default_begin_transparency_group);
 dev_proc_end_transparency_group(gx_default_end_transparency_group);
 dev_proc_begin_transparency_mask(gx_default_begin_transparency_mask);
@@ -418,6 +419,7 @@ dev_proc_strip_copy_rop2(gx_forward_strip_copy_rop2);
 dev_proc_strip_tile_rect_devn(gx_forward_strip_tile_rect_devn);
 dev_proc_copy_alpha_hl_color(gx_forward_copy_alpha_hl_color);
 dev_proc_transform_pixel_region(gx_forward_transform_pixel_region);
+dev_proc_fill_stroke_path(gx_forward_fill_stroke_path);
 
 /* ---------------- Implementation utilities ---------------- */
 int gx_default_get_param(gx_device *dev, char *Param, void *list);
@@ -501,11 +503,11 @@ bool gx_outputfile_is_separate_pages(const char *fname, gs_memory_t *memory);
  */
 int gx_device_open_output_file(const gx_device * dev, char *fname,
                                bool binary, bool positionable,
-                               FILE ** pfile);
+                               gp_file ** pfile);
 
 /* Close the output file for a device. */
 int gx_device_close_output_file(const gx_device * dev, const char *fname,
-                                FILE *file);
+                                gp_file *file);
 
 /* Delete the current output file for a device (file must be closed first) */
 int gx_device_delete_output_file(const gx_device * dev, const char *fname);
