@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -26,6 +26,9 @@
 #include "gscspace.h"
 #include "gxcindex.h"
 #include "gxfrac.h"
+#include "gxtmap.h"
+#include "gsdcolor.h"
+#include "gxht.h"
 
 /*
  * We represent a halftone tile as a rectangular super-cell consisting of
@@ -153,14 +156,6 @@ typedef ht_mask_t ht_sample_t;
  * See gxbitmap.h for more details about strip halftones.
  */
 typedef struct gx_ht_cache_s gx_ht_cache;
-#ifndef gx_ht_order_DEFINED
-#  define gx_ht_order_DEFINED
-typedef struct gx_ht_order_s gx_ht_order;
-#endif
-#ifndef gx_ht_tile_DEFINED
-#  define gx_ht_tile_DEFINED
-typedef struct gx_ht_tile_s gx_ht_tile;
-#endif
 typedef struct gx_ht_order_procs_s {
 
     /* Define the size of each element of bit_data. */
@@ -276,11 +271,6 @@ extern_st(st_ht_order_component_element);
     "gx_ht_order_component[]", ht_order_element_enum_ptrs,\
     ht_order_element_reloc_ptrs, st_ht_order_component)
 
-#ifndef gx_device_halftone_DEFINED
-#  define gx_device_halftone_DEFINED
-typedef struct gx_device_halftone_s gx_device_halftone;
-#endif
-
 /*
  * Device Halftone Structure definition.  See comments before
  * gx_gstate_dev_ht_install() for more information on this structure and its
