@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -155,15 +155,15 @@ gs_private_st_simple(st_active_line, active_line, "active_line");
 static void
 print_active_line(const gs_memory_t *mem, const char *label, const active_line * alp)
 {
-    dmlprintf5(mem, "[f]%s 0x%lx(%d): x_current=%f x_next=%f\n",
-               label, (ulong) alp, alp->direction,
+    dmlprintf5(mem, "[f]%s "PRI_INTPTR"(%d): x_current=%f x_next=%f\n",
+               label, (intptr_t)alp, alp->direction,
                fixed2float(alp->x_current), fixed2float(alp->x_next));
-    dmlprintf5(mem, "    start=(%f,%f) pt_end=0x%lx(%f,%f)\n",
+    dmlprintf5(mem, "    start=(%f,%f) pt_end="PRI_INTPTR"(%f,%f)\n",
                fixed2float(alp->start.x), fixed2float(alp->start.y),
-               (ulong) alp->pseg,
+               (intptr_t)alp->pseg,
                fixed2float(alp->end.x), fixed2float(alp->end.y));
-    dmlprintf2(mem, "    prev=0x%lx next=0x%lx\n",
-               (ulong) alp->prev, (ulong) alp->next);
+    dmlprintf2(mem, "    prev="PRI_INTPTR" next="PRI_INTPTR"\n",
+               (intptr_t)alp->prev, (intptr_t)alp->next);
 }
 static void
 print_line_list(const gs_memory_t *mem, const active_line * flp)
@@ -173,8 +173,8 @@ print_line_list(const gs_memory_t *mem, const active_line * flp)
     for (lp = flp; lp != 0; lp = lp->next) {
         fixed xc = lp->x_current, xn = lp->x_next;
 
-        dmlprintf3(mem, "[f]0x%lx(%d): x_current/next=%g",
-                  (ulong) lp, lp->direction,
+        dmlprintf3(mem, "[f]"PRI_INTPTR"(%d): x_current/next=%g",
+                  (intptr_t)lp, lp->direction,
                   fixed2float(xc));
         if (xn != xc)
             dmprintf1(mem, "/%g", fixed2float(xn));
@@ -469,7 +469,7 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
                                         fo.fixed_flat,
                                         params->rule,
                                         pdevc,
-                                        (!fill_by_trapezoids && fo.fill_direct) ? -1 : (int)pgs->log_op);
+                                       (!fill_by_trapezoids && fo.fill_direct) ? -1 : (int)pgs->log_op);
         if (ppath == &ffpath)
             gx_path_free(ppath, "gx_general_fill_path");
         return code;
@@ -663,6 +663,30 @@ gx_default_fill_path(gx_device * pdev, const gs_gstate * pgs,
     return code;
 }
 
+/*
+ * Fill/Stroke a path.  This is the default implementation of the driver
+ * fill_path procedure.
+ */
+int
+gx_default_fill_stroke_path(gx_device * pdev, const gs_gstate * pgs,
+                            gx_path * ppath,
+                            const gx_fill_params * params_fill,
+                            const gx_device_color * pdevc_fill,
+                            const gx_stroke_params * params_stroke,
+                            const gx_device_color * pdevc_stroke,
+                            const gx_clip_path * pcpath)
+{
+    int code = dev_proc(pdev, fill_path)(pdev, pgs, ppath, params_fill, pdevc_fill, pcpath);
+
+    if (code < 0)
+        return code;
+    /* Swap colors to make sure the pgs colorspace is correct for stroke */
+    gs_swapcolors_quick(pgs);
+    code = dev_proc(pdev, stroke_path)(pdev, pgs, ppath, params_stroke, pdevc_stroke, pcpath);
+    gs_swapcolors_quick(pgs);
+    return code;
+}
+
 /* Free the line list. */
 static void
 free_line_list(line_list *ll)
@@ -1313,7 +1337,7 @@ remove_al(const line_list *ll, active_line *alp)
     alp->prev->next = nlp;
     if (nlp)
         nlp->prev = alp->prev;
-    if_debug1m('F', ll->memory, "[F]drop 0x%lx\n", (ulong) alp);
+    if_debug1m('F', ll->memory, "[F]drop "PRI_INTPTR"\n", (intptr_t)alp);
 }
 
 /*
@@ -1592,8 +1616,8 @@ resort_x_line(active_line * alp)
     if (next)
         next->prev = prev;
     while (x_order(prev, alp) > 0) {
-        if_debug2('F', "[F]swap 0x%lx,0x%lx\n",
-                   (ulong) alp, (ulong) prev);
+        if_debug2('F', "[F]swap "PRI_INTPTR","PRI_INTPTR"\n",
+                   (intptr_t)alp, (intptr_t)prev);
         next = prev, prev = prev->prev;
     }
     alp->next = next;
@@ -2129,8 +2153,8 @@ range_alloc(coord_range_list_t *pcrl)
 static void
 range_delete(coord_range_list_t *pcrl, coord_range_t *pcr)
 {
-    if_debug3('Q', "[Qr]delete 0x%lx: [%d,%d)\n", (ulong)pcr, pcr->rmin,
-              pcr->rmax);
+    if_debug3('Q', "[Qr]delete "PRI_INTPTR": [%d,%d)\n",
+              (intptr_t)pcr, pcr->rmin, pcr->rmax);
     pcr->prev->next = pcr->next;
     pcr->next->prev = pcr->prev;
     pcr->next = pcrl->freed;
@@ -2250,13 +2274,13 @@ range_list_add(coord_range_list_t *pcrl, coord_value_t rmin, coord_value_t rmax)
      * abut if the adjacent range is the special min or max range.
      */
     if (rmin < pcr->rmin) {
-        if_debug3('Q', "[Qr]update 0x%lx => [%d,%d)\n", (ulong)pcr, rmin,
-                  pcr->rmax);
+        if_debug3('Q', "[Qr]update "PRI_INTPTR" => [%d,%d)\n",
+                  (intptr_t)pcr, rmin, pcr->rmax);
         pcr->rmin = rmin;
     }
     if (rmax > pcr->rmax) {
-        if_debug3('Q', "[Qr]update 0x%lx => [%d,%d)\n", (ulong)pcr, pcr->rmin,
-                  rmax);
+        if_debug3('Q', "[Qr]update "PRI_INTPTR" => [%d,%d)\n",
+                  (intptr_t)pcr, pcr->rmin, rmax);
         pcr->rmax = rmax;
     }
     pcrl->current = pcr->next;
@@ -2268,7 +2292,7 @@ range_list_add(coord_range_list_t *pcrl, coord_value_t rmin, coord_value_t rmax)
 
         if (prev == 0)
             return_error(gs_error_VMerror);
-        if_debug3('Q', "[Qr]insert 0x%lx: [%d,%d)\n", (ulong)prev, rmin, rmax);
+        if_debug3('Q', "[Qr]insert "PRI_INTPTR": [%d,%d)\n", (intptr_t)prev, rmin, rmax);
         prev->rmin = rmin, prev->rmax = rmax;
         (prev->prev = pcr->prev)->next = prev;
         prev->next = pcr;
