@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -19,6 +19,9 @@
 #ifndef gxfill_INCLUDED
 #  define gxfill_INCLUDED
 
+#include "gzpath.h"
+#include "gxdevcli.h"
+
 enum
 {
     DIR_UP = 1,
@@ -28,10 +31,8 @@ enum
 };
 
 /* Define the structure for keeping track of active lines. */
-#ifndef active_line_DEFINED
-#  define active_line_DEFINED
 typedef struct active_line_s active_line;
-#endif
+
 struct active_line_s {
     gs_fixed_point start;	/* x,y where line starts */
     gs_fixed_point end; 	/* x,y where line ends */
@@ -113,10 +114,8 @@ typedef struct fill_options_s {
 } fill_options;
 
 /* Line list structure */
-#ifndef line_list_DEFINED
-#  define line_list_DEFINED
 typedef struct line_list_s line_list;
-#endif
+
 struct line_list_s {
     gs_memory_t *memory;
     active_line *active_area;	/* allocated active_line list */
