@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -127,6 +127,7 @@ typedef struct gs_gstate_color_s {
     gs_color_space *color_space; /* after substitution */
     gs_client_color *ccolor;
     gx_device_color *dev_color;
+    bool effective_opm;
 } gs_gstate_color;
 
 /*
@@ -201,6 +202,7 @@ typedef struct gs_xstate_trans_flags {
 #define gs_currentcolor_inline(pgs)       ((pgs)->color[0].ccolor)
 #define gs_currentcolorspace_inline(pgs)  ((pgs)->color[0].color_space)
 #define gs_altdevicecolor_inline(pgs)     ((pgs)->color[1].dev_color)
+#define gs_currentcolor_eopm(pgs)         ((pgs)->color[0].effective_opm)
 
 #define char_tm_only(pgs) *(gs_matrix *)&(pgs)->char_tm
 
@@ -238,9 +240,7 @@ struct gs_gstate_s {
     gs_point subpath_start;
     bool clamp_coordinates;
     gs_logical_operation_t log_op;
-    gx_color_value alpha;
     gs_blend_mode_t blend_mode;
-    gs_transparency_source_t opacity, shape;
     gs_xstate_trans_flags_t trans_flags;
     gs_id soft_mask_id;
     bool text_knockout;
@@ -249,7 +249,6 @@ struct gs_gstate_s {
     gx_device *trans_device;  /* trans device has all mappings to group color space */
     bool overprint;
     int overprint_mode;
-    int effective_overprint_mode;
     bool stroke_overprint;
     float flatness;
     gs_fixed_point fill_adjust; /* A path expansion for fill; -1 = dropout prevention*/
@@ -303,7 +302,7 @@ struct gs_gstate_s {
     gs_matrix textmatrix;
     /* Current colors (non-stroking, and stroking) */
     gs_gstate_color color[2];
-
+    int is_fill_color;
     /* Font: */
     gs_font *font;
     gs_font *root_font;
@@ -324,9 +323,9 @@ struct gs_gstate_s {
   0, 0, { gx_line_params_initial }, 0,\
    { (float)(scale), 0.0, 0.0, (float)(-(scale)), 0.0, 0.0 },\
   false, {0, 0}, {0, 0}, false, \
-  lop_default, gx_max_color_value, BLEND_MODE_Compatible,\
-{ 1.0 }, { 1.0 }, {0, 0}, 0, 0/*false*/, 0, 0/*false*/, 0, 0/*false*/, 0, 0, 0/*false*/, 1.0,  \
-   { fixed_half, fixed_half }, 0/*false*/, 1/*true*/, 0/*false*/, 1.0,\
+  lop_default, BLEND_MODE_Compatible,\
+  {0, 0}, 0, 1/*true*/, 0, 0/*false*/, 0, 0/*false*/, 0, 0/*false*/, 1.0,  \
+   { fixed_half, fixed_half }, 0/*false*/, 1/*true*/, 0/*false*/, (float)0.02,\
   1, 1/* bpt true */, 0, 0, 0, INIT_CUSTOM_COLOR_PTR	/* 'Custom color' callback pointer */  \
   gx_default_get_cmap_procs
 
@@ -343,10 +342,7 @@ struct gs_gstate_s {
     s->subpath_start = __state_init.subpath_start; \
     s->clamp_coordinates = __state_init.clamp_coordinates; \
     s->log_op = __state_init.log_op; \
-    s->alpha = __state_init.alpha; \
     s->blend_mode = __state_init.blend_mode; \
-    s->opacity = __state_init.opacity; \
-    s->shape = __state_init.shape; \
     s->trans_flags = __state_init.trans_flags; \
     s->soft_mask_id = __state_init.soft_mask_id; \
     s->text_knockout = __state_init.text_knockout; \
@@ -355,7 +351,6 @@ struct gs_gstate_s {
     s->trans_device = __state_init.trans_device; \
     s->overprint = __state_init.overprint; \
     s->overprint_mode = __state_init.overprint_mode; \
-    s->effective_overprint_mode = __state_init.effective_overprint_mode; \
     s->stroke_overprint = __state_init.stroke_overprint; \
     s->flatness = __state_init.flatness; \
     s->fill_adjust = __state_init.fill_adjust; \
@@ -370,6 +365,15 @@ struct gs_gstate_s {
     s->icc_profile_cache = __state_init.icc_profile_cache; \
     s->get_cmap_procs = __state_init.get_cmap_procs; \
     s->show_gstate = NULL; \
+    s->is_fill_color = 1; \
+    s->strokeconstantalpha = 1.0; \
+    s->fillconstantalpha = 1.0; \
+    s->alphaisshape = 0; \
+    s->texthscaling = 100.0; \
+    s->textspacing = 0.0; \
+    s->textleading = 0.0; \
+    s->wordspacing = 0.0; \
+    s->textrise = 0.0; \
   } while (0)
 
 struct_proc_finalize(gs_gstate_finalize);
@@ -416,10 +420,6 @@ struct_proc_finalize(gs_gstate_finalize);
 /* gs_gstate_initial. */
 int gs_gstate_initialize(gs_gstate * pgs, gs_memory_t * mem);
 
-/* Make a temporary copy of a gs_gstate.  Note that this does not */
-/* do all the necessary reference counting, etc. */
-gs_gstate * gs_gstate_copy_temp(const gs_gstate * pgs, gs_memory_t * mem);
-
 /* Increment reference counts to note that a graphics state has been copied. */
 void gs_gstate_copied(gs_gstate * pgs);
 
@@ -437,8 +437,15 @@ int gs_currentscreenphase_pgs(const gs_gstate *, gs_int_point *, gs_color_select
     (pgs)->current_point.x = xx;\
     (pgs)->current_point.y = yy;
 
-int gs_swapcolors(gs_gstate *);
-void gs_swapcolors_quick(gs_gstate *);
+/* The const is a lie. The swapcolors stuff is a bit of a hack; a late
+ * addition to the graphics library, which has to be used in many places
+ * where the device interface only has a const gs_gstate available to it
+ * rather than a non-const one. In all these cases, we flip the colors
+ * during a call to a function, then swap it back again before we leave,
+ * so the net effect of the call remains that the gstate is not changed.
+ * Rather than breaking const at every single callsite, we move the const
+ * breaking into the function itself. */
+void gs_swapcolors_quick(const gs_gstate *);
 
 /* Set the graphics_type_tag iff the requested tag bit is not set in the dev_color and	*/
 /* unset the dev_color so that gx_set_dev_color will remap (encode) with the new tag.	*/
