@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -58,6 +58,9 @@ gx_path_subpath_start_point(const gx_path * ppath, gs_fixed_point * ppt)
 int
 gx_path_bbox(gx_path * ppath, gs_fixed_rect * pbox)
 {
+    if (ppath == NULL) {
+        return_error(gs_error_unknownerror) ;
+    }
     if (ppath->bbox_accurate) {
         /* The bounding box was set by setbbox. */
         *pbox = ppath->bbox;
