@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2008 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Stream wrapper for the AES cypher implementation */
 /* Requires scommon.h; strimpl.h if any templates are referenced */
 
@@ -37,10 +39,7 @@ struct stream_aes_state_s
     aes_context *ctx;		/* from aes.h */
 };
 
-#ifndef stream_aes_state_DEFINED
-#define stream_aes_state_DEFINED
 typedef struct stream_aes_state_s stream_aes_state;
-#endif
 
 int s_aes_set_key(stream_aes_state * state,
                         const unsigned char *key, int keylength);
