@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -40,6 +40,7 @@
 #define ARCH_SIZEOF_LONG (1 << ARCH_LOG2_SIZEOF_LONG)
 #define ARCH_SIZEOF_LONG_LONG (1 << ARCH_LOG2_SIZEOF_LONG_LONG)
 #define ARCH_INTS_ARE_SHORT (ARCH_SIZEOF_INT == ARCH_SIZEOF_SHORT)
+#define ARCH_SIZEOF_INT64_T 8
 
 /* Define whether we are on a large- or small-memory machine. */
 /* Currently, we assume small memory and 16-bit ints are synonymous. */
@@ -85,6 +86,7 @@ typedef ulong bits32;
 #define max_ushort ARCH_MAX_USHORT
 #define max_uint ARCH_MAX_UINT
 #define max_ulong ARCH_MAX_ULONG
+#define max_size_t ARCH_MAX_SIZE_T
 
 /* Minimum and maximum values for pointers. */
 #if ARCH_PTRS_ARE_SIGNED
