@@ -33,6 +33,15 @@
 
  */
 
+#include <stdlib.h>		/* for abs() and free */
+
+/* Get this definition in before we read memento.h */
+static void
+unvectored_free(void *x)
+{
+  free(x);
+}
+
 #if ( 6 > GS_VERSION_MAJOR )
 
 #include <string.h>
@@ -59,7 +68,6 @@
 #else /* 6 <= GS_VERSION_MAJOR */
 
 #include "math_.h"
-#include <stdlib.h>		/* for abs() */
 #include <sys/utsname.h>	/* for uname(2) */
 #include <ctype.h>		/* for toupper(3) */
 
@@ -969,7 +977,11 @@ escv_beginpage(gx_device_vector * vdev)
     {
       time_t t;
 
+#ifdef CLUSTER
+      memset(&t, 0, sizeof(t));
+#else
       time(&t);
+#endif
 
       lputs(s, " DATE=\"");
       {
@@ -977,8 +989,14 @@ escv_beginpage(gx_device_vector * vdev)
         char   str[32];
         size_t i;
 
+#ifdef CLUSTER
+        memset(&tm, 0, sizeof(tm));
+        strcpy(str, "1970/01/01 00:00:00");
+        i = strlen(str);
+#else
         tm =  localtime( &t );
         i = strftime(str, 30, "%Y/%m/%d %H:%M:%S", tm);
+#endif
         if ( 30 >= i )
           str[i] = '\0';
 
@@ -1014,7 +1032,8 @@ escv_beginpage(gx_device_vector * vdev)
       if (sysname)
         {
           lputs(s, sysname );
-          free (sysname);
+          /* Carefully avoid memento interfering here. */
+          unvectored_free(sysname);
           sysname = NULL;
         }
     }
@@ -1773,11 +1792,11 @@ escv_output_page(gx_device * dev, int num_copies, int flush)
 static int
 escv_close(gx_device *dev)
 {
-  gx_device_vector	*const vdev = (gx_device_vector *) dev;
-  FILE		*f = vdev->file;
+  gx_device_vector *const vdev = (gx_device_vector *) dev;
+  gp_file          *f = vdev->file;
 
   /* λɤϷǤ */
-  (void)fprintf(f, ESC_GS "rhE" "\033\001@EJL \012@EJL EJ \012\033\001@EJL \012");
+  (void)gp_fprintf(f, ESC_GS "rhE" "\033\001@EJL \012@EJL EJ \012\033\001@EJL \012");
 
   gdev_vector_close_file(vdev);
 
@@ -2035,6 +2054,7 @@ escv_put_params(gx_device * dev, gs_param_list * plist)
     ecode = code;
   pmediae:
     param_signal_error(plist, param_name, ecode);
+    /* Fall through. */
   case 1:
     if(!pdev->MediaType){
       pdev->MediaType = 0;
@@ -2549,7 +2569,10 @@ escv_begin_image(gx_device * dev,
   }
 
   /* Write the image/colorimage/imagemask preamble. */
-  gs_matrix_invert(&pim->ImageMatrix, &imat);
+  code = gs_matrix_invert(&pim->ImageMatrix, &imat);
+  if (code < 0)
+      return code;
+
   gs_matrix_multiply(&imat, &ctm_only(pgs), &imat);
 
   ty = imat.ty;
