@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -347,7 +347,7 @@ const gx_device_bit gs_bitrgbtags_device =
             gx_default_end_page
         }, /* page_procs */
         { 0 }, /* procs */
-        { 0 }, /* skip */
+        GX_CLIST_MUTATABLE_DEVICE_DEFAULTS,
         {
           bittags_print_page,
           gx_default_print_page_copies,
@@ -364,17 +364,11 @@ const gx_device_bit gs_bitrgbtags_device =
         -1,    /* Duplex_set */
         false, /* file_is_new */
         NULL,  /* file */
-        0,     /* buffer_space */
-        NULL,  /* buf */
-        NULL,  /* buffer_memory */
-        NULL,  /* bandlist_memory */
-        0,     /* clist_disable_mask */
         false, /* bg_print_requested */
         {0},   /* bg_print */
         0,     /* num_render_threads_requested */
         NULL,  /* saved_pages_list */
-        {0},   /* save_procs_while_delaying_erasepage */
-        {0}    /* orig_procs */
+        {0}    /* save_procs_while_delaying_erasepage */
     };
 
 static void
@@ -792,19 +786,22 @@ bit_print_page(gx_device_printer * pdev, gp_file * prn_stream)
     int bottom = ((gx_device_bit *)pdev)->LastLine >= pdev->height ?  pdev->height - 1 :
                  ((gx_device_bit *)pdev)->LastLine;
     int line_count = any_abs(bottom - lnum);
-    int i, step = lnum > bottom ? -1 : 1;
+    int code = 0, i, step = lnum > bottom ? -1 : 1;
 
     if (in == 0)
         return_error(gs_error_VMerror);
     if ((lnum == 0) && (bottom == 0))
         line_count = pdev->height - 1;		/* default when LastLine == 0, FirstLine == 0 */
     for (i = 0; i <= line_count; i++, lnum += step) {
-        gdev_prn_get_bits(pdev, lnum, in, &data);
+        code = gdev_prn_get_bits(pdev, lnum, in, &data);
+        if (code < 0)
+            goto done;
         if (!nul)
             gp_fwrite(data, 1, line_size, prn_stream);
     }
+done:
     gs_free_object(pdev->memory, in, "bit_print_page(in)");
-    return 0;
+    return code;
 }
 
 /* For tags device go ahead and add in the size so that we can strip and create
@@ -835,10 +832,11 @@ bittags_print_page(gx_device_printer * pdev, gp_file * prn_stream)
         line_count = pdev->height - 1;		/* default when LastLine == 0, FirstLine == 0 */
     for (i = 0; i <= line_count; i++, lnum += step) {
         if ((code = gdev_prn_get_bits(pdev, lnum, in, &data)) < 0)
-            return code;
+            goto done;
         if (!nul)
             gp_fwrite(data, 1, line_size, prn_stream);
     }
+done:
     gs_free_object(pdev->memory, in, "bit_print_page(in)");
     return 0;
 }
