@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -18,15 +18,17 @@
 #ifndef gdevbmp_INCLUDED
 #  define gdevbmp_INCLUDED
 
+#include "gxclist.h"
+
 /* Define the default X and Y resolution. */
 #define X_DPI 72
 #define Y_DPI 72
 
 /* Write the BMP file header.  This procedure is used for all formats. */
-int write_bmp_header(gx_device_printer *pdev, FILE *file);
+int write_bmp_header(gx_device_printer *pdev, gp_file *file);
 
 /* Write a BMP header for separated CMYK output. */
-int write_bmp_separated_header(gx_device_printer *pdev, FILE *file);
+int write_bmp_separated_header(gx_device_printer *pdev, gp_file *file);
 
 /* 24-bit color mappers */
 dev_proc_map_rgb_color(bmp_map_16m_rgb_color);
