@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -1054,6 +1054,8 @@ pnmcmyk_print_page(gx_device_printer *pdev, gp_file *pstream)
             !strncmp(pdev->fname, "/dev/null", min(strlen(pdev->fname), 9));
         int (*row_proc) (gx_device_printer *, byte *, int, gp_file *);
 
+        if (data == NULL)
+            return_error(gs_error_VMerror);
         if (!output_is_nul) {
             if (gp_fprintf(pstream, "P5\n") < 0) {	/* PGM raw */
                 code = gs_note_error(gs_error_ioerror);
