@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -129,7 +129,7 @@ RELOC_PTRS_END
 static int
 psd_spec_op(gx_device *dev_, int op, void *data, int datasize)
 {
-    if (op == gxdso_supports_devn) {
+    if (op == gxdso_supports_devn || op == gxdso_skip_icc_component_validation) {
         return true;
     }
     return gdev_prn_dev_spec_op(dev_, op, data, datasize);
@@ -453,7 +453,11 @@ psd_prn_open(gx_device * pdev)
     pdev->color_info.depth = pdev->color_info.num_components *
                              pdev_psd->devn_params.bitspercomponent;
     pdev->color_info.separable_and_linear = GX_CINFO_SEP_LIN;
-    pdev->icc_struct->supports_devn = true;
+    set_linear_color_bits_mask_shift(pdev);
+    if (pdev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+        pdev->icc_struct->supports_devn = false;
+    else
+        pdev->icc_struct->supports_devn = true;
     code = gdev_prn_open_planar(pdev, true);
     return code;
 }
@@ -1258,11 +1262,10 @@ psd_write_image_data(psd_write_ctx *xc, gx_device_printer *pdev)
     if (sep_line == NULL)
         return_error(gs_error_VMerror);
 
-    code = gx_downscaler_init_planar_trapped(&ds, (gx_device *)pdev, &params, num_comp,
-                                             psd_dev->downscale.downscale_factor, 0, bpc, bpc,
-                                             psd_dev->downscale.trap_w,
-                                             psd_dev->downscale.trap_h,
-                                             psd_dev->downscale.trap_order);
+    code = gx_downscaler_init_planar(&ds, (gx_device *)pdev,
+                                     bpc, bpc, num_comp,
+                                     &psd_dev->downscale,
+                                     &params);
     if (code < 0)
         goto cleanup;
 
