@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -14,6 +14,11 @@
 */
 
 
+#ifndef rinkj_byte_stream_h_INCLUDED
+#define rinkj_byte_stream_h_INCLUDED
+
+#include "gp.h"
+
 /* Bytestream abstraction for Rinkj driver. */
 
 typedef struct _RinkjByteStream RinkjByteStream;
@@ -35,4 +40,6 @@ int
 rinkj_byte_stream_close (RinkjByteStream *bs);
 
 RinkjByteStream *
-rinkj_byte_stream_file_new (FILE *f);
+rinkj_byte_stream_file_new (gp_file *f);
+
+#endif
