@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -16,6 +16,9 @@
  * Unicode code points.
  */
 
+#ifndef gdevagl_h_INCLUDED
+#define gdevagl_h_INCLUDED
+
 typedef struct single_glyph_list_s {
   const char *Glyph;
   unsigned short Unicode;
@@ -33,5 +36,7 @@ typedef struct treble_glyph_list_s {
 
 typedef struct quad_glyph_list_s {
   const char *Glyph;
-  short Unicode[4];
+  unsigned short Unicode[4];
 } quad_glyph_list_t;
+
+#endif
