@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -1132,6 +1132,10 @@ static int write_key_as_string_encrypted(const gx_device_pdf *pdev, const byte *
     memcpy(buffer, str, size);
     s_arcfour_process_buffer(&sarc4, buffer, size);
     stream_write(&sout, buffer, size);
+    /* Another case where we use sclose() and not s_close_filters(), because the
+     * buffer we supplied to s_init_filter is a heap based C object, so we
+     * must not free it.
+     */
     sclose(&sout); /* Writes ')'. */
     gs_free_object(pdev->pdf_memory, buffer, "Free encryption buffer");
     return 0;
@@ -1690,14 +1694,14 @@ static int hash_cos_stream(const cos_object_t *pco0, gs_md5_state_t *md5, gs_md5
 {
     const cos_stream_t *pcs = (const cos_stream_t *)pco0;
     cos_stream_piece_t *pcsp = pcs->pieces;
-    FILE *sfile = pdev->streams.file;
+    gp_file *sfile = pdev->streams.file;
     byte *ptr;
     int64_t position_save;
     int result;
 
     sflush(pdev->strm);
     sflush(pdev->streams.strm);
-    position_save = gp_ftell_64(sfile);
+    position_save = gp_ftell(sfile);
 
     if (!pcsp)
         return -1;
@@ -1709,10 +1713,10 @@ static int hash_cos_stream(const cos_object_t *pco0, gs_md5_state_t *md5, gs_md5
             result = gs_note_error(gs_error_VMerror);
             return result;
         }
-        if (gp_fseek_64(sfile, pcsp->position, SEEK_SET) != 0)
+        if (gp_fseek(sfile, pcsp->position, SEEK_SET) != 0)
 	  return_error(gs_error_ioerror);
 
-        if (fread(ptr, 1, pcsp->size, sfile) != pcsp->size) {
+        if (gp_fread(ptr, 1, pcsp->size, sfile) != pcsp->size) {
             gs_free(pdev->memory, ptr, sizeof (byte), pcsp->size, "hash_cos_stream");
             result = gs_note_error(gs_error_ioerror);
             return result;
@@ -1722,7 +1726,7 @@ static int hash_cos_stream(const cos_object_t *pco0, gs_md5_state_t *md5, gs_md5
         pcsp = pcsp->next;
     }
     gs_md5_finish(md5, (gs_md5_byte_t *)hash);
-    if (gp_fseek_64(sfile, position_save, SEEK_SET) != 0)
+    if (gp_fseek(sfile, position_save, SEEK_SET) != 0)
       return_error(gs_error_ioerror);
 
     return 0;
@@ -1805,7 +1809,7 @@ cos_stream_contents_write(const cos_stream_t *pcs, gx_device_pdf *pdev)
     cos_stream_piece_t *pcsp;
     cos_stream_piece_t *last;
     cos_stream_piece_t *next;
-    FILE *sfile = pdev->streams.file;
+    gp_file *sfile = pdev->streams.file;
     int64_t end_pos;
     bool same_file = (pdev->sbstack_depth > 0);
     int code;
@@ -1829,13 +1833,13 @@ cos_stream_contents_write(const cos_stream_t *pcs, gx_device_pdf *pdev)
             if (code < 0)
                 return code;
         } else {
-            end_pos = gp_ftell_64(sfile);
-            if (gp_fseek_64(sfile, pcsp->position, SEEK_SET) != 0)
+            end_pos = gp_ftell(sfile);
+            if (gp_fseek(sfile, pcsp->position, SEEK_SET) != 0)
 	      return_error(gs_error_ioerror);
             code = pdf_copy_data(s, sfile, pcsp->size, ss);
             if (code < 0)
                 return code;
-            if (gp_fseek_64(sfile, end_pos, SEEK_SET) != 0)
+            if (gp_fseek(sfile, end_pos, SEEK_SET) != 0)
 	      return_error(gs_error_ioerror);
         }
     }
