@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -41,7 +41,11 @@
  * Windows and Linux fail with coordinates outside +/- 16383. Hence, we
  * limit coordinates to 16k, with a little slop.
  */
-#define MAX_USER_COORD 16300
+/* 28/05/2020 This was only being applied to text and a subset of paths. Since
+ * Acrobat 4 is now more than 20 years old, lets just drop support for it. The
+ * PDF specification never had this limit, just Adobe's software.
+ */
+/* #define MAX_USER_COORD 16300 */
 
 /* ---------------- Statically allocated sizes ---------------- */
 /* These should really be dynamic.... */
@@ -326,6 +330,7 @@ typedef struct pdf_page_s {
     pdf_page_dsc_info_t dsc_info;
     bool NumCopies_set; /* ps2write only. */
     int NumCopies;      /* ps2write only. */
+    float UserUnit;     /* pdfwrite only */
 } pdf_page_t;
 #define private_st_pdf_page()        /* in gdevpdf.c */\
   gs_private_st_ptrs2(st_pdf_page, pdf_page_t, "pdf_page_t",\
@@ -479,8 +484,9 @@ struct pdf_font_cache_elem_s {
 typedef struct pdf_viewer_state_s {
     int transfer_not_identity;        /* bitmask */
     gs_id transfer_ids[4];
-    float opacity_alpha; /* state.opacity.alpha */
-    float shape_alpha; /* state.shape.alpha */
+    float strokeconstantalpha;
+    float fillconstantalpha;
+    bool alphaisshape;
     gs_blend_mode_t blend_mode; /* state.blend_mode */
     gs_id halftone_id;
     gs_id black_generation_id;
@@ -618,8 +624,8 @@ struct gx_device_pdf_s {
     pdf_page_dsc_info_t page_dsc_info; /* current page */
     /* Additional graphics state */
     bool fill_overprint, stroke_overprint;
+    int rendering_intent;
     bool remap_fill_color, remap_stroke_color;
-    int overprint_mode;
     gs_id halftone_id;
     gs_id transfer_ids[4];
     int transfer_not_identity;        /* bitmask */
@@ -900,6 +906,7 @@ struct gx_device_pdf_s {
                                      * doing JPEG pass through we write the JPEG data here, and don't write
                                      * anything in the image processing routines.
                                      */
+    float UserUnit;
 };
 
 #define is_in_page(pdev)\
@@ -960,6 +967,7 @@ extern const gx_device_vector_procs pdf_vector_procs;
 dev_proc_fill_rectangle(gdev_pdf_fill_rectangle);
 dev_proc_fill_path(gdev_pdf_fill_path);
 dev_proc_stroke_path(gdev_pdf_stroke_path);
+dev_proc_fill_stroke_path(gdev_pdf_fill_stroke_path);
 dev_proc_fillpage(gdev_pdf_fillpage);
     /* In gdevpdfi.c */
 dev_proc_begin_typed_image(gdev_pdf_begin_typed_image);
@@ -1164,7 +1172,12 @@ int pdf_copy_data_safe(stream *s, gp_file *file, gs_offset_t position, long coun
 /* Add the encryption filter. */
 int pdf_begin_encrypt(gx_device_pdf * pdev, stream **s, gs_id object_id);
 /* Remove the encryption filter. */
-void pdf_end_encrypt(gx_device_pdf * pdev);
+static int inline pdf_end_encrypt(gx_device_pdf *pdev)
+{
+    if (pdev->KeyLength)
+        return 1;
+    return 0;
+}
 /* Initialize encryption. */
 int pdf_encrypt_init(const gx_device_pdf * pdev, gs_id object_id, stream_arcfour_state *psarc4);
 
