@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -359,7 +359,7 @@ pdf_base_font_alloc(gx_device_pdf *pdev, pdf_base_font_t **ppbfont,
             font_name.size -= SUBSET_PREFIX_SIZE;
         }
     } else {
-        gs_sprintf(fnbuf, ".F%lx", (ulong)copied);
+        gs_sprintf(fnbuf, ".F" PRI_INTPTR, (intptr_t)copied);
         font_name.data = (byte *)fnbuf;
         font_name.size = strlen(fnbuf);
     }
@@ -509,7 +509,7 @@ pdf_write_FontFile_entry(gx_device_pdf *pdev, pdf_base_font_t *pbfont)
         FontFile_key = "/FontFile2";
         break;
     default:			/* Type 1/2, CIDFontType 0 */
-        if (pdev->ResourcesBeforeUsage)
+        if (!pdev->HaveCFF)
             FontFile_key = "/FontFile";
         else
             FontFile_key = "/FontFile3";
