@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -75,6 +75,7 @@ case 8: switch (pdfont->FontType) {
  case ft_MicroType:
  case ft_GL2_stick_user_defined:
  case ft_user_defined:
+ case ft_PDF_user_defined:
  case ft_GL2_531:
      ENUM_RETURN(pdfont->u.simple.v);
  case ft_CID_encrypted:
@@ -88,6 +89,7 @@ case 9: switch (pdfont->FontType) {
  case ft_MicroType:
  case ft_GL2_stick_user_defined:
  case ft_user_defined:
+ case ft_PDF_user_defined:
  case ft_GL2_531:
      ENUM_RETURN(pdfont->u.simple.s.type3.char_procs);
  case ft_CID_encrypted:
@@ -101,6 +103,7 @@ case 10: switch (pdfont->FontType) {
  case ft_MicroType:
  case ft_GL2_stick_user_defined:
  case ft_user_defined:
+ case ft_PDF_user_defined:
  case ft_GL2_531:
      ENUM_RETURN(pdfont->u.simple.s.type3.cached);
  case ft_CID_encrypted:
@@ -114,6 +117,7 @@ case 11: switch (pdfont->FontType) {
  case ft_MicroType:
  case ft_GL2_stick_user_defined:
  case ft_user_defined:
+ case ft_PDF_user_defined:
  case ft_GL2_531:
      ENUM_RETURN(pdfont->u.simple.s.type3.Resources);
  case ft_CID_encrypted:
@@ -144,6 +148,7 @@ RELOC_PTRS_WITH(pdf_font_resource_reloc_ptrs, pdf_font_resource_t *pdfont)
     case ft_MicroType:
     case ft_GL2_stick_user_defined:
     case ft_user_defined:
+    case ft_PDF_user_defined:
     case ft_GL2_531:
         RELOC_VAR(pdfont->u.simple.Encoding);
         RELOC_VAR(pdfont->u.simple.v);
@@ -452,6 +457,7 @@ int font_resource_free(gx_device_pdf *pdev, pdf_font_resource_t *pdfont)
         case ft_MicroType:
         case ft_GL2_stick_user_defined:
         case ft_user_defined:
+        case ft_PDF_user_defined:
         case ft_GL2_531:
             if(pdfont->u.simple.Encoding) {
                 gs_free_object(pdev->pdf_memory, pdfont->u.simple.Encoding, "Free simple Encoding");
@@ -566,6 +572,7 @@ font_resource_simple_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
     pfres->u.simple.BaseEncoding = -1;
     pfres->u.simple.preferred_encoding_index = -1;
     pfres->u.simple.last_reserved_char = -1;
+    pfres->TwoByteToUnicode = 1;
     *ppfres = pfres;
     return 0;
 }
