@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -81,11 +81,6 @@ append_text_move(pdf_text_state_t *pts, double dw)
     rounded = floor(dw + 0.5);
     if (fabs(dw - rounded) < 0.001)
         dw = rounded;
-    if (dw < -MAX_USER_COORD) {
-        /* Acrobat reader 4.0c, 5.0 can't handle big offsets.
-           Adobe Reader 6 can. */
-        return -1;
-    }
     if (dw != 0) {
         if (count == MAX_TEXT_BUFFER_MOVES)
             return -1;
@@ -173,47 +168,11 @@ add_text_delta_move(gx_device_pdf *pdev, const gs_matrix *pmat)
          * for the xhow and once for the Width override. Otherwise, we do
          * want to use TJ as it makes for smaller files.
          */
-        if (pts->can_use_TJ && dnotw == 0 && pts->buffer.count_chars > 0 &&
-            /*
-             * Acrobat Reader limits the magnitude of user-space
-             * coordinates.  Also, AR apparently doesn't handle large
-             * positive movement values (negative X displacements), even
-             * though the PDF Reference says this bug was fixed in AR3.
-             *
-             * Old revisions used the upper threshold 1000 for tdw,
-             * but it appears too big when a font sets a too big
-             * character width in setcachedevice. Particularly this happens
-             * with a Type 3 font generated by Aldus Freehand 4.0
-             * to represent a texture - see bug #687051.
-             * The problem is that when the Widths is multiplied
-             * to the font size, the viewer represents the result
-             * with insufficient fraction bits to represent the precise width.
-             * We work around that problem here restricting tdw
-             * with a smaller threshold 990. Our intention is to
-             * disable Tj when the real glyph width appears smaller
-             * than 1% of the width specified in setcachedevice.
-             * A Td instruction will be generated instead.
-             * Note that the value 990 is arbitrary and may need a
-             * further adjustment.
-             */
-             /* Revised the above. It seems unreasonable to use a fixed
-              * value which is not based on the point size, when the problem is
-              * caused by a large point size being multiplied by the width. The
-              * original fix also caused bitmap fonts (from PCL and other sources)
-              * to fail to use kerning, as these fonts are scaled to 1 point and
-              * therefore use large kerning values. Instead we check the kerned value
-              * multiplied by the point size of the font.
-              */
-            (tdw >= -MAX_USER_COORD && (tdw * pts->in.size) < MAX_USER_COORD)
-            ) {
+        if (pts->can_use_TJ && dnotw == 0 && pts->buffer.count_chars > 0) {
             /* Use TJ. */
             int code;
 
-            if (tdw < MAX_USER_COORD || pdev->CompatibilityLevel > 1.4)
-                code = append_text_move(pts, tdw);
-            else
-                return -1;
-
+            code = append_text_move(pts, tdw);
             if (code >= 0)
                 goto finish;
         }
@@ -815,7 +774,7 @@ int pdf_set_PaintType0_params (gx_device_pdf *pdev, gs_gstate *pgs, float size,
             if (code < 0)
                 return code;
             if (pdev->text->text_state->in.render_mode == ptsv->render_mode){
-                code = pdf_prepare_stroke(pdev, pgs);
+                code = pdf_prepare_stroke(pdev, pgs, false);
                 if (code >= 0)
                     code = gdev_vector_prepare_stroke((gx_device_vector *)pdev,
                                               pgs, NULL, NULL, 1);
