@@ -1,4 +1,4 @@
-/* Copyright (C) 2019 Artifex Software, Inc.
+/* Copyright (C) 2019-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -105,8 +105,7 @@ static const pl_interp_characteristics_t jbig2_characteristics = {
     jbig2_detect_language,
     "Artifex",
     "0.01",
-    "18 Nov 2019",
-    1 /* minimum input size */
+    "18 Nov 2019"
 };
 
 /* Get implementation's characteristics */
@@ -370,7 +369,7 @@ static JB2_Error JB2_Callback my_output(unsigned char *buffer,
 }
 
 #else
-static void my_errors(void *data, const char *msg, Jbig2Severity severity, int32_t seg_idx)
+static void my_errors(void *data, const char *msg, Jbig2Severity severity, uint32_t seg_idx)
 {
     /* Do nothing */
 }
