@@ -1,4 +1,4 @@
-/* Copyright (C) 2019 Artifex Software, Inc.
+/* Copyright (C) 2019-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -119,8 +119,7 @@ static const pl_interp_characteristics_t jp2k_characteristics = {
     jp2k_detect_language,
     "Artifex",
     "0.01",
-    "22 Nov 2019",
-    1 /* minimum input size */
+    "22 Nov 2019"
 };
 
 /* Get implementation's characteristics */
