@@ -1,4 +1,4 @@
-/* Copyright (C) 2019 Artifex Software, Inc.
+/* Copyright (C) 2019-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -104,7 +104,6 @@ jpg_detect_language(const char *s, int len)
             strncmp("JFIF", s+6, 5) == 0)
             return 100;
     }
-    /* FIXME: Other formats go here */
 
     return 0;
 }
@@ -114,8 +113,7 @@ static const pl_interp_characteristics_t jpg_characteristics = {
     jpg_detect_language,
     "Artifex",
     "0.01",
-    "11 Nov 2019",
-    1 /* minimum input size */
+    "11 Nov 2019"
 };
 
 /* GS's fakakta jpeg integration insists on putting a
