@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -83,14 +83,17 @@ build_foreground(pcl_state_t * pcs,
     bool is_default = false;
     int code = 0;
 
+    if (!pindexed) {
+        return_error(gs_error_invalidaccess);
+    }
+
     /*
      * Check for a request for the default foreground. Since there are only
      * three fixed palettes, it is sufficient to check that the palette provided
      * is fixed and has two entries. The default foreground is black, which is
      * the second of the two entries.
      */
-    if ((pindexed != 0) &&
-        (pindexed->pfixed) && (pindexed->num_entries == 2) && (pal_entry == 1)) {
+    if ((pindexed->pfixed) && (pindexed->num_entries == 2) && (pal_entry == 1)) {
         is_default = true;
         if (pcs->pdflt_frgrnd != 0) {
             pcl_frgrnd_copy_from(*ppfrgrnd, pcs->pdflt_frgrnd);
