@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -401,7 +401,8 @@ new_page_size(pcl_state_t * pcs,
      * an underline to be put out.
      */
     pcs->underline_enabled = false;
-    pcl_home_cursor(pcs);
+    code = pcl_home_cursor(pcs);
+    if (code < 0) return code;
     /*
      * this is were we initialized the cursor position
      */
@@ -602,9 +603,9 @@ pcl_cursor_moved(pcl_state_t * pcs)
 int
 pcl_end_page(pcl_state_t * pcs, pcl_print_condition_t condition)
 {
-    int code = 0;
-
-    pcl_break_underline(pcs);   /* (could mark page) */
+    int code = pcl_break_underline(pcs);   /* (could mark page) */
+    if (code < 0)
+        return code;
 
     /* If we are conditionally printing (normal case) check if the
        page is marked */
@@ -694,7 +695,9 @@ set_page_size(pcl_args_t * pargs, pcl_state_t * pcs)
     code = pcl_end_page_if_marked(pcs);
     if (code < 0)
         return code;
-    pcl_home_cursor(pcs);
+    code = pcl_home_cursor(pcs);
+    if (code < 0)
+        return code;
 
     for (i = 0; i < pcl_paper_type_count; i++) {
         if (tag == PAPER_SIZES[i].tag) {
@@ -724,7 +727,9 @@ set_paper_source(pcl_args_t * pargs, pcl_state_t * pcs)
 
     if (code < 0)
         return code;
-    pcl_home_cursor(pcs);
+    code = pcl_home_cursor(pcs);
+    if (code < 0)
+        return code;
     /* Do not change the page side if the wanted paper source is the same as the actual one */
     if (pcs->paper_source != i) {
         pcs->back_side = false;
@@ -822,7 +827,9 @@ set_print_direction(pcl_args_t * pargs, pcl_state_t * pcs)
     if ((i <= 270) && (i % 90 == 0)) {
         i /= 90;
         if (i != pcs->xfm_state.print_dir) {
-            pcl_break_underline(pcs);
+            int code = pcl_break_underline(pcs);
+            if (code < 0)
+                return code;
             pcs->xfm_state.print_dir = i;
             update_xfm_state(pcs, 0);
             pcl_continue_underline(pcs);
@@ -856,9 +863,9 @@ set_left_margin(pcl_args_t * pargs, pcl_state_t * pcs)
     if (lmarg < pcs->margins.right) {
         pcs->margins.left = lmarg;
         if (pcs->cap.x < lmarg)
-            pcl_set_cap_x(pcs, lmarg, false, false);
+            code = pcl_set_cap_x(pcs, lmarg, false, false);
     }
-    return 0;
+    return code;
 }
 
 /*
@@ -881,10 +888,10 @@ set_right_margin(pcl_args_t * pargs, pcl_state_t * pcs)
     if (rmarg > pcs->margins.left) {
         pcs->margins.right = rmarg;
         if (pcs->cap.x > rmarg)
-            pcl_set_cap_x(pcs, rmarg, false, false);
+            code = pcl_set_cap_x(pcs, rmarg, false, false);
     }
 
-    return 0;
+    return code;
 }
 
 /*
@@ -980,7 +987,7 @@ pcl_media_type(pcl_args_t * pargs, pcl_state_t * pcs)
         int code = pcl_end_page_if_marked(pcs);
 
         if (code >= 0)
-            pcl_home_cursor(pcs);
+            code = pcl_home_cursor(pcs);
         return (code < 0 ? code : e_Unimplemented);
     } else
         return e_Range;
@@ -1147,7 +1154,7 @@ pcl_print_quality(pcl_args_t * pargs, pcl_state_t * pcs)
         int code = pcl_end_page_if_marked(pcs);
 
         if (code >= 0)
-            pcl_home_cursor(pcs);
+            code = pcl_home_cursor(pcs);
         return (code < 0 ? code : 0);
     } else
         return e_Range;
