@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -25,6 +25,7 @@
 #include "pcursor.h"
 #include "rtmisc.h"
 #include "rtgmode.h"
+#include "rtraster.h"
 
 /* We don't know whether an Enable Display Functions takes effect while */
 /* defining a macro.  To play it safe, we're providing both options. */
@@ -700,6 +701,10 @@ pcl_process(pcl_parser_state_t * pst, pcl_state_t * pcs,
         if (mcode < 0 && code >= 0)
             code = mcode;
     }
+    if (code < 0) {
+        /* This avoids memento squeeze leaks. */
+        (void) pcl_complete_raster(pcs);
+    }
     return code;
 }
 
