@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -40,6 +40,7 @@
 #include "pcpalet.h"
 #include "rtgmode.h"
 #include "gsicc_manage.h"
+#include "pcparam.h"
 
 /* Configuration table for modules */
 extern const pcl_init_t pcparse_init;
@@ -143,7 +144,7 @@ pcl_gstate_client_copy_for(void *to,
 }
 
 static void
-pcl_gstate_client_free(void *old, gs_memory_t * mem)
+pcl_gstate_client_free(void *old, gs_memory_t * mem, gs_gstate *pgs)
 {
 }
 
@@ -160,26 +161,52 @@ static const gs_gstate_client_procs pcl_gstate_procs = {
 
 typedef struct pcl_interp_instance_s
 {
-    pl_interp_implementation_t pl;    /* common part: must be first */
     gs_memory_t *memory;        /* memory allocator to use */
     pcl_state_t pcs;            /* pcl state */
     pcl_parser_state_t pst;     /* parser state */
 } pcl_interp_instance_t;
 
-/* Get implemtation's characteristics */
+static int
+pcl_detect_language(const char *s, int length)
+{
+    int count;
+    int len;
+
+    if (length < 2)
+        return 0;
+    if (s[0] == 27) {
+        if (s[1] == 'E')
+            return 100;
+        return 80;
+    }
+
+    /* Count the number of ESC's */
+    for (count = 0, len = length; len > 0; len--, s++)
+    {
+        if (*s == 27)
+            count++;
+    }
+
+    if (count > 10 || count > length/20)
+        return 80;
+    if (count > 0)
+        return 20;
+
+    return 0;
+}
+
+/* Get implementation's characteristics */
 static const pl_interp_characteristics_t *      /* always returns a descriptor */
-pcl_impl_characteristics(const pl_interp_implementation_t * impl        /* implementation of interpereter to alloc */
-    )
+pcl_impl_characteristics(const pl_interp_implementation_t * impl)        /* implementation of interpreter to alloc */
 {
 #define PCLVERSION NULL
 #define PCLBUILDDATE NULL
     static pl_interp_characteristics_t pcl_characteristics = {
         "PCL",
-        "\033E",
+        pcl_detect_language,
         "Artifex",
         PCLVERSION,
-        PCLBUILDDATE,
-        17                      /* size of min buffer == sizeof UEL */
+        PCLBUILDDATE
     };
     return &pcl_characteristics;
 }
@@ -218,6 +245,8 @@ pcl_impl_allocate_interp_instance(pl_interp_implementation_t *impl,
         return gs_error_VMerror;
     }
 
+    memset(&pcli->pcs, 0, sizeof(pcl_state_t));
+
     gsicc_init_iccmanager(pgs);
 
     pcli->memory = mem;
@@ -234,12 +263,17 @@ pcl_impl_allocate_interp_instance(pl_interp_implementation_t *impl,
     {
         int code = pcl_do_registrations(&pcli->pcs, &pcli->pst);
 
-        if (code < 0)
+        if (code < 0) {
+            if (pcli->pcs.pids != NULL)
+                gs_free_object(mem, pcli->pcs.pids, "PCL gsave");
+            gs_gstate_free(pgs);
+            gs_free_object(mem, pcli, "pcl_allocate_interp_instance(pcl_interp_instance_t)");
             return (code);
+        }
     }
 
     pcli->pcs.pjls = pl_main_get_pjl_instance(mem);
-    
+
     /* Return success */
     impl->interp_client_data = pcli;
     return 0;
@@ -252,12 +286,12 @@ pcl_get_personality(pl_interp_implementation_t * impl, gx_device * device)
 {
     pcl_interp_instance_t *pcli  = impl->interp_client_data;
     char *personality = pl_main_get_pcl_personality(pcli->memory);
-    
+
     if (!strcmp(personality, "PCL5C"))
         return pcl5c;
     else if (!strcmp(personality, "PCL5E"))
         return pcl5e;
-    /* 
+    /*
      * match RTL or any string containing "GL" we see many variants in
      * test files: HPGL/2, HPGL2 etc.
      */
@@ -278,17 +312,14 @@ pcl_set_icc_params(pl_interp_implementation_t * impl, gs_gstate * pgs)
     return pl_set_icc_params(pcli->memory, pgs);
 }
 
-/* Set a device into an interperter instance */
+/* Prepare interp instance for the next "job" */
 static int                      /* ret 0 ok, else -ve error code */
-pcl_impl_set_device(pl_interp_implementation_t * impl,    /* interp instance to use */
-                    gx_device * device  /* device to set (open or closed) */
-    )
+pcl_impl_init_job(pl_interp_implementation_t * impl,       /* interp instance to start job in */
+                  gx_device                  * device)
 {
-
-    int code;
+    int code = 0;
     pcl_interp_instance_t *pcli = impl->interp_client_data;
     gs_memory_t *mem = pcli->memory;
-    
     enum
     { Sbegin, Ssetdevice, Sinitg, Sgsave1, Spclgsave, Sreset, Serase,
             Sdone } stage;
@@ -338,10 +369,14 @@ pcl_impl_set_device(pl_interp_implementation_t * impl,    /* interp instance to
     if ((code = gs_erasepage(pcli->pcs.pgs)) < 0)
         goto pisdEnd;
 
-    /* Do device-dependent pcl inits */
+    /* Initialize the PCL interpreter and parser */
     stage = Sreset;
     if ((code = pcl_do_resets(&pcli->pcs, pcl_reset_initial)) < 0)
         goto pisdEnd;
+
+    if ((code = pcl_process_init(&pcli->pst, &pcli->pcs)) < 0)
+        goto pisdEnd;
+
     /* provide a PCL graphic state we can return to */
     stage = Spclgsave;
     if ((code = pcl_gsave(&pcli->pcs)) < 0)
@@ -360,31 +395,37 @@ pcl_impl_set_device(pl_interp_implementation_t * impl,    /* interp instance to
         case Sreset:           /* pcl_do_resets failed */
         case Serase:           /* gs_erasepage failed */
             /* undo 1st gsave */
-            gs_grestore_only(pcli->pcs.pgs);    /* destroys gs_save stack */
+
+            if (gs_grestore_only(pcli->pcs.pgs) < 0)    /* destroys gs_save stack */
+                return_error(gs_error_Fatal);
             /* fall thru to next */
 
         case Sgsave1:          /* 1st gsave failed */
             /* undo setdevice */
-            gs_nulldevice(pcli->pcs.pgs);
+            if (gs_nulldevice(pcli->pcs.pgs) < 0)
+                return_error(gs_error_Fatal);
             /* fall thru to next */
 
         case Ssetdevice:       /* gs_setdevice failed */
         case Sbegin:           /* nothing left to undo */
             break;
     }
+
+    /* Warn the device we use ROPs */
+    if (code == 0) {
+        code = put_param1_bool(&pcli->pcs, "LanguageUsesROPs", true);
+        if (!device->is_open)
+            code = gs_opendevice(device);
+    }
+
     return code;
 }
 
-/* Prepare interp instance for the next "job" */
-static int                      /* ret 0 ok, else -ve error code */
-pcl_impl_init_job(pl_interp_implementation_t * impl       /* interp instance to start job in */
-    )
+/* Do any setup for parser per-cursor */
+static int                      /* ret 0 or +ve if ok, else -ve error code */
+pcl_impl_process_begin(pl_interp_implementation_t * impl)
 {
-    int code = 0;
-    pcl_interp_instance_t *pcli = impl->interp_client_data;
-
-    pcl_process_init(&pcli->pst);
-    return code;
+    return 0;
 }
 
 /* Parse a cursor-full of data */
@@ -399,6 +440,12 @@ pcl_impl_process(pl_interp_implementation_t * impl,       /* interp instance to
     return code;
 }
 
+static int                      /* ret 0 or +ve if ok, else -ve error code */
+pcl_impl_process_end(pl_interp_implementation_t * impl)
+{
+    return 0;
+}
+
 /* Skip to end of job ret 1 if done, 0 ok but EOJ not found, else -ve error code */
 static int
 pcl_impl_flush_to_eoj(pl_interp_implementation_t * impl,  /* interp instance to flush for */
@@ -429,15 +476,12 @@ static int                      /* ret 0 or +ve if ok, else -ve error code */
 pcl_impl_process_eof(pl_interp_implementation_t * impl    /* interp instance to process data job in */
     )
 {
-    int code;
     pcl_interp_instance_t *pcli = impl->interp_client_data;
-
-    pcl_process_init(&pcli->pst);
-    code = pcl_end_page_if_marked(&pcli->pcs);
+    int code = pcl_process_init(&pcli->pst, &pcli->pcs);
     if (code < 0)
         return code;
     /* force restore & cleanup if unexpected data end was encountered */
-    return 0;
+    return pcl_end_page_if_marked(&pcli->pcs);
 }
 
 /* Report any errors after running a job */
@@ -460,24 +504,12 @@ pcl_impl_report_errors(pl_interp_implementation_t * impl, /* interp instance to
 
 /* Wrap up interp instance after a "job" */
 static int                      /* ret 0 ok, else -ve error code */
-pcl_impl_dnit_job(pl_interp_implementation_t * impl       /* interp instance to wrap up job in */
-    )
+pcl_impl_dnit_job(pl_interp_implementation_t * impl)       /* interp instance to wrap up job in */
 {
     pcl_interp_instance_t *pcli = impl->interp_client_data;
     pcl_state_t *pcs = &pcli->pcs;
-
-    if (pcs->raster_state.graphics_mode)
-        pcl_end_graphics_mode(pcs);
-    return 0;
-}
-
-/* Remove a device from an interperter instance */
-static int                      /* ret 0 ok, else -ve error code */
-pcl_impl_remove_device(pl_interp_implementation_t * impl  /* interp instance to use */
-    )
-{
     int code;
-    pcl_interp_instance_t *pcli = impl->interp_client_data;
+    gx_device *device = gs_currentdevice(pcs->pgs);
 
     /* Note: "PCL" grestore. */
     code = pcl_grestore(&pcli->pcs);
@@ -489,7 +521,22 @@ pcl_impl_remove_device(pl_interp_implementation_t * impl  /* interp instance to
     if (code < 0)
         return code;
 
-    return pcl_do_resets(&pcli->pcs, pcl_reset_permanent);
+    code = pcl_do_resets(&pcli->pcs, pcl_reset_permanent);
+    if (code < 0)
+        return code;
+
+    if (pcs->raster_state.graphics_mode)
+        code = pcl_end_graphics_mode(pcs);
+
+    if (code >= 0) {
+        /* Warn the device that ROP usage has come to an end */
+        code = put_param1_bool(&pcli->pcs, "LanguageUsesROPs", false);
+
+        if (!device->is_open)
+            code = gs_opendevice(device);
+    }
+
+    return code;
 }
 
 /* Deallocate a interpreter instance */
@@ -536,15 +583,20 @@ pcl_end_page_top(pcl_state_t * pcs, int num_copies, int flush)
 pl_interp_implementation_t pcl_implementation = {
     pcl_impl_characteristics,
     pcl_impl_allocate_interp_instance,
-    pcl_impl_set_device,
+    NULL,                       /* get_device_memory */
+    NULL,                       /* set_param */
+    NULL,                       /* add_path */
+    NULL,                       /* post_args_init */
     pcl_impl_init_job,
+    NULL,                       /* run_prefix_commands */
     NULL,                       /* process_file */
+    pcl_impl_process_begin,
     pcl_impl_process,
+    pcl_impl_process_end,
     pcl_impl_flush_to_eoj,
     pcl_impl_process_eof,
     pcl_impl_report_errors,
     pcl_impl_dnit_job,
-    pcl_impl_remove_device,
     pcl_impl_deallocate_interp_instance,
     NULL
 };
