@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -638,7 +638,7 @@ hpgl_SC(hpgl_args_t * pargs, hpgl_state_t * pgls)
             return e_Range;
         case 4:
             type = hpgl_scaling_anisotropic;
-            hpgl_arg_c_int(pgls->memory, pargs, &type);
+            (void) hpgl_arg_c_int(pgls->memory, pargs, &type);
             switch (type) {
                 case hpgl_scaling_anisotropic: /* 0 */
                     if (xy[0] == xy[1] || xy[2] == xy[3])
