@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -440,7 +440,7 @@ hpgl_set_graphics_line_attribute_state(hpgl_state_t * pgls,
      * is not documented PCLTRM.  Pen widths are maintained in
      * plotter units
      */
-    if (render_mode != hpgl_rm_character && pen_wid <= 14.0) {
+    if (pen_wid <= 14.0) {
         hpgl_call(gs_setlinejoin(pgls->pgs, gs_join_none));
         hpgl_call(gs_setlinecap(pgls->pgs, gs_cap_butt));
         hpgl_call(gs_setlinewidth(pgls->pgs, pen_wid));
@@ -450,7 +450,6 @@ hpgl_set_graphics_line_attribute_state(hpgl_state_t * pgls,
 
     switch (render_mode) {
 
-        case hpgl_rm_character:
         case hpgl_rm_polygon:
         case hpgl_rm_clip_and_fill_polygon:
             hpgl_call(gs_setlinejoin(pgls->pgs, gs_join_round));
@@ -839,6 +838,7 @@ hpgl_polyfill(hpgl_state_t * pgls, hpgl_rendering_mode_t render_mode)
 static int
 hpgl_fill_polyfill_background(hpgl_state_t * pgls)
 {
+    int code, code2;
     /* conditionally mark page as dirty */
     hpgl_call(pcl_mark_page_for_path(pgls));
     /* if we are drawing on a transparent background */
@@ -847,17 +847,20 @@ hpgl_fill_polyfill_background(hpgl_state_t * pgls)
     /* preserve the current foreground color */
     hpgl_call(hpgl_gsave(pgls));
     /* fill a white region. */
-    hpgl_call(gs_setgray(pgls->pgs, 1.0));
-    hpgl_call(gs_fill(pgls->pgs));
+    code = gs_setgray(pgls->pgs, 1.0);
+    if (code >= 0)
+        code = gs_fill(pgls->pgs);
     /* restore the foreground color */
-    hpgl_call(hpgl_grestore(pgls));
-    return 0;
+    code2 = hpgl_grestore(pgls);
+    return (code < 0) ? code : code2;
 }
 
 static int
 hpgl_polyfill_using_current_line_type(hpgl_state_t * pgls,
                                       hpgl_rendering_mode_t render_mode)
 {
+    int code;
+
     /* gsave and grestore used to preserve the clipping region */
     hpgl_call(hpgl_gsave(pgls));
 
@@ -866,13 +869,17 @@ hpgl_polyfill_using_current_line_type(hpgl_state_t * pgls,
      * beginning at the anchor corner replicate lines
      */
     if (pgls->g.fill_type == hpgl_even_odd_rule)
-        hpgl_call(gs_eoclip(pgls->pgs));
+        code = gs_eoclip(pgls->pgs);
     else
-        hpgl_call(gs_clip(pgls->pgs));
-    hpgl_call(hpgl_fill_polyfill_background(pgls));
-    hpgl_call(hpgl_polyfill(pgls, render_mode));
-    hpgl_call(hpgl_grestore(pgls));
-    return 0;
+        code = gs_clip(pgls->pgs);
+    if (code >= 0)  code = hpgl_fill_polyfill_background(pgls);
+    if (code >= 0)  code = hpgl_polyfill(pgls, render_mode);
+
+    {
+        int code2 = hpgl_grestore(pgls);
+        if (code >= 0)  code = code2;
+        return code;
+    }
 }
 
 static gs_rop3_t
@@ -1565,7 +1572,7 @@ hpgl_draw_current_path(hpgl_state_t * pgls, hpgl_rendering_mode_t render_mode)
                                        min(scale.x,
                                            scale.y) * 0.0375 * 0.2835));
                         }
-                        pcl_mark_page_for_path(pgls);
+                        hpgl_call(pcl_mark_page_for_path(pgls));
                         hpgl_call(gs_stroke(pgls->pgs));
                         break;
 
@@ -1587,11 +1594,17 @@ hpgl_draw_current_path(hpgl_state_t * pgls, hpgl_rendering_mode_t render_mode)
                         if ((pgls->g.fill.type != hpgl_FT_pattern_one_line) &&
                             (pgls->g.fill.type !=
                              hpgl_FT_pattern_two_lines)) {
+                            int code2 = 0;
                             hpgl_call(hpgl_gsave(pgls));
                             /* all character fills appear to have 0 fill adjustment */
                             gs_setfilladjust(pgls->pgs, 0, 0);
-                            hpgl_call((*fill) (pgs));
-                            hpgl_call(hpgl_grestore(pgls));
+                            /* Always restore before handling any error. */
+                            code = (*fill) (pgs);
+                            code2 = hpgl_grestore(pgls);
+                            if (code >=0 && code2 < 0) {
+                                code = code2;
+                            }
+                            hpgl_call(code);
                         }
                         if ((pgls->g.bitmap_fonts_allowed) || (hpgl_is_currentfont_stick_or_arc(pgls))) /* no edging */
                             hpgl_call(hpgl_clear_current_path(pgls));
