@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -599,7 +599,7 @@ hpgl_init_label_buffer(hpgl_state_t * pgls)
 }
 
 /* release the character buffer */
-static int
+void
 hpgl_destroy_label_buffer(hpgl_state_t * pgls)
 {
     gs_free_object(pgls->memory, pgls->g.label.buffer,
@@ -607,7 +607,6 @@ hpgl_destroy_label_buffer(hpgl_state_t * pgls)
     pgls->g.label.char_count = 0;
     pgls->g.label.buffer_size = 0;
     pgls->g.label.buffer = 0;
-    return 0;
 }
 
 /* add a single character to the line buffer */
@@ -793,7 +792,7 @@ hpgl_print_char(hpgl_state_t * pgls, uint ch)
         bool use_show = hpgl_use_show(pgls, font);
         gs_matrix pre_rmat, advance_mat;
         int angle = 0;
-        gs_text_enum_t *penum;
+        gs_text_enum_t *penum = NULL;
         int code;
         gs_point start_pt, end_pt;
         hpgl_real_t space_width;
@@ -944,11 +943,13 @@ hpgl_print_char(hpgl_state_t * pgls, uint ch)
                 /* we just check the current position for
                    "insidedness" - this seems to address the dirty
                    page issue in practice. */
-                pcl_mark_page_for_current_pos(pgls);
-                gs_text_release(penum, "hpgl_print_char");
+                code = pcl_mark_page_for_current_pos(pgls);
             }
-            if (code < 0)
+            gs_text_release(penum, "hpgl_print_char");
+            if (code < 0) {
+                hpgl_free_stick_fonts(pgls);
                 return code;
+            }
             gs_setmatrix(pgs, &advance_mat);
             if (angle >= 0) {
                 /* Compensate for bitmap font non-rotation. */
@@ -1054,7 +1055,7 @@ hpgl_get_character_origin_offset(hpgl_state_t * pgls, int origin,
                                  gs_point * offset)
 {
     double pos_x = 0.0, pos_y = 0.0;
-    double off_x, off_y;
+    double off_x = 0.0, off_y = 0.0;
     hpgl_real_t adjusted_height = height;
 
 #ifdef CHECK_UNIMPLEMENTED
@@ -1373,8 +1374,8 @@ hpgl_process_buffer(hpgl_state_t * pgls, gs_point * offset)
                                 if (pgls->g.bitmap_fonts_allowed)       /* no mirroring */
                                     label_advance = fabs(label_advance);
                             }
-                            hpgl_move_cursor_by_characters(pgls, 0, -1,
-                                                           &label_advance);
+                            hpgl_call(hpgl_move_cursor_by_characters(pgls, 0, -1,
+                                                           &label_advance));
                             continue;
                         }
                         spaces = 0, lines = -1;
@@ -1399,8 +1400,8 @@ hpgl_process_buffer(hpgl_state_t * pgls, gs_point * offset)
                     default:
                         goto print;
                 }
-                hpgl_move_cursor_by_characters(pgls, spaces, lines,
-                                               (const hpgl_real_t *)0);
+                hpgl_call(hpgl_move_cursor_by_characters(pgls, spaces, lines,
+                                               (const hpgl_real_t *)0));
                 continue;
             }
           print:{
@@ -1452,13 +1453,18 @@ hpgl_LB(hpgl_args_t * pargs, hpgl_state_t * pgls)
     const byte *p = pargs->source.ptr;
     const byte *rlimit = pargs->source.limit;
     bool print_terminator = pgls->g.label.print_terminator;
+    int code = 0;
 
     if (pargs->phase == 0) {
         /* initialize the character buffer and CTM first time only */
         hpgl_call(hpgl_draw_current_path(pgls, hpgl_rm_vector));
         hpgl_call(hpgl_init_label_buffer(pgls));
         hpgl_call(hpgl_set_ctm(pgls));
-        hpgl_call(hpgl_set_clipping_region(pgls, hpgl_rm_vector));
+        code = hpgl_set_clipping_region(pgls, hpgl_rm_vector);
+        if (code < 0) {
+            hpgl_destroy_label_buffer(pgls);
+            return code;
+        }
         pgls->g.label.initial_pos = pgls->g.pos;
         hpgl_call(hpgl_add_point_to_path
                   (pgls, pgls->g.pos.x, pgls->g.pos.y,
@@ -1481,35 +1487,41 @@ hpgl_LB(hpgl_args_t * pargs, hpgl_state_t * pgls)
             if (!print_terminator) {
                 gs_point lo_offsets;
 
-                hpgl_call(hpgl_process_buffer(pgls, &lo_offsets));
-                hpgl_call(hpgl_destroy_label_buffer(pgls));
+                code = hpgl_process_buffer(pgls, &lo_offsets);
+                hpgl_destroy_label_buffer(pgls);
+                if (code < 0) {
+                    hpgl_free_stick_fonts(pgls);
+                    return code;
+                }
                 pargs->source.ptr = p;
                 /*
                  * Depending on the DV/LO combination, conditionally
                  * restore the initial position, per TRM 23-78.
                  */
                 if (!hpgl_can_concat_labels(pgls)) {
-                    hpgl_call(hpgl_add_point_to_path(pgls,
+                    code = hpgl_add_point_to_path(pgls,
                                                      pgls->g.
                                                      carriage_return_pos.x,
                                                      pgls->g.
                                                      carriage_return_pos.y,
                                                      hpgl_plot_move_absolute,
-                                                     true));
+                                                     true);
                 } else {
                     /* undo the label origin offsets */
-                    hpgl_call(hpgl_add_point_to_path
+                    code = hpgl_add_point_to_path
                               (pgls, lo_offsets.x, lo_offsets.y,
-                               hpgl_plot_move_relative, false));
+                               hpgl_plot_move_relative, false);
                 }
-                /* clear the current path terminating carriage
-                   returns and linefeeds will leave "moveto's" in
-                   the path */
-                hpgl_call(hpgl_clear_current_path(pgls));
                 /* also clean up stick fonts - they are likely to
                    become dangling references in the current font
                    scheme since they don't have a dictionary entry */
                 hpgl_free_stick_fonts(pgls);
+                if (code < 0)
+                    return code;
+                /* clear the current path terminating carriage
+                   returns and linefeeds will leave "moveto's" in
+                   the path */
+                hpgl_call(hpgl_clear_current_path(pgls));
                 return 0;
             }
             /*
@@ -1529,9 +1541,14 @@ hpgl_LB(hpgl_args_t * pargs, hpgl_state_t * pgls)
         if (GL_LB_CH == CR && !pgls->g.transparent_data) {
             gs_point lo_offsets;
 
-            hpgl_call(hpgl_process_buffer(pgls, &lo_offsets));
-            hpgl_call(hpgl_destroy_label_buffer(pgls));
-            hpgl_call(hpgl_init_label_buffer(pgls));
+            code = hpgl_process_buffer(pgls, &lo_offsets);
+            hpgl_destroy_label_buffer(pgls);
+            if (code >= 0)
+                code = hpgl_init_label_buffer(pgls);
+            if (code < 0) {
+                hpgl_free_stick_fonts(pgls);
+                return code;
+            }
         }
     }
     pargs->source.ptr = p;
@@ -1558,6 +1575,8 @@ hpgl_free_stick_fonts(hpgl_state_t * pgls)
     pl_font_t *font = &pgls->g.stick_font[pgls->g.font_selected]
         [pfs->params.proportional_spacing];
 
+    pl_font_glyph_width_cache_remove_nodes(font);
+
     /* no stick fonts - nothing to do */
     if (font->pfont == 0)
         return;
@@ -1575,23 +1594,30 @@ hpgl_print_symbol_mode_char(hpgl_state_t * pgls)
     int saved_origin = pgls->g.label.origin;
     gs_point save_pos = pgls->g.pos;
     gs_point lo_offsets;
+    int code;
 
     hpgl_call(hpgl_gsave(pgls));
     /* HAS this need checking.  I don't know how text direction
        and label origin interact in symbol mode */
     pgls->g.label.origin = 5;
     /* HAS - alot of work for one character */
-    hpgl_call(hpgl_clear_current_path(pgls));
-    hpgl_call(hpgl_init_label_buffer(pgls));
-    hpgl_call(hpgl_buffer_char(pgls, pgls->g.symbol_mode));
-    hpgl_call(hpgl_process_buffer(pgls, &lo_offsets));
-    hpgl_call(hpgl_destroy_label_buffer(pgls));
-    hpgl_call(hpgl_grestore(pgls));
+    if (((code = hpgl_clear_current_path(pgls)) >= 0) &&
+        ((code = hpgl_init_label_buffer(pgls)) >= 0)) {
+        if ((code = hpgl_buffer_char(pgls, pgls->g.symbol_mode)) >= 0)
+            code = hpgl_process_buffer(pgls, &lo_offsets);
+        hpgl_destroy_label_buffer(pgls);
+    }
+    {
+        /* must grestore here from earlier gsave */
+        int code2 = hpgl_grestore(pgls);
+        if (code >= 0) code = code2;
+    }
+    hpgl_free_stick_fonts(pgls);
     /* restore the origin */
     pgls->g.label.origin = saved_origin;
-    hpgl_call(hpgl_set_current_position(pgls, &save_pos));
-    hpgl_free_stick_fonts(pgls);
-    return 0;
+    if (code >= 0)
+        hpgl_call(hpgl_set_current_position(pgls, &save_pos));
+    return code;
 }
 
 /* Initialization */
