@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -324,10 +324,10 @@ pjl_side_effects(pjl_parser_state_t * pst, char *variable, char *value,
                            (int)(pjl_media[indx].width) :
                            (int)(pjl_media[indx].height));
         int formlines = pjl_calc_formlines_new_page_size(page_length);
-        char value[32];
+        char text[32];
 
-        gs_sprintf(value, "%d", formlines);
-        pjl_set(pst, (char *)"FORMLINES", value, defaults);
+        gs_sprintf(text, "%d", formlines);
+        pjl_set(pst, (char *)"FORMLINES", text, defaults);
     }
     /* fill in other side effects here */
     return;
@@ -825,7 +825,7 @@ pjl_delete_file(pjl_parser_state_t * pst, char *pathname)
     pjl_parsed_filename_to_string(fname, pathname);
     if (pjl_verify_file_operation(pst, fname) < 0)
         return -1;
-    return unlink(fname);
+    return gp_unlink(pst->mem, fname);
 }
 
 /* handle pattern foo = setting, e.g. volume = "0:", name = 0:]pcl.
@@ -1183,14 +1183,20 @@ pjl_get_named_resource(pjl_parser_state * pst, char *name, byte * data)
 
 /* set the initial environment to the default environment, this should
    be done at the beginning of each job */
-void
+int
 pjl_set_init_from_defaults(pjl_parser_state_t * pst)
 {
-    free_pjl_environment(pst->mem, &pst->envir);
-    set_pjl_environment(pst->mem, &pst->envir, pst->defaults);
+    int code = free_pjl_environment(pst->mem, &pst->envir);
+    if (code < 0)
+        return code;
+    code = set_pjl_environment(pst->mem, &pst->envir, pst->defaults);
+    if (code < 0)
+        return code;
 
-    free_pjl_fontsource(pst->mem, &pst->font_envir);
-    set_pjl_fontsource(pst->mem, &pst->font_envir, pst->font_defaults);
+    code = free_pjl_fontsource(pst->mem, &pst->font_envir);
+    if (code < 0)
+        return code;
+    return set_pjl_fontsource(pst->mem, &pst->font_envir, pst->font_defaults);
 }
 
 void
