@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -50,8 +50,7 @@ pjl_impl_characteristics(const pl_interp_implementation_t * impl)        /* impl
         pjl_detect_language,
         "Artifex",
         PJLVERSION,
-        PJLBUILDDATE,
-        17                      /* sizeof min buffer == sizeof UEL */
+        PJLBUILDDATE
     };
     return &pjl_characteristics;
 }
@@ -65,7 +64,7 @@ pjl_impl_allocate_interp_instance(pl_interp_implementation_t *impl,
     pjl_parser_state *pjls = pjl_process_init(mem);
 
     impl->interp_client_data = pjls;
-    
+
     if (pjls == NULL)
         return gs_error_VMerror;
 
@@ -82,8 +81,7 @@ pjl_impl_init_job(pl_interp_implementation_t *impl,       /* interp instance to
     if (pjls == NULL)
         return gs_error_VMerror;
     /* copy the default state to the initial state */
-    pjl_set_init_from_defaults(pjls);
-    return 0;
+    return pjl_set_init_from_defaults(pjls);
 }
 
 static int
