@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -1074,7 +1074,9 @@ pl_tt_build_char(gs_show_enum * penum, gs_gstate * pgs, gs_font * pfont,
         gs_make_scaling(scale, scale, &smat);
         sbox.p.x = w2[2], sbox.p.y = w2[3];
         sbox.q.x = w2[4], sbox.q.y = w2[5];
-        gs_bbox_transform(&sbox, &smat, &sbox);
+        code = gs_bbox_transform(&sbox, &smat, &sbox);
+        if (code < 0)
+            return code;
         ipx = (int)sbox.p.x, ipy = (int)sbox.p.y;
         iqx = (int)ceil(sbox.q.x), iqy = (int)ceil(sbox.q.y);
         /* Set up the memory device for the bitmap.  NB should check code. */
@@ -1118,7 +1120,7 @@ pl_tt_build_char(gs_show_enum * penum, gs_gstate * pgs, gs_font * pfont,
                 return code;
         }
         /* Make sure we clear the entire bitmap. */
-        memset(pmdev->base, 0, bitmap_raster(pmdev->width) * pmdev->height);
+        memset(pmdev->base, 0, (size_t)bitmap_raster(pmdev->width) * pmdev->height);
         code = gx_set_device_color_1(pgs);     /* write 1's */
         if (code < 0)
             return code;
