@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -406,7 +406,7 @@ pl_glyph_name(gs_font * pfont, gs_glyph glyph, gs_const_string * pstr)
         }
         /* skip over the post header */
         numGlyphs = u16(postp + 32);
-        if (glyph < 0 || glyph > numGlyphs - 1) {
+        if (glyph > numGlyphs - 1) {
             if_debug1m('=', pfont->memory,
                        "[=]glyph index %lx out of range\n", glyph);
             return -1;
@@ -414,7 +414,7 @@ pl_glyph_name(gs_font * pfont, gs_glyph glyph, gs_const_string * pstr)
         /* glyph name index starts at post + 34 each entry is 2 bytes */
         glyph_name_index = u16(postp + 34 + (glyph * 2));
         /* this shouldn't happen */
-        if (glyph_name_index < 0 && glyph_name_index > 0x7fff)
+        if (glyph_name_index > 0x7fff)
             return -1;
         /* mac easy */
         if (glyph_name_index < 258) {
@@ -465,11 +465,14 @@ pl_glyph_name(gs_font * pfont, gs_glyph glyph, gs_const_string * pstr)
 static int
 pl_decode_glyph(gs_font * font, gs_glyph glyph, int ch, ushort *unicode_return, unsigned int length)
 {
-    pl_font_t *plfont = (pl_font_t *) font->client_data;
+
+    if (ch < 0 || ch > 255)
+        return (int) GS_NO_CHAR;
 
     if (length == 0)
         return 1;
-    *unicode_return = plfont->last_char;
+
+    *unicode_return = (ushort)ch;
     return 1;
 }
 
@@ -488,7 +491,7 @@ pl_font_glyph_width_cache_node_add(pl_font_t *plfont,
     if (plfont->widths_cache_nitems > PL_MAX_WIDTHS_CACHE_NITEMS) {
         pl_font_glyph_width_cache_remove_nodes(plfont);
     }
-    
+
     node = (pl_glyph_width_node_t *) gs_alloc_bytes(plfont->pfont->memory,
                                                  sizeof
                                                  (pl_glyph_width_node_t),
@@ -559,7 +562,7 @@ pl_font_char_width(const pl_font_t * plfont, const void *pgs,
     if (pl_font_glyph_width_cache_node_search(plfont, char_code, pwidth) >= 0) {
         return(code);
     }
-    
+
     if ((code = (*(plfont)->char_width) (plfont, pgs, char_code, pwidth)) == 0) {
 
         /* at least here, ignore the return value - if we fail to add a node
@@ -1082,11 +1085,11 @@ int
 pl_load_tt_font(stream * in, gs_font_dir * pdir, gs_memory_t * mem,
                 long unique_id, pl_font_t ** pplfont, char *font_name)
 {
-    byte *tt_font_datap;
+    byte *tt_font_datap = NULL;
     ulong size;
     int code;
-    gs_font_type42 *pfont;
-    pl_font_t *plfont;
+    gs_font_type42 *pfont = NULL;
+    pl_font_t *plfont = NULL;
     byte *file_name = NULL;
     gs_const_string pfname;
 
@@ -1094,7 +1097,8 @@ pl_load_tt_font(stream * in, gs_font_dir * pdir, gs_memory_t * mem,
         file_name =
             gs_alloc_bytes(mem, pfname.size + 1, "pl_load_tt_font file_name");
         if (!file_name) {
-            return (gs_error_VMerror);
+            sfclose(in);
+            return_error(gs_error_VMerror);
         }
         /* the stream code guarantees the string is null terminated */
         memcpy(file_name, pfname.data, pfname.size + 1);
@@ -1103,49 +1107,58 @@ pl_load_tt_font(stream * in, gs_font_dir * pdir, gs_memory_t * mem,
     /* get the data from the file */
     code = pl_alloc_tt_fontfile_buffer(in, mem, &tt_font_datap, &size);
     if (code < 0)
-        return_error(gs_error_VMerror);
+        goto error;
     /* Make a Type 42 font out of the TrueType data. */
     pfont = gs_alloc_struct(mem, gs_font_type42, &st_gs_font_type42,
                             "pl_tt_load_font(gs_font_type42)");
+    if (pfont == NULL) {
+        code = gs_error_VMerror;
+        goto error;
+    }
+    memset(pfont, 0, sizeof(*pfont));
     plfont = pl_alloc_font(mem, "pl_tt_load_font(pl_font_t)");
-
-    if (pfont == 0 || plfont == 0)
-        code = gs_note_error(gs_error_VMerror);
-    else {                      /* Initialize general font boilerplate. */
-        code =
-            pl_fill_in_font((gs_font *) pfont, plfont, pdir, mem, font_name);
-        if (code >= 0) {        /* Initialize TrueType font boilerplate. */
-            plfont->header = tt_font_datap;
-            plfont->header_size = size;
-            plfont->scaling_technology = plfst_TrueType;
-            plfont->font_type = plft_Unicode;
-            plfont->large_sizes = true;
-            plfont->offsets.GT = 0;
-            plfont->is_xl_format = false;
-            pl_fill_in_tt_font(pfont, tt_font_datap, unique_id);
-            code = gs_definefont(pdir, (gs_font *) pfont);
-        }
+    if (plfont == NULL) {
+        code = gs_error_VMerror;
+        goto error;
     }
 
-    if (code < 0) {
-        gs_free_object(mem, plfont, "pl_tt_load_font(pl_font_t)");
-        gs_free_object(mem, pfont, "pl_tt_load_font(gs_font_type42)");
-        pl_free_tt_fontfile_buffer(mem, tt_font_datap);
-        return code;
-    }
+    /* Initialize general font boilerplate. */
+    code = pl_fill_in_font((gs_font *) pfont, plfont, pdir, mem, font_name);
+    if (code < 0)
+        goto error;
+
+    /* Initialize TrueType font boilerplate. */
+    plfont->header = tt_font_datap;
+    plfont->header_size = size;
+    plfont->scaling_technology = plfst_TrueType;
+    plfont->font_type = plft_Unicode;
+    plfont->large_sizes = true;
+    plfont->offsets.GT = 0;
+    plfont->is_xl_format = false;
+    code = pl_fill_in_tt_font(pfont, tt_font_datap, unique_id);
+    if (code < 0)
+        goto error;
+    code = gs_definefont(pdir, (gs_font *) pfont);
+    if (code < 0)
+        goto error;
 
     code =
         pl_fapi_passfont(plfont, 0, NULL, NULL, plfont->header + 6,
                          plfont->header_size - 6);
-    if (file_name) {
+    if (code < 0)
+        goto error;
+    if (file_name)
         gs_free_object(mem, file_name, "pl_load_tt_font file_name");
-    }
-    if (code < 0) {
-        return (code);
-    }
 
     *pplfont = plfont;
     return 0;
+
+error:
+    gs_free_object(mem, plfont, "pl_tt_load_font(pl_font_t)");
+    gs_free_object(mem, pfont, "pl_tt_load_font(gs_font_type42)");
+    pl_free_tt_fontfile_buffer(mem, tt_font_datap);
+    gs_free_object(mem, file_name, "pl_load_tt_font file_name");
+    return_error(code);
 }
 
 /* load resident font data to ram */
@@ -1157,7 +1170,7 @@ pl_load_resident_font_data_from_file(gs_memory_t * mem, pl_font_t * plfont)
     byte *data;
 
     if (plfont->font_file && !plfont->font_file_loaded) {
-        stream *in = sfopen(plfont->font_file, gp_fmode_rb, mem);
+        stream *in = sfopen(plfont->font_file, "r", mem);
 
         if (in == NULL)
             return -1;
