@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -14,7 +14,12 @@
 */
 
 
+#ifndef plparams_h_INCLUDED
+#define plparams_h_INCLUDED
+
 /* plparams.h */
 
 int pcl_pjl_pdfmark(gs_memory_t *mem, gx_device *device, char *pdfmark);
 int pcl_pjl_setdistillerparams(gs_memory_t *mem, gx_device *device, char *distillerparams);
+
+#endif
