@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -196,7 +196,7 @@ px_gstate_client_copy_for(void *to, void *from, gs_gstate_copy_reason_t reason)
 #undef pxto
 }
 static void
-px_gstate_client_free(void *old, gs_memory_t * mem)
+px_gstate_client_free(void *old, gs_memory_t * mem, gs_gstate *pgs)
 {
     px_gstate_t *pxgs = old;
 
