@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -79,7 +79,7 @@ add_lines(px_args_t * par, px_state_t * pxs,
         pxeDataType_t type = (pxeDataType_t) par->pv[2]->value.i;
         int point_size = (type == eUByte || type == eSByte ? 2 : 4);
 
-        while (par->source.position < num_points * point_size) {
+        while (par->source.position < (ulong)num_points * point_size) {
             const byte *dp = par->source.data;
             int px, py;
 
@@ -151,7 +151,7 @@ add_curves(px_args_t * par, px_state_t * pxs,
 
         if (num_points % 3)
             return_error(errorIllegalDataLength);
-        while (par->source.position < num_points * point_size) {
+        while (par->source.position < (ulong)num_points * point_size) {
             const byte *dp = par->source.data;
             int points[6];
             int i;
@@ -295,8 +295,10 @@ setup_arc(px_arc_params_t * params, const px_value_t * pbox,
         if ((dx3 == 0 && dy3 == 0) || (dx4 == 0 && dy4 == 0))
             return_error(errorIllegalAttributeValue);
         {
-            double ang3 = atan2(dy3 * xr, dx3 * yr) * radians_to_degrees;
-            double ang4 = atan2(dy4 * xr, dx4 * yr) * radians_to_degrees;
+            double ang3 = atan2((double)(dy3 * xr),
+                                (double)(dx3 * yr)) * radians_to_degrees;
+            double ang4 = atan2((double)(dy4 * xr),
+                                (double)(dx4 * yr)) * radians_to_degrees;
 
             if (rotated)
                 ang3 += 180, ang4 += 180;
