@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -21,21 +21,21 @@
 #  define pxpthr_INCLUDED
 
 /* set passthrough contiguous mode */
-void pxpcl_passthroughcontiguous(bool contiguous);
+void pxpcl_passthroughcontiguous(px_state_t * pxs, bool contiguous);
 
 /* end passthrough contiguous mode */
 void pxpcl_endpassthroughcontiguous(px_state_t * pxs);
 
 /* reset pcl's page */
-void pxpcl_pagestatereset(void);
+void pxpcl_pagestatereset(px_state_t* pxs);
 
 /* release the passthrough state */
-void pxpcl_release(void);
+void pxpcl_release(px_state_t* pxs);
 
 /* set variables in pcl's state that are special to pass through mode,
    these override the default pcl state variables when pcl is
    entered. */
-void pxPassthrough_pcl_state_nonpage_exceptions(px_state_t * pxs);
+int pxPassthrough_pcl_state_nonpage_exceptions(px_state_t * pxs);
 
 int pxpcl_selectfont(px_args_t * par, px_state_t * pxs);
 
