@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -44,6 +44,7 @@
 #include "pltop.h"
 #include "plmain.h"
 #include "gsicc_manage.h"
+#include "gxdevsop.h"
 
 /* Imported operators */
 px_operator_proc(pxEndPage);
@@ -128,28 +129,38 @@ typedef struct pxl_interp_instance_s
     gs_memory_t *memory;        /* memory allocator to use */
     px_parser_state_t *st;      /* parser state */
     px_state_t *pxs;            /* interp state */
-    gs_gstate *pgs;              /* grafix state */
-    enum
-    { PSHeader, PSXL, PSDone }
-    processState;               /* interp's processing state */
-         px_stream_header_process_t headerState;        /* used to decode stream header */
+    gs_gstate *pgs;             /* graphics state */
+    enum {
+        PSHeader,
+        PSXL,
+        PSDone
+    } processState;             /* interp's processing state */
+    px_stream_header_process_t headerState;        /* used to decode stream header */
 } pxl_interp_instance_t;
 
-/* Get implemtation's characteristics */
+static int
+pxl_detect_language(const char *s, int len)
+{
+   if (len < 11)
+       return 0;
+   if (memcmp(s, ") HP-PCL XL", 11) == 0)
+       return 100;
+   return 0;
+}
+
+/* Get implementation's characteristics */
 static const pl_interp_characteristics_t *      /* always returns a descriptor */
-pxl_impl_characteristics(const pl_interp_implementation_t * impl        /* implementation of interpereter to alloc */
-    )
+pxl_impl_characteristics(const pl_interp_implementation_t * impl)        /* implementation of interpreter to alloc */
 {
     /* version and build date are not currently used */
 #define PXLVERSION NULL
 #define PXLBUILDDATE NULL
     static pl_interp_characteristics_t pxl_characteristics = {
         "PCLXL",
-        ") HP-PCL XL",
+        pxl_detect_language,
         "Artifex",
         PXLVERSION,
-        PXLBUILDDATE,
-        px_parser_min_input_size
+        PXLBUILDDATE
     };
     return &pxl_characteristics;
 }
@@ -198,7 +209,7 @@ pxl_impl_allocate_interp_instance(pl_interp_implementation_t *impl,
     pxs->pjls = pl_main_get_pjl_instance(mem);
 
     /* The PCL instance is needed for PassThrough mode */
-    pxs->pcls =pl_main_get_pcl_instance(mem);
+    pxs->pcls = pl_main_get_pcl_instance(mem);
 
     /* Return success */
     impl->interp_client_data = pxli;
@@ -212,23 +223,50 @@ pxl_set_icc_params(pl_interp_implementation_t * impl, gs_gstate * pgs)
     return pl_set_icc_params(pxli->memory, pgs);
 }
 
-/* Set a device into an interperter instance */
-/* ret 0 ok, else -ve error code */
-static int 
-pxl_impl_set_device(pl_interp_implementation_t * impl,
-                    gx_device * device)
+/*
+ * Set a Boolean parameter.
+ */
+static int
+put_param_bool(pxl_interp_instance_t *pxli, gs_param_name pkey, bool value)
 {
+    gs_c_param_list list;
     int code;
+
+    gs_c_param_list_write(&list, pxli->memory);
+    code = param_write_bool((gs_param_list *) & list, pkey, &value);
+    if (code >= 0) {
+        gs_c_param_list_read(&list);
+
+        code = gs_gstate_putdeviceparams(pxli->pgs,
+                                         gs_currentdevice(pxli->pgs),
+                                         (gs_param_list *) &list);
+    }
+
+    gs_c_param_list_release(&list);
+    return code;
+}
+
+/* Prepare interp instance for the next "job" */
+/* ret 0 ok, else -ve error code */
+static int
+pxl_impl_init_job(pl_interp_implementation_t * impl,
+                  gx_device                  * device)
+{
+    int code = 0;
     pxl_interp_instance_t *pxli = impl->interp_client_data;
     px_state_t *pxs = pxli->pxs;
     gs_memory_t *mem = pxli->memory;
-    
+
     enum { Sbegin, Ssetdevice, Sinitg, Sgsave, Serase, Sdone } stage;
 
-    stage = Sbegin;
+    px_reset_errors(pxli->pxs);
+    px_process_init(pxli->st, true);
 
-    if ((code = gs_opendevice(device)) < 0)
-        goto pisdEnd;
+    /* set input status to: expecting stream header */
+    px_stream_header_init(&pxli->headerState, pxli->st, pxli->pxs);
+    pxli->processState = PSHeader;
+
+    stage = Sbegin;
 
     pxs->interpolate = pl_main_get_interpolate(mem);
     pxs->nocache = pl_main_get_nocache(mem);
@@ -289,25 +327,23 @@ pxl_impl_set_device(pl_interp_implementation_t * impl,
         case Sbegin:           /* nothing left to undo */
             break;
     }
+
+    /* Warn the device that PXL uses ROPs. */
+    if (code == 0) {
+        code = put_param_bool(pxli, "LanguageUsesROPs", true);
+
+        if (!device->is_open)
+            code = gs_opendevice(device);
+    }
+
     return code;
 }
 
-/* Prepare interp instance for the next "job" */
-/* ret 0 ok, else -ve error code */
+/* Do any setup for parser per-cursor */
 static int
-pxl_impl_init_job(pl_interp_implementation_t * impl)
+pxl_impl_process_begin(pl_interp_implementation_t * impl)
 {
-    int code = 0;
-    pxl_interp_instance_t *pxli = impl->interp_client_data;
-
-    px_reset_errors(pxli->pxs);
-    px_process_init(pxli->st, true);
-
-    /* set input status to: expecting stream header */
-    px_stream_header_init(&pxli->headerState, pxli->st, pxli->pxs);
-    pxli->processState = PSHeader;
-
-    return code;
+    return 0;
 }
 
 /* Parse a cursor-full of data */
@@ -366,6 +402,13 @@ pxl_impl_process(pl_interp_implementation_t * impl,
     return code;
 }
 
+static int
+pxl_impl_process_end(pl_interp_implementation_t * impl)
+{
+    return 0;
+}
+
+
 /* Skip to end of job ret 1 if done, 0 ok but EOJ not found, else -ve error code */
 static int
 pxl_impl_flush_to_eoj(pl_interp_implementation_t * impl,
@@ -398,6 +441,8 @@ pxl_impl_process_eof(pl_interp_implementation_t * impl)
     pxl_interp_instance_t *pxli = impl->interp_client_data;
 
     px_state_cleanup(pxli->pxs);
+    px_process_init(pxli->st, true);
+
     return 0;
 }
 
@@ -416,12 +461,16 @@ pxl_impl_report_errors(pl_interp_implementation_t * impl,
     const char *subsystem = (code <= px_error_next ? "KERNEL" : "GRAPHICS");
     char message[px_max_error_line + 1];
     int N = 0;
-    int y;
+    int y = 0;
 
     if (code >= 0)
         return code;            /* not really an error */
-    if (report & eErrorPage)
-        y = px_begin_error_page(pxs);
+    if (report & eErrorPage) {
+        int ecode = px_begin_error_page(pxs, &y);
+        if (ecode < 0) {
+            return ecode;
+        }
+    }
     while ((N = px_error_message_line(message, N, subsystem,
                                       code, st, pxs)) >= 0) {
         if ((report & eBackChannel) || force_to_cout)
@@ -449,22 +498,25 @@ pxl_impl_report_errors(pl_interp_implementation_t * impl,
 static int
 pxl_impl_dnit_job(pl_interp_implementation_t * impl)
 {
+    int code;
     pxl_interp_instance_t *pxli = impl->interp_client_data;
+    gx_device *device = gs_currentdevice(pxli->pgs);
 
     px_stream_header_dnit(&pxli->headerState);
     px_state_cleanup(pxli->pxs);
-    return 0;
-}
-
-/* Remove a device from an interperter instance */
-/* ret 0 ok, else -ve error code */
-static int
-pxl_impl_remove_device(pl_interp_implementation_t * impl)
-{
-    pxl_interp_instance_t *pxli = impl->interp_client_data;
-
+    px_process_init(pxli->st, true);
     /* return to original gstate  */
-    return gs_grestore_only(pxli->pgs);        /* destroys gs_save stack */
+    code = gs_grestore_only(pxli->pgs);        /* destroys gs_save stack */
+
+    /* Warn the device that ROP usage has come to an end */
+    if (code >= 0) {
+        code = put_param_bool(pxli, "LanguageUsesROPs", false);
+
+        if (!device->is_open)
+            code = gs_opendevice(device);
+    }
+
+    return code;
 }
 
 /* Deallocate a interpreter instance */
@@ -502,15 +554,20 @@ pxl_end_page_top(px_state_t * pxls, int num_copies, int flush)
 pl_interp_implementation_t pxl_implementation = {
     pxl_impl_characteristics,
     pxl_impl_allocate_interp_instance,
-    pxl_impl_set_device,
+    NULL,                       /* get_device_memory */
+    NULL,                       /* set_param */
+    NULL,                       /* add_path */
+    NULL,                       /* post_args_init */
     pxl_impl_init_job,
+    NULL,                       /* run_prefix_commands */
     NULL,                       /* process_file */
+    pxl_impl_process_begin,
     pxl_impl_process,
+    pxl_impl_process_end,
     pxl_impl_flush_to_eoj,
     pxl_impl_process_eof,
     pxl_impl_report_errors,
     pxl_impl_dnit_job,
-    pxl_impl_remove_device,
     pxl_impl_deallocate_interp_instance,
     NULL
 };
