@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# Portions Copyright (C) 2001-2019 Artifex Software Inc.
+# Portions Copyright (C) 2001-2020 Artifex Software Inc.
 #   
 #   This software is distributed under license and may not be copied, modified
 #   or distributed except as expressly authorized under the terms of that
@@ -118,7 +118,6 @@ pxl_tags_dict = {
     'SetHalftoneMethod' :       0x6d,
     'SetFillMode' :             0x6e,
     'SetFont' :                 0x6f,
-    'SetHalftoneMethod' :       0x6d,
     'SetLineCap' :              0x71,
     'SetLineDash' :             0x70,
     'SetLineJoin' :             0x72,
@@ -195,7 +194,7 @@ pxl_enumerations_dict = {
                       'eEnvelopeTray=6', 'eThirdCassette=7', 'External Trays=8-255' ],
     'MediaDestination' :  [ 'eDefaultDestination=0', 'eFaceDownBin=1', 'eFaceUpBin=2',
                             'eJobOffsetBin=3', 'External Bins=5-255' ],
-    'LineCapStyle' : [ 'eButtCap=0' 'eRoundCap=1', 'eSquareCap=2', 'eTriangleCap=3' ],
+    'LineCapStyle' : [ 'eButtCap=0', 'eRoundCap=1', 'eSquareCap=2', 'eTriangleCap=3' ],
     'LineJoin' : [ 'eMiterJoin=0', 'eRoundJoin=1', 'eBevelJoin=2', 'eNoJoin=3' ],
     'Measure' : [ 'eInch=0', 'eMillimeter=1', 'eTenthsOfAMillimeter=2' ],
     'MediaSize' : [ 'eDefault = 96', 'eLetterPaper=0', 'eLegalPaper=1', 'eA4Paper=2',
@@ -374,9 +373,6 @@ class pxl_asm:
         # output is always little endian.
         self.assembled_binding = '<'
         
-    def nullAttributeList(self):
-        return 0
-
     # does not consume the string
     def next_string(self):
         index = self.index
