@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -914,9 +914,9 @@ WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
          * initializes it here.
          */
         tw = (TW *)(((CREATESTRUCT FAR *)lParam)->lpCreateParams);
-        SetWindowLong(hwnd, 0, (LONG)tw);
+        SetWindowLongPtr(hwnd, 0, (LONG_PTR)tw);
     }
-    tw = (TW *)GetWindowLong(hwnd, 0);
+    tw = (TW *)GetWindowLongPtr(hwnd, 0);
 
     switch(message) {
         case WM_SYSCOMMAND:
