@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -357,6 +357,7 @@ zdeletefile(i_ctx_t *i_ctx_p)
             return code;
         }
     }
+
     code = (*pname.iodev->procs.delete_file)(pname.iodev, pname.fname);
 
     if (code >= 0 && is_temp)
@@ -733,7 +734,8 @@ ztempfile(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
     const char *pstr;
     char fmode[4];
-    int code = parse_file_access_string(op, fmode);
+    char fmode_temp[4];
+    int code = parse_file_access_string(op, fmode_temp);
     char *prefix = NULL;
     char *fname= NULL;
     uint fnlen;
@@ -750,7 +752,7 @@ ztempfile(i_ctx_t *i_ctx_p)
         goto done;
     }
 
-    strcat(fmode, gp_fmode_binary_suffix);
+    snprintf(fmode, sizeof(fmode), "%s%s", fmode_temp, gp_fmode_binary_suffix);
     if (r_has_type(op - 1, t_null))
         pstr = gp_scratch_file_name_prefix;
     else {
