@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -25,7 +25,7 @@
  * This routine translates a gs_separation_name value into a character string
  * pointer and a string length.
  */
-int gs_get_colorname_string(const gs_memory_t *mem,
+int gs_get_colorname_string(gs_gstate *pgs,
                             gs_separation_name colorname_index,
                             unsigned char **ppstr,
                             unsigned int *pname_size);
