@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -36,7 +36,9 @@ const char iodev_dtype_stdio[] = "Special";
           iodev_no_delete_file, iodev_no_rename_file, iodev_no_file_status,\
           iodev_no_enumerate_files, NULL, NULL,\
           iodev_no_get_params, iodev_no_put_params\
-        }\
+        }, \
+        NULL, \
+        NULL \
 }
 
 /*
@@ -74,21 +76,17 @@ const gx_io_device gs_iodev_stderr =
  * call to file_close_disable.
  */
 
-static int
-    s_stdin_read_process(stream_state *, stream_cursor_read *,
-                         stream_cursor_write *, bool);
-
 static int
 stdin_init(gx_io_device * iodev, gs_memory_t * mem)
 {
-    mem->gs_lib_ctx->stdin_is_interactive = true;
+    mem->gs_lib_ctx->core->stdin_is_interactive = true;
     return 0;
 }
 
 static void
 stdin_finit(gx_io_device * iodev, gs_memory_t * mem)
 {
-    mem->gs_lib_ctx->stdin_is_interactive = false;
+    mem->gs_lib_ctx->core->stdin_is_interactive = false;
     return;
 }
 
@@ -101,19 +99,20 @@ s_stdin_read_process(stream_state * st, stream_cursor_read * ignore_pr,
     int wcount = (int)(pw->limit - pw->ptr);
     int count;
     gs_memory_t *mem = st->memory;
+    gs_lib_ctx_core_t *core = mem->gs_lib_ctx->core;
 
     if (wcount <= 0)
         return 0;
 
     /* do the callout */
-    if (mem->gs_lib_ctx->stdin_fn)
-        count = (*mem->gs_lib_ctx->stdin_fn)
-            (mem->gs_lib_ctx->caller_handle, (char *)pw->ptr + 1,
-             mem->gs_lib_ctx->stdin_is_interactive ? 1 : wcount);
+    if (core->stdin_fn)
+        count = (*core->stdin_fn)
+            (core->std_caller_handle, (char *)pw->ptr + 1,
+             core->stdin_is_interactive ? 1 : wcount);
     else
         count = gp_stdin_read((char *)pw->ptr + 1, wcount,
-                      mem->gs_lib_ctx->stdin_is_interactive,
-                      mem->gs_lib_ctx->fstdin);
+                      core->stdin_is_interactive,
+                      core->fstdin);
 
     pw->ptr += (count < 0) ? 0 : count;
     return ((count < 0) ? ERRC : (count == 0) ? EOFC : count);
@@ -197,7 +196,7 @@ s_stdout_write_process(stream_state * st, stream_cursor_read *pr,
     if (count == 0)
         return 0;
     written = outwrite(st->memory, (const char *)pr->ptr + 1, count);
-    if (written < count)
+    if (written != count)
         return ERRC;
     pr->ptr += written;
     return 0;
