@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -14,6 +14,9 @@
 */
 
 
+#ifndef ghostxps_h_INCLUDED
+#define ghostxps_h_INCLUDED
+
 /* combined internal header for the XPS interpreter */
 
 #include "memory_.h"
@@ -457,3 +460,5 @@ int xps_fclose(gp_file *file);
 #define xps_fread gp_fread
 #define xps_fclose gp_fclose
 #endif
+
+#endif
