@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -28,7 +28,8 @@ xps_set_color(xps_context_t *ctx, gs_color_space *cs, float *samples)
 
     if (ctx->opacity_only)
     {
-        gs_setopacityalpha(ctx->pgs, 1.0);
+        gs_setfillconstantalpha(ctx->pgs, 1.0);
+        gs_setstrokeconstantalpha(ctx->pgs, 1.0);
         gs_setgray(ctx->pgs, samples[0]);
     }
     else
@@ -38,7 +39,8 @@ xps_set_color(xps_context_t *ctx, gs_color_space *cs, float *samples)
         for (i = 0; i < n; i++)
             cc.paint.values[i] = samples[i + 1];
 
-        gs_setopacityalpha(ctx->pgs, samples[0]);
+        gs_setfillconstantalpha(ctx->pgs, samples[0]);
+        gs_setstrokeconstantalpha(ctx->pgs, samples[0]);
         gs_setcolorspace(ctx->pgs, cs);
         gs_setcolor(ctx->pgs, &cc);
     }
