@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -13,9 +13,14 @@
 
 /* Font API support  */
 
+#ifndef xpsfapi_h_INCLUDED
+#define xpsfapi_h_INCLUDED
+
 int
 xps_fapi_rebuildfont(gs_font *pfont);
 
 int
 xps_fapi_passfont(gs_font *pfont, char *fapi_request, char *file_name,
                   byte *font_data, int font_data_len);
+
+#endif
