@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -214,7 +214,8 @@ xps_hash_debug(xps_hash_table_t *table)
         if (!table->entries[i].value)
             dlprintf1("table % 4d: empty\n", i);
         else
-            dlprintf3("table % 4d: key=%s value=%p\n", i,
-                      table->entries[i].key, table->entries[i].value);
+            dlprintf3("table % 4d: key=%s value="PRI_INTPTR"\n", i,
+                      table->entries[i].key,
+                      (intptr_t)table->entries[i].value);
     }
 }
