@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -65,7 +65,7 @@ xps_clip(xps_context_t *ctx)
 void
 xps_fill(xps_context_t *ctx)
 {
-    if (gs_currentopacityalpha(ctx->pgs) < 0.001)
+    if (gs_getfillconstantalpha(ctx->pgs) < 0.001)
         gs_newpath(ctx->pgs);
     else if (ctx->fill_rule == 0) {
         if (gs_eofill(ctx->pgs) == gs_error_Remap_Color){
