@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -208,7 +208,8 @@ xps_debug_resource_dictionary(xps_context_t *ctx, xps_resource_t *dict)
     {
         if (dict->base_uri)
             dmprintf1(ctx->memory, "URI = '%s'\n", dict->base_uri);
-        dmprintf2(ctx->memory, "KEY = '%s' VAL = %p\n", dict->name, dict->data);
+        dmprintf2(ctx->memory, "KEY = '%s' VAL = "PRI_INTPTR"\n",
+                  dict->name, (intptr_t)dict->data);
         if (dict->parent)
         {
             dmputs(ctx->memory, "PARENT = {\n");
