@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -62,8 +62,7 @@ xps_paint_tiling_brush_clipped(struct tile_closure_s *c)
 static int
 xps_paint_tiling_brush(const gs_client_color *pcc, gs_gstate *pgs)
 {
-    const gs_client_pattern *ppat = gs_getpattern(pcc);
-    struct tile_closure_s *c = ppat->client_data;
+    struct tile_closure_s *c = pcc->pattern->client_data;
     xps_context_t *ctx = c->ctx;
     gs_gstate *saved_pgs;
     int code;
@@ -200,7 +199,7 @@ static int
 xps_remap_pattern(const gs_client_color *pcc, gs_gstate *pgs)
 {
     gs_client_pattern *ppat = (gs_client_pattern *)gs_getpattern(pcc);
-    struct tile_closure_s *c = ppat->client_data;
+    struct tile_closure_s *c = pcc->pattern->client_data;
     xps_context_t *ctx = c->ctx;
     int code;
 
@@ -327,6 +326,7 @@ xps_parse_tiling_brush(xps_context_t *ctx, char *base_uri, xps_resource_t *dict,
         gs_client_color gscolor;
         gs_color_space *cs;
         bool sa;
+        float opacity;
 
         closure.ctx = ctx;
         closure.base_uri = base_uri;
@@ -346,7 +346,6 @@ xps_parse_tiling_brush(xps_context_t *ctx, char *base_uri, xps_resource_t *dict,
         gspat.PaintType = 1;
         gspat.TilingType = 2;
         gspat.PaintProc = xps_remap_pattern;
-        gspat.client_data = &closure;
 
         /* We need to know if this tiling brush includes transparency.
            We could do a proper scan, but for now we'll be lazy and just look
@@ -383,8 +382,18 @@ xps_parse_tiling_brush(xps_context_t *ctx, char *base_uri, xps_resource_t *dict,
         sa = gs_currentstrokeadjust(ctx->pgs);
         gs_setstrokeadjust(ctx->pgs, false);
         gs_makepattern(&gscolor, &gspat, &transform, ctx->pgs, NULL);
+        gscolor.pattern->client_data = &closure;
         gs_setpattern(ctx->pgs, &gscolor);
+        /* If the tiling brush has an opacity, it was already set in the group
+           that we are filling.  Reset to 1.0 here to avoid double application
+           when the tiling actually occurs */
+        opacity = gs_getfillconstantalpha(ctx->pgs);
+        gs_setfillconstantalpha(ctx->pgs, 1.0);
+        gs_setstrokeconstantalpha(ctx->pgs, 1.0);
         xps_fill(ctx);
+        gs_setfillconstantalpha(ctx->pgs, opacity);
+        gs_setstrokeconstantalpha(ctx->pgs, opacity);
+
         gs_setstrokeadjust(ctx->pgs, sa);
         gsicc_adjust_profile_rc(cs->cmm_icc_profile_data, -1, "xps_parse_tiling_brush");
 
