@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2020 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -48,7 +48,7 @@ static inline int getlong(gp_file *file)
 static void *
 xps_zip_alloc_items(xps_context_t *ctx, int items, int size)
 {
-    void *item = xps_alloc(ctx, items * size);
+    void *item = xps_alloc(ctx, (size_t)items * size);
     if (!item) {
         gs_throw(gs_error_VMerror, "out of memory: item.\n");
         return NULL;
